import bpy
from ..icons.custom_icons import get_icon
from ..utils.utils import (
    refresh,
    get_fcurve_nice_name,
    fcurves_active_object,
    has_animation,
    get_selected_keys,
)
from bpy.types import Menu
from bpy.utils import (
    register_class,
    unregister_class
)

class ANIMCOPY_MT_animcopy_menu(Menu):
    bl_label = "AnimCopy"

    def draw(self, context):
        layout = self.layout

        # Check if context.object exists
        if context.object is None:
            layout.label(text="No active object.", icon='ERROR')
            return

        # Check if the context.area exists and is of type 'GRAPH_EDITOR'
        if context.area and context.area.type == 'GRAPH_EDITOR':
            fcurves = fcurves_active_object(context, "SELECTED_FCURVES")
            if len(fcurves):
                cts = layout.operator("graph.copy_to_selected", text="Copy Animation to Selected", icon_value=get_icon("AnimCopy"))
                cts.selection_mode = "SELECTED_FCURVES"
            elif has_animation(context.object):
                cts = layout.operator("graph.copy_to_selected", text="Copy Animation to Selected", icon_value=get_icon("AnimCopy"))
                cts.selection_mode = "ALL"
            else:
                layout.label(text="Active object has no animation data.", icon='INFO')

            if context.selected_editable_fcurves:
                layout.separator()
                # Always show the "Copy F-Curve Data (all)" option
                copy_all_op = layout.operator("graph.copy_fcurves", text="Copy F-Curve Data (all)", icon_value=get_icon("Copy_Timing"))
                copy_all_op.use_selected_keys = False

                # Check if any selected editable F-Curves have selected keyframe points
                has_selected_keyframes = any(
                    len(get_selected_keys(fcurve)) > 0 for fcurve in context.selected_editable_fcurves
                )

                # If there are selected keyframe points, show the "Copy F-Curve (only selected keyframes)" option
                if has_selected_keyframes:
                    copy_selected_op = layout.operator("graph.copy_fcurves", text="Copy F-Curve Data (only selected keyframes)", icon_value=get_icon("Copy_Timing"))
                    copy_selected_op.use_selected_keys = True

                pt = layout.operator("graph.paste_timing", text="Paste Timing", icon_value=get_icon("Paste_Timing"))
                pt.handle_angles = False
                pt.handle_lengths = False
                pt.paste_values = False

                pte = layout.operator("graph.paste_timing", text="Paste Timing and Ease", icon_value=get_icon("Paste_Timing"))
                pte.handle_angles = True
                pte.handle_lengths = True
                pte.paste_values = False

                ptv = layout.operator("graph.paste_timing", text="Paste Timing and Values", icon_value=get_icon("Paste_Timing"))
                ptv.handle_angles = False
                ptv.handle_lengths = False
                ptv.paste_values = True

                pa = layout.operator("graph.paste_timing", text="Paste All", icon_value=get_icon("Paste_Timing"))
                pa.handle_angles = True
                pa.handle_lengths = True
                pa.paste_values = True

                layout.separator()
                layout.operator("graph.copy_fcurves", text="Copy Ease", icon_value=get_icon("Copy_Ease"))
                layout.operator("graph.paste_ease", text="Paste Ease", icon_value=get_icon("Paste_Ease"))
                layout.separator()
        else:
            # Handle cases where the area is not 'GRAPH_EDITOR'
            if context.object:
                anim_types = has_animation(context.object)
                
                if anim_types:                    
                    cts = layout.operator(
                        "graph.copy_to_selected",
                        text="Copy Animation to Selected",
                        icon='COPYDOWN'
                    )
                    cts.selection_mode = "ALL"
                else:
                    layout.label(text="Active object has no animation data.", icon='INFO')                    
            else:
                layout.label(text="No active object selected.", icon='ERROR')


classes = (ANIMCOPY_MT_animcopy_menu,)

def menu_func(self, context):
    layout = self.layout
    layout.menu("ANIMCOPY_MT_animcopy_menu")

def register():
    for cls in classes:
        register_class(cls)
    bpy.types.VIEW3D_MT_pose_context_menu.append(menu_func)
    bpy.types.VIEW3D_MT_object_context_menu.append(menu_func)
    bpy.types.DOPESHEET_MT_channel_context_menu.append(menu_func)
    bpy.types.GRAPH_MT_context_menu.append(menu_func)

def unregister():
    for cls in reversed(classes):
        unregister_class(cls)
    bpy.types.VIEW3D_MT_pose_context_menu.remove(menu_func)
    bpy.types.VIEW3D_MT_object_context_menu.remove(menu_func)
    bpy.types.DOPESHEET_MT_channel_context_menu.remove(menu_func)
    bpy.types.GRAPH_MT_context_menu.remove(menu_func)

if __name__ == "__main__":
    register()