bl_info = {
    "name": "Delete Static Channels",
    "description": "Delete F-Curve channels from Graph Editor based on user-specified options",
    "author": "Blastframe",
    "version": (1, 0, 21),
    "blender": (2, 80, 0),
    "location": "Graph Editor > Channel Menu",
    "doc_url": "https://blastframe.com/docs/delete-static-channels/",
    "tracker_url": "https://blastframe.com/contact/",
    "category": "Animation",
}

_need_reload = "delete_static_channels" in locals()

from .operators import (
    delete_static_channels
)

from .utils import (
    utils,
    check_updates,
)

from . import preferences

if _need_reload:
    import importlib
    utils = importlib.reload(utils)
    check_updates = importlib.reload(check_updates)
    preferences = importlib.reload(preferences)
    delete_static_channels = importlib.reload(delete_static_channels)

def register():
    from .icons.custom_icons import register_custom_icons
    register_custom_icons()
    preferences.register()
    check_updates.register()
    delete_static_channels.register()

def unregister():
    from .icons.custom_icons import unregister_custom_icons
    unregister_custom_icons()
    delete_static_channels.unregister()
    check_updates.unregister()
    preferences.unregister()
