import bpy
from bpy.props import (
    BoolProperty,
    StringProperty,
    IntProperty
)
from .icons.custom_icons import get_icon
from bpy.types import AddonPreferences
import rna_keymap_ui

addon_keymaps = []

def get_user_keymap_item(keymap_name, keymap_item_idname, multiple_entries=False):
    wm = bpy.context.window_manager
    kc = wm.keyconfigs.user

    km = kc.keymaps.get(keymap_name)
    if km:
        if multiple_entries:
            return km, [i[1] for i in km.keymap_items.items() if i[0] == keymap_item_idname]
        else:
            return km, km.keymap_items.get(keymap_item_idname)
    return None, None

def add_hotkey():
    addon_keymaps.clear()

    wm = bpy.context.window_manager
    kc = wm.keyconfigs.addon
    if kc:
        keymaps = [
            ('Graph Editor', 'GRAPH_EDITOR'),
            ('Dopesheet', 'DOPESHEET_EDITOR'),
            ('Animation Channels', 'EMPTY')
        ]

        for (km_name, space_type) in keymaps:
            km = kc.keymaps.new(name=km_name, space_type=space_type)
            kmi = km.keymap_items.new(
                "graph.delete_static_channels", type='D', value='PRESS', ctrl=False)
            addon_keymaps.append((km, kmi))

def remove_hotkey():   
    for km, kmi in addon_keymaps:
        if kmi:
            km.keymap_items.remove(kmi)  
    addon_keymaps.clear()

class DELETESTATICCHANNELS_preferences(AddonPreferences):
    bl_idname = __package__

    update: BoolProperty(name="Update", default=False)
    version: StringProperty(name="Version", default="")
    autocheck: BoolProperty(name="Auto-check for Update", default=True)
    debug_mode: BoolProperty(name="Debug Mode", default=False)
    weeks: IntProperty(
        min = 1,
        max = 52,
        soft_min = 1,
        soft_max = 52,
        name="Frequency (weeks)",
        default=2
    )

    def draw(self, context):
        layout = self.layout
        final_keymap_set = set()
        final_keymap = dict()

        col = layout.column()
        col.separator()
        col.label(text="Hotkeys", icon="EVENT_CTRL")
        box = col.box()    
        col = box.column()

        for km, kmi in addon_keymaps:
            user_km, user_kmis = get_user_keymap_item(km.name, kmi.idname, multiple_entries=True)
            if user_kmis:
                for user_kmi in user_kmis:
                    final_keymap_set.add((user_km, user_kmi))
                    if not user_km in final_keymap:
                        final_keymap[user_km] = []
                    final_keymap[user_km].append(user_kmi)
            else:
                final_keymap_set.add((user_km, None))
                if not user_km in final_keymap:
                    final_keymap[user_km] = []
                final_keymap[user_km].append(None)
        
        wm = bpy.context.window_manager
        kc = wm.keyconfigs.user

        for user_km, user_kmis in final_keymap.items():
            user_km = user_km.active()
            col.separator()
            col.label(text=user_km.name)
            col.context_pointer_set("keymap", user_km)
            unique_kmis = list(dict.fromkeys(user_kmis))
            for user_kmi in unique_kmis:
                if user_kmi:
                    rna_keymap_ui.draw_kmi(["ADDON", "USER", "DEFAULT"], kc, user_km, user_kmi, col, 0)

        col = layout.column()
        col.prop(self, "debug_mode")
        col.separator()
        col.prop(self, "autocheck")
        row = col.row()
        row.enabled = self.autocheck
        box = row.box()
        box.use_property_split = True
        box.use_property_decorate = False
        box.prop(self, "weeks")

        if bpy.app.version >= (4, 2, 0):
            product_name = __package__.split(".")[2]
        else:
            product_name = __package__.split(".")[0]

        if self.update:
            col.alert = True
            col.label(text="Update Available. %s. Please download from the store where you purchased this Add-on." % (str(self.version)))
        elif self.version != "":
            col.label(text="You have the latest version.")
        col.alert = False
        col.separator()
        row = col.row(align=True)
        row.scale_y = 1.3
        row.operator("wm.url_open", text="Blastframe", icon_value=get_icon("Blastframe_Logo_White")).url = "https://blastframe.com"
        row.operator("wm.url_open", text="YouTube", icon_value=get_icon("YouTube")).url = "https://www.youtube.com/@blastframe"
        row.operator("wm.url_open", text="Blender Market", icon_value=get_icon("Blender_Market")).url = "https://blendermarket.com/creators/blastframe"

def register():
    from bpy.utils import register_class, unregister_class
    try:
        unregister_class(DELETESTATICCHANNELS_preferences)
    except RuntimeError:
        pass
    register_class(DELETESTATICCHANNELS_preferences)
    add_hotkey()

def unregister():
    remove_hotkey()
    from bpy.utils import unregister_class
    try:
        unregister_class(DELETESTATICCHANNELS_preferences)
    except RuntimeError as e:
        print(f"Error unregistering {DELETESTATICCHANNELS_preferences}: {e}")