bl_info = {
    "name": "GraphKit",
    "description": "Tools for manipulating F-Curves and keyframes in the Graph Editor",
    "author": "Blastframe",
    "version": (2, 6, 9),
    "blender": (2, 80, 0),
    "location": "Graph Editor > Sidebar > GraphKit Panel, Graph Editor Menus, and Graph Editor Context Menu",
    "doc_url": "https://blastframe.com/docs/graphkit/",
    "tracker_url": "https://blastframe.com/contact/",
    "category": "Animation"
}

_need_reload = "amplify" in locals()

from . import preferences
from .preferences import update_toggles

from .operators import (
    align,
    amplify,
    hold,
    overlap,
    range_to_loop,
    perfect_loop,
    repeat,
    divide,
    distribute,
    fade,
    flatten_holds,
    refresh,
    randomize,
    select,
    tween,
    visibility
)

from .ui import (panel,
                 menu)

from .utils import (
    utils,
    check_updates,
)
from .utils.utils import get_prefs

from .data_classes import animation_data
from .properties import props

if _need_reload:
    import importlib
    utils = importlib.reload(utils)
    check_updates = importlib.reload(check_updates)
    preferences = importlib.reload(preferences)
    panel = importlib.reload(panel)
    menu = importlib.reload(menu)

    animation_data = importlib.reload(animation_data)
    tween = importlib.reload(tween)
    hold = importlib.reload(hold)
    range_to_loop = importlib.reload(range_to_loop)
    perfect_loop = importlib.reload(perfect_loop)
    repeat = importlib.reload(repeat)
    overlap = importlib.reload(overlap)
    divide = importlib.reload(divide)
    distribute = importlib.reload(distribute)
    fade = importlib.reload(fade)
    flatten_holds = importlib.reload(flatten_holds)
    align = importlib.reload(align)
    amplify = importlib.reload(amplify)
    refresh = importlib.reload(refresh)
    randomize = importlib.reload(randomize)
    select = importlib.reload(select)
    visibility = importlib.reload(visibility)
    props = importlib.reload(props)

from . import preferences

modules = [preferences, check_updates, tween, range_to_loop,
    perfect_loop, repeat, overlap, fade, flatten_holds, distribute,
    divide, hold, align, amplify, randomize, refresh, select, visibility, menu,
    panel]

def register():
    from .icons.custom_icons import register_custom_icons
    register_custom_icons()
    for mod in modules:
        mod.register()
    update_toggles(get_prefs())


def unregister():
    for mod in reversed(modules):
        mod.unregister()

    from .icons.custom_icons import unregister_custom_icons
    unregister_custom_icons()