import bpy
from ..utils.utils import (
    refresh,
    get_fcurve_selected_keys,
    get_selected_keys,
    get_prefs
)
from bpy.types import Operator

prefs = None
debug_mode = False
class GRAPHKIT_OT_flatten_holds(Operator):
    """Flatten hold keyframes"""
    bl_idname = "graph.flatten_holds"
    bl_label = "Flatten Holds"
    bl_options = { "REGISTER", "UNDO" }

    selection_mode: bpy.props.EnumProperty(
        name= "Target",
        default="SELECTED_FCURVE",
        description= "Selection to use for flattening holds.",
        items= [("SELECTED_FCURVE","Selected F-Curves","Flatten holds of all keyframes of selected F-Curves.","IPO_EASE_IN_OUT",0),
                ("SELECTED_KEYS","Selected Keyframes","Flatten holds of selected keyframes.","KEYFRAME_HLT",1),
                ("PLAYBACK_RANGE", "Playback Range", "Flatten holds of keyframes in the Playback Range.", "TIME", 2)
                ]
    )

    threshold: bpy.props.FloatProperty(
        name= "Threshold",
        step=0.001,
        default=0.1,
        description= "Threshold for difference in keyframes",
        soft_min = 0.001, soft_max = 30.0
    )

    def flatten_holds(self,keyframes):
        for i,keyframe in enumerate(keyframes):
            if i == len(keyframes)-1:
                break
            first_element = keyframe.co[1]
            next_element = keyframes[i+1].co[1]
            difference = next_element-first_element
            global debug_mode
            if debug_mode:
                print("first_element",first_element,"next_element",next_element,"difference",difference)
            if abs(difference) < self.threshold:
                keyframe.handle_left[1] = first_element
                keyframe.handle_right[1] = first_element
                keyframes[i+1].co[1] = first_element
                keyframes[i+1].handle_left[1] = keyframes[i+1].co[1]
                keyframes[i+1].handle_right[1] = keyframes[i+1].co[1]
                if debug_mode:
                    print("abs(difference) < self.threshold")

    @classmethod
    def poll(self,context):
        return context.selected_visible_fcurves

    def execute(self, context):
        global prefs,debug_mode
        prefs = get_prefs()
        debug_mode = prefs.debug_mode
        fcurves = context.selected_editable_fcurves[:]
        for fcurve in fcurves:
            if debug_mode:
                print(fcurve.data_path,fcurve.array_index)
            selection = get_selected_keys(fcurve,self.selection_mode)
            self.flatten_holds(selection)

        refresh()
        return {'FINISHED'}

def register():
    bpy.utils.register_class(GRAPHKIT_OT_flatten_holds)

def unregister():
    bpy.utils.unregister_class(GRAPHKIT_OT_flatten_holds)
