import bpy
from bpy.types import Operator
from bpy.props import EnumProperty

obj_types = [
            ("ALL", "All", "All object types", "BLENDER", 0),
            ("MESH", "Meshes", "Mesh objects", "MESH_DATA", 1),
            ("CURVE", "Curves", "Curve objects", "CURVE_DATA", 2),
            ("SURFACE", "Surfaces", "Surface objects", "SURFACE_DATA", 3),
            ("META", "Metaballs", "Metaball objects", "META_DATA", 4),
            ("FONT", "Text", "Text objects", "FONT_DATA", 5),
            ("POINTCLOUD", "Point Clouds", "Point Cloud objects", "POINTCLOUD_DATA", 6),
            ("FIELD", "Force Fields", "Force Field objects", "OUTLINER_OB_FORCE_FIELD", 7),
            ("VOLUME", "Volumes", "Volume objects", "VOLUME_DATA", 8),
            ("GPENCIL", "Grease Pencil", "Grease Pencil objects", "GREASEPENCIL", 9),
            None,
            ("ARMATURE", "Armatures", "Armature objects", "ARMATURE_DATA", 10),
            ("LATTICE", "Lattices", "Lattice objects", "LATTICE_DATA", 11),
            None,
            ("EMPTY", "Empties", "Empty objects", "EMPTY_DATA", 12),
            None,
            ("LIGHT", "Lights", "Light objects", "LIGHT_DATA", 13),
            None,
            ("CAMERA", "Cameras", "Camera objects", "CAMERA_DATA", 14),
            None,
            ("SPEAKER", "Speakers", "Speaker objects", "SPEAKER", 15),
            None,
            ("MATERIAL", "Materials", "Material objects", "MATERIAL", 16)
]

def update_visibility(scene, obj_type):
    if obj_type == "ALL":
        bpy.ops.graph.reveal(select=False)
        return

    # Hide all objects and their F-Curves initially
    for obj in scene.objects:
        if obj.animation_data and obj.animation_data.action:
            for fcurve in obj.animation_data.action.fcurves:
                fcurve.hide = True

    # Hide all materials' F-Curves
    for mat in bpy.data.materials:
        if mat.node_tree and mat.node_tree.animation_data and mat.node_tree.animation_data.action:
            for fcurve in mat.node_tree.animation_data.action.fcurves:
                fcurve.hide = True

    # Unhide objects and F-Curves of the specified type
    for obj in scene.objects:
        if obj.type == obj_type:
            if obj.animation_data and obj.animation_data.action:
                for fcurve in obj.animation_data.action.fcurves:
                    fcurve.hide = False

    # Special handling for materials
    if obj_type == 'MATERIAL':
        for mat in bpy.data.materials:
            if mat.node_tree and mat.node_tree.animation_data and mat.node_tree.animation_data.action:
                for fcurve in mat.node_tree.animation_data.action.fcurves:
                    fcurve.hide = False

class GRAPHKIT_OT_solo_fcurves(Operator):
    """Update Visibility of F-Curves and Objects"""
    bl_idname = "graph.solo_fcurves_by_type"
    bl_label = "Update Visibility"
    bl_options = {'REGISTER', 'UNDO'}

    object_type: EnumProperty(
        name="Object Type",
        items=obj_types
    )

    def execute(self, context):
        update_visibility(context.scene, self.object_type)
        return {'FINISHED'}

def register():
    bpy.utils.register_class(GRAPHKIT_OT_solo_fcurves)

def unregister():
    bpy.utils.unregister_class(GRAPHKIT_OT_solo_fcurves)

if __name__ == "__main__":
    register()
