import bpy
from bpy.types import (Operator,PropertyGroup)
from bpy.props import (EnumProperty,
                        PointerProperty,
                        FloatProperty,
                        BoolProperty)
from ..utils.utils import (refresh,
                           get_prefs,
                           get_groups,
                           get_fcurves,
                           get_modifiers)

def mute_all_fcurves(self,context):
    for fcurve in bpy.context.visible_fcurves:
        fcurve.mute = not self.mute
    for group in get_groups():
        if hasattr(group,"mute"):
            group.mute = not self.mute

    refresh()

def lock_all_fcurves(self,context):
    for fcurve in bpy.context.visible_fcurves:
        fcurve.lock = self.lock
    for group in get_groups():
        if hasattr(group,"lock"):
            group.lock = self.lock

    refresh()

def hide_all_fcurves(self,context):
    fcurves = bpy.context.visible_fcurves
    if len(fcurves) == 0:
        fcurves = []
        for action in bpy.data.actions:
            for fcurve in action.fcurves:
                fcurves.append(fcurve)

    for fcurve in fcurves:
        fcurve.hide = not self.hide
    for group in get_groups():
        if hasattr(group,"hide"):
            group.hide = not self.hide
    refresh()

def toggle_all_modifiers(self,context):
    for modifier in get_modifiers():
        modifier.mute = not self.modifiers

def toggle_all_pins(self,context):
    fcurves = bpy.context.visible_fcurves
    actions = [fcurve.id_data for fcurve in fcurves]

    for o in bpy.data.objects:
        if o.animation_data and o.animation_data.action in actions:
            if hasattr(o.animation_data,"use_pin"):
                o.animation_data.use_pin = self.pin
            if hasattr(o.animation_data.action,"use_pin"):
                o.animation_data.action.use_pin = self.pin

    for group in get_groups():
        if hasattr(group,"use_pin"):
            group.use_pin = self.pin

class GRAPHKIT_PG_graph_editor_props(PropertyGroup):
    pin: BoolProperty(
        name = "Pin",
        description = "Toggles pin property of F-Curves and groups visible in the Graph Editor",
        default = False,
        update = toggle_all_pins
    )

    hide: BoolProperty(
        name = "Hide",
        description = "Toggles hide property of F-Curves and groups visible in the Graph Editor",
        default = False,
        update = hide_all_fcurves
    )
    
    lock: BoolProperty(
        name = "Lock",
        description = "Toggles lock property of F-Curves and groups visible in the Graph Editor",
        default = False,
        update = lock_all_fcurves
    )

    mute: BoolProperty(
        name = "Mute",
        description = "Toggles mute property of F-Curves and groups visible in the Graph Editor",
        default = False,
        update = mute_all_fcurves
    )

    modifiers: BoolProperty(
        name = "Modifiers",
        description = "Toggles modifiers of F-Curves and groups visible in the Graph Editor",
        default = True,
        update = toggle_all_modifiers
    )

"""
classes = (GRAPHKIT_PG_graph_editor_props,)
def menu_func(self, context):
    layout = self.layout
    layout.separator()
    props = context.scene.graph_props
    row = layout.row(align=True)

    prefs = get_prefs()

    if not props or not hasattr(props,"pin") or not prefs.show_toggles:
        return

    if prefs.show_pin:
        if props.pin:
            row.prop(props,"pin", text="", icon="PINNED")
        else:
            row.prop(props,"pin", text="", icon="UNPINNED") 

    if prefs.show_hide:
        if props.hide:
            row.prop(props,"hide", text="", icon="HIDE_OFF")
        else:
            row.prop(props,"hide", text="", icon="HIDE_ON") 

    if prefs.show_modifiers:
        if props.modifiers:
            row.prop(props,"modifiers", text="", icon="MODIFIER_ON")
        else:
            row.prop(props,"modifiers", text="", icon="MODIFIER_OFF")

    if prefs.show_mute:
        if props.mute:
            row.prop(props,"mute", text="", icon="CHECKBOX_HLT")
        else:
            row.prop(props,"mute", text="", icon="CHECKBOX_DEHLT")

    if prefs.show_lock:
        if props.lock:
            row.prop(props,"lock", text="", icon="LOCKED")
        else:
            row.prop(props,"lock", text="", icon="UNLOCKED")

    layout.separator()

def register():
    from bpy.utils import register_class
    for cls in classes:
        register_class(cls)
    prefs = get_prefs()
    if prefs.show_toggles:
        if prefs.toggle_location == "MENU_LEFT":
            bpy.types.GRAPH_MT_editor_menus.prepend(menu_func)
        elif prefs.toggle_location == "MENU_RIGHT":
            bpy.types.GRAPH_MT_editor_menus.append(menu_func)
        elif prefs.toggle_location == "HEADER_RIGHT":
            bpy.types.GRAPH_HT_header.append(menu_func)
        
    bpy.types.Scene.graph_props = PointerProperty(type=GRAPHKIT_PG_graph_editor_props)

def unregister():
    del bpy.types.Scene.graph_props

    prefs = get_prefs()
    if prefs.show_toggles:
        if prefs.toggle_location in ["MENU_LEFT","MENU_RIGHT"]:
            bpy.types.GRAPH_MT_editor_menus.remove(menu_func)
        elif prefs.toggle_location == "HEADER_RIGHT":
            bpy.types.GRAPH_HT_header.remove(menu_func)

    from bpy.utils import unregister_class
    for cls in reversed(classes):
        unregister_class(cls)
"""