import bpy
from bpy.types import Menu
from ..icons.custom_icons import get_icon
from ..operators.select import (GRAPHKIT_OT_select_random_fcurves,
                                GRAPHKIT_OT_select_by_side,
                                GRAPHKIT_OT_select_by_type,
                                GRAPHKIT_OT_select_objects_from_fcurves)
from ..operators.visibility import GRAPHKIT_OT_solo_fcurves


class GRAPHKIT_MT_graphkit_context_menu(Menu):
    bl_idname = 'GRAPHKIT_MT_graphkit_context_menu'
    bl_label = "GraphKit"
    bl_description = "GraphKit Menu"

    def draw(self, context):
        layout = self.layout
        layout.operator("graph.align", icon_value=get_icon("Align"))
        layout.operator("graph.amplify", icon_value=get_icon("Amplify"))
        layout.operator("graph.distribute", icon_value=get_icon("Distribute"))
        layout.operator("graph.divide", icon_value=get_icon("Divide"))
        layout.operator("graph.fade", icon_value=get_icon("Fade"))
        layout.operator("graph.flatten_holds", icon_value=get_icon("Flatten_Holds"))
        layout.operator("graph.hold", icon="SNAP_FACE")
        layout.operator("graph.overlap", icon="ARROW_LEFTRIGHT")
        layout.operator("graph.perfect_loop", icon="FILE_REFRESH")
        layout.operator("graph.randomize", icon_value=get_icon("Randomize"))
        layout.operator("graph.range_to_loop", icon="PREVIEW_RANGE")
        layout.operator("graph.repeat", icon_value=get_icon("Repeat"))
        layout.separator()
        layout.operator_menu_enum(GRAPHKIT_OT_select_by_type.bl_idname, "object_type", text="Select All by Type")
        layout.operator_menu_enum(GRAPHKIT_OT_solo_fcurves.bl_idname, "object_type", text="Solo All by Type")
        layout.separator()
        layout.operator(GRAPHKIT_OT_select_objects_from_fcurves.bl_idname, icon="OBJECT_DATAMODE")
        layout.operator(GRAPHKIT_OT_select_random_fcurves.bl_idname, icon="FCURVE")
        sl = layout.operator(GRAPHKIT_OT_select_by_side.bl_idname, text="Select Left Side F-Curves")
        sl.side = "LEFT"
        sr = layout.operator(GRAPHKIT_OT_select_by_side.bl_idname, text="Select Right Side F-Curves")
        sr.side = "RIGHT"

class GRAPHKIT_MT_graphkit_key_menu(Menu):
    bl_idname = 'GRAPHKIT_MT_graphkit_key_menu'
    bl_label = "GraphKit"
    bl_description = "GraphKit Menu"

    def draw(self, context):
        layout = self.layout
        layout.operator("graph.align", icon_value=get_icon("Align"))
        layout.operator("graph.amplify", icon_value=get_icon("Amplify"))
        layout.operator("graph.distribute", icon_value=get_icon("Distribute"))
        layout.operator("graph.divide", icon_value=get_icon("Divide"))
        layout.operator("graph.fade", icon_value=get_icon("Fade"))
        layout.operator("graph.flatten_holds", icon_value=get_icon("Flatten_Holds"))
        layout.operator("graph.hold", icon="SNAP_FACE")
        layout.operator("graph.overlap", icon="ARROW_LEFTRIGHT")
        layout.operator("graph.perfect_loop", icon="FILE_REFRESH")
        layout.operator("graph.randomize", icon_value=get_icon("Randomize"))
        layout.operator("graph.range_to_loop", icon="PREVIEW_RANGE")
        layout.operator("graph.repeat", icon_value=get_icon("Repeat"))

class GRAPHKIT_MT_graphkit_select_menu(Menu):
    bl_idname = 'GRAPHKIT_MT_graphkit_select_menu'
    bl_label = "GraphKit"
    bl_description = "GraphKit Menu"

    def draw(self, context):
        layout = self.layout
        layout.operator_menu_enum(GRAPHKIT_OT_select_by_type.bl_idname, "object_type", text="Select All by Type")
        layout.operator(GRAPHKIT_OT_select_objects_from_fcurves.bl_idname, icon="OBJECT_DATAMODE")
        layout.operator(GRAPHKIT_OT_select_random_fcurves.bl_idname, icon="FCURVE")
        sl = layout.operator(GRAPHKIT_OT_select_by_side.bl_idname, text="Select Left Side F-Curves")
        sl.side = "LEFT"
        sr = layout.operator(GRAPHKIT_OT_select_by_side.bl_idname, text="Select Right Side F-Curves")
        sr.side = "RIGHT"

def menu_func(self, context):
    layout = self.layout
    layout.separator()
    layout.menu(GRAPHKIT_MT_graphkit_key_menu.bl_idname)

def select_menu_func(self, context):
    layout = self.layout
    layout.separator()
    layout.menu(GRAPHKIT_MT_graphkit_select_menu.bl_idname)

def context_menu_func(self, context):
    layout = self.layout
    layout.separator()
    layout.menu(GRAPHKIT_MT_graphkit_context_menu.bl_idname)

def channel_menu_func(self, context):
    layout = self.layout
    layout.separator()
    layout.operator_menu_enum(GRAPHKIT_OT_solo_fcurves.bl_idname, "object_type", text="Solo All by Type")

classes = (GRAPHKIT_MT_graphkit_context_menu,
           GRAPHKIT_MT_graphkit_key_menu,
           GRAPHKIT_MT_graphkit_select_menu,
           )

def register():
    from bpy.utils import register_class
    for cls in classes:
        register_class(cls)
    bpy.types.GRAPH_MT_key.append(menu_func)
    bpy.types.GRAPH_MT_select.append(select_menu_func)
    bpy.types.GRAPH_MT_channel.append(channel_menu_func)
    bpy.types.GRAPH_MT_context_menu.append(context_menu_func)

def unregister():
    bpy.types.GRAPH_MT_context_menu.remove(context_menu_func)
    bpy.types.GRAPH_MT_channel.remove(channel_menu_func)
    bpy.types.GRAPH_MT_select.remove(select_menu_func)
    bpy.types.GRAPH_MT_key.remove(menu_func)
    from bpy.utils import unregister_class
    for cls in reversed(classes):
        unregister_class(cls)