import bpy
from bpy.types import Panel, Operator
from bpy.props import FloatProperty
from ..utils.utils import get_data_fcurve,get_prefs
from ..icons.custom_icons import get_icon

bpy.ordered_fcurves = []
bpy.ordered_objects = []

prefs = None

def ordered_selection(context):
    global prefs
    obj = None
    objs = None

    if context.mode == "OBJECT":
        obj = context.object
        objs = context.selected_objects
    elif context.mode == "POSE":
        obj = context.active_pose_bone
        objs = context.selected_pose_bones

    if obj and objs and (context.mode == "OBJECT" or context.mode == "POSE"):
        obj_length = len(objs)

        if obj_length == 0:
            bpy.ordered_objects = []
        else:
            if obj_length == 1:
                bpy.ordered_objects = []
                bpy.ordered_objects.append({"name":obj.name,"type":type(obj)})
            elif obj_length > len(bpy.ordered_objects):
                for o in objs:
                    if ({"name":o.name,"type":type(o)} in bpy.ordered_objects) is False:
                        bpy.ordered_objects.append({"name":o.name,"type":type(o)})
            elif obj_length < len(bpy.ordered_objects):
                obj_names = [o.name for o in objs] 
                for o in bpy.ordered_objects:
                    if (o["name"] in obj_names) is False:
                        bpy.ordered_objects.remove(o)

        """
        if prefs.debug_mode == True:
            if bpy.ordered_objects:
                print("OBJECTS",[(i,o) for i,o in enumerate(bpy.ordered_objects)])
        """
    else:
        bpy.ordered_objects = []

    fcurves = context.selected_editable_fcurves
    fcurve = None
    if fcurves:
        fcurve = fcurves[0]

    if fcurve and fcurves:
        if len(context.selected_editable_fcurves) == 0:
            bpy.ordered_fcurves = []
        else:
            if len(context.selected_editable_fcurves) == 1:
                bpy.ordered_fcurves = []               
                if fcurve:
                    fcu = get_data_fcurve(fcurve)
                    if fcu:
                        bpy.ordered_fcurves.append(fcu)
            elif len(context.selected_editable_fcurves) > len(bpy.ordered_fcurves):
                names = [(f.action,f.data_path,f.array_index) for f in bpy.ordered_fcurves]
                for f in fcurves:
                    fcu = get_data_fcurve(f)
                    if fcu:
                        if ((fcu.action,fcu.data_path,fcu.array_index) in names) is False:
                            bpy.ordered_fcurves.append(fcu)
            elif len(context.selected_editable_fcurves) < len(bpy.ordered_fcurves):
                names = [(fcurve.id_data.name,f.data_path,f.array_index) for f in fcurves]
                for fcu in bpy.ordered_fcurves:
                    if ((fcu.action,fcu.data_path,fcu.array_index) in names) is False:
                        if fcu in bpy.ordered_objects:
                            bpy.ordered_objects.remove(fcu)

        """
        if prefs.debug_mode == True:
            if bpy.ordered_fcurves:
                print("FCURVES",[(i, f.action, f.nice_fcurve_name()) for i,f in enumerate(bpy.ordered_fcurves)])
        """
    else:
        bpy.ordered_fcurves = []

def draw_toggles(layout,context):
    props = context.scene.graph_props
    prefs = get_prefs()

    if not props or not hasattr(props,"pin") or not prefs.show_toggles:
        return

    grid= layout.grid_flow(row_major=True, columns=0, align=True)
    grid.alignment="EXPAND"

    if prefs.show_pin:
        if props.pin:
            grid.prop(props,"pin", text="", icon="PINNED")
        else:
            grid.prop(props,"pin", text="", icon="UNPINNED") 

    if prefs.show_hide:
        if props.hide:
            grid.prop(props,"hide", text="", icon="HIDE_OFF")
        else:
            grid.prop(props,"hide", text="", icon="HIDE_ON") 

    if prefs.show_modifiers:
        if props.modifiers:
            grid.prop(props,"modifiers", text="", icon="MODIFIER_ON")
        else:
            grid.prop(props,"modifiers", text="", icon="MODIFIER_OFF")

    if prefs.show_mute:
        if props.mute:
            grid.prop(props,"mute", text="", icon="CHECKBOX_HLT")
        else:
            grid.prop(props,"mute", text="", icon="CHECKBOX_DEHLT")

    if prefs.show_lock:
        if props.lock:
            grid.prop(props,"lock", text="", icon="LOCKED")
        else:
            grid.prop(props,"lock", text="", icon="UNLOCKED")
    
    if prefs.show_refresh:
        grid.operator("wm.refresh_graphkit", text="", icon="FILE_REFRESH")
    
    layout.separator()

class GRAPHKIT_OT_set_tween_amt(Operator):
    """Set Tween Amount"""
    bl_idname = "scene.set_float_value"
    bl_label = "Set Float Value"

    value: FloatProperty()

    @classmethod
    def description(cls,context,properties) -> str:
        return f"Tween to {properties.value}"

    def execute(self, context):
        scene = context.scene
        tween_props = scene.tween_props
        tween_props.tween_amt = self.value
        if self.value < -1.0 or self.value > 1.0:
            self.report({"WARNING"},f"Tween amount set to {self.value}")
        else:
            self.report({"INFO"},f"Tween amount set to {self.value}")
        return {'FINISHED'}

class GRAPHKIT_OT_toggle_presets(Operator):
    """Toggle Tween Preset Buttons"""
    bl_idname = "scene.toggle_float_buttons"
    bl_label = ""

    def execute(self, context):
        scene = context.scene
        tween_props = scene.tween_props
        tween_props.show_presets = not tween_props.show_presets
        return {'FINISHED'}

class GRAPHKIT_PT_graphkit_panel(Panel):
    """Creates a Panel in the Graph Editor context"""
    bl_label = ""
    bl_category = "GraphKit"
    bl_idname = "GRAPHKIT_PT_graphkit_panel"
    bl_space_type = "GRAPH_EDITOR"
    bl_region_type = "UI"
    bl_context = "scene"

    def draw_header(self, context):
        global prefs
        prefs = get_prefs()
        if prefs.icons_only:
            self.layout.label(text="", icon="TOOL_SETTINGS")
        else:
            self.layout.label(text="GraphKit", icon="TOOL_SETTINGS")            

    def draw(self, context):
        global prefs
        prefs = get_prefs()
        ordered_selection(context)

        layout = self.layout
        scene = context.scene

        if prefs.toggle_location == "GRAPHKIT_PANEL_TOP":
            draw_toggles(layout,context)

        tween_props = scene.tween_props

        #layout.separator()

        col = layout.column(align=True)
        if context.selected_visible_fcurves:
            col.enabled = True
        else:
            col.enabled = False

        row = col.row(align=True)
        row.prop(tween_props, "tween_amt", slider=True)

        icon = "TRIA_UP" if tween_props.show_presets else "TRIA_DOWN"
        row.operator(GRAPHKIT_OT_toggle_presets.bl_idname, text="", icon=icon)

        if tween_props.show_presets:
            row = col.row(align=True)
            row.prop(tween_props, "tween_type")
            #row = col.row(align=True)
            row = col.grid_flow(row_major=True, columns=13, even_columns=True, even_rows=True, align=True)
            row.scale_y = 0.75
            if prefs.show_alerts:
                row.alert = True
            row.operator("scene.set_float_value", text=" ").value = -1.5
            row.operator("scene.set_float_value", text=" ").value = -1.25
            row.alert = False
            row.operator("scene.set_float_value", text="-1").value = -1.0
            row.operator("scene.set_float_value", text=" ").value = -0.75
            row.operator("scene.set_float_value", text=" ").value = -0.5
            row.operator("scene.set_float_value", text=" ").value = -0.25
            row.operator("scene.set_float_value", text="0").value = 0.0
            row.operator("scene.set_float_value", text=" ").value = 0.25
            row.operator("scene.set_float_value", text=" ").value = 0.5
            row.operator("scene.set_float_value", text=" ").value = 0.75
            row.operator("scene.set_float_value", text="1").value = 1.0
            if prefs.show_alerts:
                row.alert = True
            row.operator("scene.set_float_value", text=" ").value = 1.25
            row.operator("scene.set_float_value", text=" ").value = 1.5
            row.alert = False

        layout.use_property_split = True
        layout.use_property_decorate = False

        layout.separator()

        col = layout.column(align=True)
        col.scale_y = prefs.scale_y
        if prefs.icons_only:
            col.operator("graph.align", text="", icon_value=get_icon("Align"))
            col.operator("graph.amplify", text="", icon_value=get_icon("Amplify"))
            col.operator("graph.distribute", text="", icon_value=get_icon("Distribute"))
            col.operator("graph.divide", text="", icon_value=get_icon("Divide"))
            col.operator("graph.fade", text="", icon_value=get_icon("Fade"))
            col.operator("graph.flatten_holds", text="", icon_value=get_icon("Flatten_Holds"))
            col.operator("graph.hold", text="", icon="SNAP_FACE")
            col.operator("graph.overlap", text="", icon="ARROW_LEFTRIGHT")
            col.operator("graph.perfect_loop", text="", icon="FILE_REFRESH")
            col.operator("graph.randomize", text="", icon_value=get_icon("Randomize"))
            col.operator("graph.range_to_loop", text="", icon="PREVIEW_RANGE")
            col.operator("graph.repeat", text="", icon_value=get_icon("Repeat"))
        else:
            col.operator("graph.align", icon_value=get_icon("Align"))
            col.operator("graph.amplify", icon_value=get_icon("Amplify"))
            col.operator("graph.distribute", icon_value=get_icon("Distribute"))
            col.operator("graph.divide", icon_value=get_icon("Divide"))
            col.operator("graph.fade", icon_value=get_icon("Fade"))
            col.operator("graph.flatten_holds", icon_value=get_icon("Flatten_Holds"))
            col.operator("graph.hold", icon="SNAP_FACE")
            col.operator("graph.overlap", icon="ARROW_LEFTRIGHT")
            col.operator("graph.perfect_loop", icon="FILE_REFRESH")
            col.operator("graph.randomize", icon_value=get_icon("Randomize"))
            col.operator("graph.range_to_loop", icon="PREVIEW_RANGE")
            col.operator("graph.repeat", icon_value=get_icon("Repeat"))
    
        layout.separator()

        if prefs.toggle_location == "GRAPHKIT_PANEL_BOTTOM":
            draw_toggles(layout,context)

classes = (GRAPHKIT_PT_graphkit_panel, GRAPHKIT_OT_set_tween_amt, GRAPHKIT_OT_toggle_presets)

def register():
    from bpy.utils import register_class
    for cls in classes:
        register_class(cls)

def unregister():
    from bpy.utils import unregister_class
    for cls in reversed(classes):
        unregister_class(cls)