import bpy, addon_utils, os, ssl, json
from bpy.types import Operator
from urllib.request import urlopen, Request
import datetime
from .utils import get_prefs
import traceback

latest_version = False
if bpy.app.version >= (4, 2, 0):
    product_name = __package__.split(".")[2]
else:
    product_name = __package__.split(".")[0]
product_id = 574  # GraphKit
friendly_name = product_name.replace("_", " ")
prefs = None

def version_compare(v1, v2):
    arr1 = v1.split(".")
    arr2 = v2.split(".")
    n = len(arr1)
    m = len(arr2)

    arr1 = [int(i) for i in arr1]
    arr2 = [int(i) for i in arr2]

    if n > m:
        for i in range(m, n):
            arr2.append(0)
    elif m > n:
        for i in range(n, m):
            arr1.append(0)

    for i in range(len(arr1)):
        if arr1[i] > arr2[i]:
            return 1
        elif arr2[i] > arr1[i]:
            return -1
    return 0

def check_updates():
    baseurl = f"https://blastframe.com/wp-json/wc/v3/products/{product_id}"
    consumer_key = 'ck_e89865eaa2e33ec882c9b54d485809ccf3fe769a'
    consumer_secret = 'cs_afe7d4ede8385fc304facdfdcb390ae017d1a4e9'
    url = f"{baseurl}?consumer_key={consumer_key}&consumer_secret={consumer_secret}"

    addon_version = [
        addon.bl_info.get("version", (-1, -1, -1))
        for addon in addon_utils.modules()
        if addon.bl_info["name"] == friendly_name
    ][0]
    installed_version = ".".join(str(x) for x in addon_version)
    prefs = get_prefs()

    if (
        not os.environ.get("PYTHONHTTPSVERIFY", "")
        and getattr(ssl, "_create_unverified_context", None)
    ):
        ssl._create_default_https_context = ssl._create_unverified_context

    req = Request(url, headers={'User-Agent': 'Mozilla/5.0'})
    data_string = None
    try:
        data_string = urlopen(req).read()
        data = json.loads(data_string)

        for item in data["meta_data"]:
            if item['key'] == "blastframe_plugin_version":
                latest_version = item['value']
                my_addon = next(addon for addon in addon_utils.modules() if addon.bl_info['name'].lower().replace(" ","_") == friendly_name.lower().replace(" ","_"))
                version = version_compare(installed_version, latest_version)
                if prefs.debug_mode:
                    print(friendly_name, f"installed_version: {installed_version}\nlatest_version: {latest_version}\nversion: {version}")
                if version < 0:
                    prefs.update = True
                    my_addon.bl_info['warning'] = f"Update Available: {latest_version}."
                else:
                    prefs.update = False
                    my_addon.bl_info['warning'] = ""
                prefs.version = latest_version
                break
    except Exception as e:
        if prefs.debug_mode:
            print(f"Error checking for updates: {e}")
        # If there is an error (like no internet connection), skip the update check
        prefs.update = False
        prefs.version = installed_version

def save_update_file(update_file, json_object):
    with open(update_file, "w") as f:
        f.write(json_object)

def auto_update_handler():
    if product_name not in bpy.context.preferences.addons:
        #hasn't been registered yet
        return

    prefs = get_prefs()

    if prefs.autocheck:
        update_file = os.path.join(os.path.dirname(__file__), "updates.json")

        last_update = {"last_update": datetime.datetime.now()}

        json_object = json.dumps(last_update, indent=4, sort_keys=True, default=str)

        if os.path.exists(update_file):
            if os.path.isfile(update_file):
                with open(update_file, "r") as f:
                    try:
                        data = json.load(f)
                    except json.decoder.JSONDecodeError:
                        print(f"{friendly_name} update file not correct. New file added.")
                        save_update_file(update_file, json_object)
                        return
                    d = datetime.datetime.strptime(data["last_update"], "%Y-%m-%d %H:%M:%S.%f")
                    duration = datetime.datetime.now() - d
                    duration = duration.days / 7
                    if duration >= prefs.weeks:
                        check_updates()
                        save_update_file(update_file, json_object)
                    else:
                        if prefs.debug_mode:
                            print(
                                f"{friendly_name} auto-update enabled. {round((prefs.weeks - duration) * 7)} "
                                "days remaining until next update check."
                            )
                        pass
        else:
            save_update_file(update_file, json_object)
            if prefs.debug_mode:
                print(f"{friendly_name} auto-save file does not exist. Creating a new file...")
                print("last_update_data", json_object)

class GRAPHKIT_OT_check_updates(Operator):
    bl_idname = f"{product_name.lower()}.check_updates"
    bl_label = "Check For Updates"
    bl_description = "Checks if there's an update for the add-on"

    def execute(self, context):
        global prefs
        prefs = get_prefs()
        check_updates()
        return {"FINISHED"}

def register():
    from bpy.utils import register_class

    register_class(GRAPHKIT_OT_check_updates)
    global prefs
    prefs = get_prefs()
    auto_update_handler()

def unregister():
    from bpy.utils import unregister_class

    unregister_class(GRAPHKIT_OT_check_updates)
