bl_info = {
    "name": "Key Selected Objects",
    "description": "Insert Keyframes & Keying Set Paths for Selected Objects and Video Clips from the Context Menu.",
    "author": "Blastframe",
    "version": (1, 2, 9),
    "blender": (2, 80, 0),
    "location": "Object Property > Context Menu",
    "doc_url": "https://blastframe.com/docs/key-selected-objects/",
    "tracker_url": "https://blastframe.com/contact/",
    "category": "Animation",
}

_need_reload = "key_selected_objects" in locals()

import bpy

from . import preferences

from .utils import (
    utils,
    check_updates,
)

from .properties import props

from .operators import (
    add_to_keying_set,
    get_outliner_selections,
    delete_all_keying_set_paths,
    key_selected_objects,
    key_selected_clips
)

if _need_reload:
    import importlib
    preferences = importlib.reload(preferences)
    check_updates = importlib.reload(check_updates)
    utils = importlib.reload(utils)
    props = importlib.reload(props)
    add_to_keying_set = importlib.reload(add_to_keying_set)
    delete_all_keying_set_paths = importlib.reload(delete_all_keying_set_paths)
    get_outliner_selections = importlib.reload(get_outliner_selections)
    key_selected_objects = importlib.reload(key_selected_objects)
    key_selected_clips = importlib.reload(key_selected_clips)

modules = [preferences, check_updates, props,
            add_to_keying_set,
            get_outliner_selections,
            key_selected_objects,
            key_selected_clips,
            delete_all_keying_set_paths]

def register():
    from .icons.custom_icons import register_custom_icons
    register_custom_icons()

    for mod in modules:
        mod.register()

def unregister():
    from .icons.custom_icons import unregister_custom_icons
    unregister_custom_icons()

    for mod in reversed(modules):
        mod.unregister()
