import bpy
from bpy.types import Operator

class SCENE_OT_delete_all_paths(Operator):
    bl_idname = "scene.delete_all_paths"
    bl_label = "Delete All Paths"
    bl_options = {'REGISTER', 'INTERNAL'}
   
    def invoke(self, context, event):
        wm = context.window_manager
        return wm.invoke_props_dialog(self)

    def execute(self, context):
        scene = context.scene
        keying_sets = bpy.data.scenes[scene.name].keying_sets
        if len(keying_sets):
            ks_paths = keying_sets.active.paths
            if len(ks_paths):
                keying_sets.active.paths.clear()
        return {'FINISHED'}

def draw_menu(self, context):
    layout = self.layout
    layout.separator()
    layout.operator(SCENE_OT_delete_all_paths.bl_idname, icon="X")

classes = (SCENE_OT_delete_all_paths,)

def register():
    from bpy.utils import register_class
    for cls in classes:
        register_class(cls)

    bpy.types.SCENE_PT_keying_set_paths.append(draw_menu)

def unregister():
    bpy.types.SCENE_PT_keying_set_paths.remove(draw_menu)
    from bpy.utils import unregister_class
    for cls in reversed(classes):
        unregister_class(cls)

if __name__ == "__main__":
    register()