import bpy
from bpy.types import Operator
from ..utils.utils import get_prefs
from ..properties.props import KEYSELECTEDOBJECTS_Props

class KEYSELECTEDOBJECTS_get_outliner_selections(Operator):
    """Get Selected Objects from Outliner"""
    bl_idname = "outliner.selected_objects"
    bl_label = "Outliner Object Selections"
    
    @classmethod
    def poll(cls, context):
        return context.area.type == 'OUTLINER'

    def execute(self, context):
        context.scene.outliner_selections.clear()

        prefs = get_prefs()
        debug_mode = prefs.debug_mode

        if not len(context.selected_ids) and debug_mode:
            print("No selected ids in the Outliner.")

        for item in context.selected_ids:
            obj = bpy.context.scene.outliner_selections.add()
            obj.identifier = item.bl_rna.identifier
            obj.name = item.name
            if debug_mode:
                print(obj.identifier,obj.name)

        return {'FINISHED'}

def register():
    bpy.utils.register_class(KEYSELECTEDOBJECTS_get_outliner_selections)
    bpy.types.Scene.outliner_selections = bpy.props.CollectionProperty(type=KEYSELECTEDOBJECTS_Props)

def unregister():
    bpy.utils.unregister_class(KEYSELECTEDOBJECTS_get_outliner_selections)
    del bpy.types.Scene.outliner_selections