import bpy
from bpy.types import Operator
from bpy.props import BoolProperty
from ..utils.utils import refresh

class OBJECT_OT_Key_Selected_Clips(Operator):
    """Key All Selected Clips"""
    bl_idname = "object.key_all_clips"
    bl_label = "Keyframe All Selected Clips"

    delete: BoolProperty(
        name="Delete Keyframes",
        description="Delete keyframes for multiple properties",
        default=False)

    def execute(self, context):
        if hasattr(context, 'button_prop'):
            if not context.button_prop.is_animatable:
                self.report({"WARNING"}, "Property is not animatable.")
                return {'CANCELLED'}

            property_name = None
            try:
                bpy.ops.ui.copy_data_path_button(full_path=True)
                d_path = context.window_manager.clipboard
                property_name = d_path.split('.')[-1]  # Extract the property name
            except Exception as e:
                self.report({"ERROR"}, f'Could not copy the button data path\n{e}')
                return {'CANCELLED'}

        sequences = context.selected_sequences
        for seq in sequences:
            # Delete keyframes if specified, otherwise insert keyframes
            if self.delete:
                seq.keyframe_delete(data_path=property_name)
            else:
                seq.keyframe_insert(data_path=property_name)
        refresh()
        return {'FINISHED'}

classes = (OBJECT_OT_Key_Selected_Clips,)

def register():
    from bpy.utils import register_class
    for cls in classes:
        register_class(cls)

def unregister():
    from bpy.utils import unregister_class
    for cls in reversed(classes):
        unregister_class(cls)