import bpy
from bpy.props import (
    BoolProperty,
    StringProperty,
    IntProperty
)
from bpy.types import AddonPreferences
from .icons.custom_icons import get_icon

class KEYSELECTEDOBJECTS_preferences(AddonPreferences):
    bl_idname = __package__

    update: BoolProperty(name="Update", default=False)
    version: StringProperty(name="Version", default="")
    autocheck: BoolProperty(name="Auto-check for Update", default=True)
    debug_mode: BoolProperty(name="Debug Mode", default=False)
    weeks: IntProperty(
        min = 1,
        max = 52,
        soft_min = 1,
        soft_max = 52,
        name="Frequency (weeks)",
        default=2
    )

    def draw(self, context):
        layout = self.layout
        col = layout.column()
        col.prop(self, "debug_mode")
        col.separator()
        col.prop(self, "autocheck")
        row = col.row()
        row.enabled = self.autocheck
        box = row.box()
        box.use_property_split = True
        box.use_property_decorate = False
        box.prop(self, "weeks")
        if bpy.app.version >= (4, 2, 0):
            product_name = __package__.split(".")[2]
        else:
            product_name = __package__.split(".")[0]
        col.operator(f"{product_name.lower()}.check_updates", text="Check For Updates", icon="FILE_REFRESH")
        if self.update:
            col.alert = True
            col.label(text=f"Update Available: {str(self.version)}. Please download from the store where you purchased this Add-on.")
        elif self.version != "":
            col.label(text="You have the latest version.")
        col.alert = False
        col.separator()
        row = layout.row(align=True)
        row.scale_y = 1.3
        row.operator("wm.url_open", text="Blastframe", icon_value=get_icon("Blastframe_Logo_White")).url = "https://blastframe.com"
        row.operator("wm.url_open", text="YouTube", icon_value=get_icon("YouTube")).url = "https://www.youtube.com/@blastframe"
        row.operator("wm.url_open", text="Blender Market", icon_value=get_icon("Blender_Market")).url = "https://blendermarket.com/creators/blastframe"

def register():
    from bpy.utils import register_class, unregister_class
    try:
        unregister_class(KEYSELECTEDOBJECTS_preferences)
    except RuntimeError:
        pass
    register_class(KEYSELECTEDOBJECTS_preferences)

def unregister():
    from bpy.utils import unregister_class
    try:
        unregister_class(KEYSELECTEDOBJECTS_preferences)
    except RuntimeError as e:
        print(f"Error unregistering {KEYSELECTEDOBJECTS_preferences}: {e}")