import bpy

def refresh():
    # Updates all animation related UI
    redraw_regions = {
        'VIEW_3D': {'UI'},
        'DOPESHEET_EDITOR': {'WINDOW', 'CHANNELS'},
        'SEQUENCE_EDITOR': {'WINDOW', 'CHANNELS'},
        'GRAPH_EDITOR': {'WINDOW', 'CHANNELS'},
        'NLA_EDITOR': {'WINDOW', 'CHANNELS'},
        'OUTLINER': {'WINDOW'},
        'PROPERTIES': {'WINDOW'},
    }

    for area in bpy.context.screen.areas:
        if area.type in redraw_regions:
            for region in area.regions:
                if region.type in redraw_regions[area.type]:
                    region.tag_redraw()

def show_error(layout,msg,msg_icon="ERROR"):
    print(msg)
    row = layout.row()
    row.label(text=msg, icon=msg_icon)
    row.enabled = False

def use_temp_override():
    version = bpy.app.version
    major = version[0]
    minor = version[1]
    if major < 3 or (major == 3 and minor < 2):
        return False
    else:
        return True

def get_prefs():
    # Attempt to determine the add-on name dynamically from available addons
    addon_name = 'key_selected_objects'  # Replace with the unique part of your add-on's name
    addons = bpy.context.preferences.addons
    
    # Loop through all add-ons to find an exact match with the last part of each key
    for key in addons.keys():
        # Split the key by '.' and check the last element
        if key.split('.')[-1].replace(' ', '_').lower() == addon_name.lower():
            return addons[key].preferences  # Return preferences if exact match is found
    
    return None  # Return None if no exact match is found

def get_debug_mode():
    product_name = __package__.split(".")[0]
    print(product_name)
    prefs = get_prefs()
    debug_mode = None
    if prefs:
        debug_mode = get_prefs().debug_mode
    else:
        debug_mode = True

    return debug_mode