import bpy, math
from bpy.types import (Panel,PropertyGroup)
from ..utils.utils import (sub_v2_v2v2,
    add_v2_v2v2,get_handle_length,get_prefs,normalize_v2_length)
from bpy.props import (BoolProperty,
                       IntProperty,
                       FloatProperty,
                       EnumProperty,
                       PointerProperty)

def rename(dp_name):
    dp_name = "euler_rotation" if dp_name == "rotation_euler" else dp_name
    dp_name = "quaternion_rotation" if dp_name == "rotation_quaternion" else dp_name
    dp_name = "delta_euler_rotation" if dp_name == "delta_rotation_euler" else dp_name
    dp_name = "delta_quaternion_rotation" if dp_name == "delta_rotation_quaternion" else dp_name
    new_name = dp_name.replace("_"," ")
    new_name = new_name.title()
    return new_name

selected_key_count = 0

def get_subtype_and_unit(obj,data_path):
    if obj:
        try:
            data = obj.bl_rna.properties[data_path]
            return data.subtype,data.unit
        except KeyError:
            try:
                data = obj.data.bl_rna.properties[data_path]
                return data.subtype,data.unit
            except:
                return None, None
    return None, None

def is_rotation(context):
    fcurves = context.selected_editable_fcurves[:]
    data_types = [fcurve.data_path.split('.')[-1] for fcurve in fcurves]
    rotations = ["rotation_euler", "rotation_quaternion", "rotation_axis_angle", 
        "delta_rotation_euler", "delta_rotation_quaternion",]    
    rotation_data_paths = [dp in rotations for dp in data_types]
    if sum(rotation_data_paths) == len(data_types):
        return True

def get_data_path(context):
    fcurves = context.selected_editable_fcurves[:]
    data_types = [fcurve.data_path.split('.')[-1] for fcurve in fcurves]
    result = all(element == data_types[0] for element in data_types)
    if (result):
        return data_types[0]
    else:
        return False

def check_selected_keys(context):
    fcurves = context.selected_editable_fcurves[:]
    selected_keys = []
    name = ""
    first_key_data_path = ""
    first_key_array_index = 0
    current_obj = None
    axis_name = [ "X", "Y", "Z" ]
    quat_name = [ "W", "X", "Y", "Z" ]
    transforms = ["location", "rotation_axis_angle", "rotation_euler", "rotation_quaternion", "scale",
        "delta_location", "delta_rotation_euler", "delta_rotation_quaternion",
        "delta_scale"
        ]

    for fcurve in fcurves:
        for keyframe in fcurve.keyframe_points:
            if keyframe.select_control_point:
                if current_obj == None:
                    for o in bpy.data.objects:
                        if o.animation_data and o.animation_data.action is fcurve.id_data:
                            #print(nice_fcurve_name(o, fcurve))
                            current_obj = o.name
                            break
                if current_obj == None:
                    current_obj = ""
                if len(selected_keys) == 0:
                    first_key_data_path = fcurve.data_path
                    first_key_array_index = fcurve.array_index
                selected_keys.append(keyframe)

    if len(selected_keys) > 0:
        if first_key_data_path in transforms:
            if "quaternion" in first_key_data_path:
                name = "%s %s"%(quat_name[first_key_array_index], rename(first_key_data_path))
            else:
                name = "%s %s"%(axis_name[first_key_array_index], rename(first_key_data_path))
        else:
            name = "%s"%(rename(first_key_data_path))

        global selected_key_count

        if current_obj != "" and selected_key_count != len(selected_keys):
            context.workspace.status_text_set("%s keys selected. "\
                "First F-Curve: %s, %s"%(len(selected_keys),current_obj,name))
        elif selected_key_count != len(selected_keys):
            context.workspace.status_text_set("%s keys selected. "\
                "First F-Curve: %s"%(len(selected_keys),name))

        selected_key_count = len(selected_keys)

        return True
    else:
        if selected_key_count != len(selected_keys):
            context.workspace.status_text_set(None)
        selected_key_count = len(selected_keys)
        return False

def get_first_key(context):
    fcurves = context.selected_editable_fcurves[:]
    for i,fcurve in enumerate(fcurves):
        for keyframe in fcurve.keyframe_points:
            if keyframe.select_control_point:
                return keyframe
    return None

def get_selected_keys():
    selected_keys = []
    fcurves = bpy.context.selected_editable_fcurves[:]
    for fcurve in fcurves:
        for keyframe in fcurve.keyframe_points:
            if keyframe.select_control_point:
                selected_keys.append(keyframe)

    return selected_keys

def update_fcurves():
    fcurves = bpy.context.selected_editable_fcurves[:]
    for fcurve in fcurves:
        for keyframe in fcurve.keyframe_points:
            if keyframe.select_control_point:
                fcurve.update()
                break

def get_interpolation(self):
    interpolations = ["CONSTANT", "LINEAR", "BEZIER", "SINE", "QUAD",
                    "CUBIC", "QUART", "QUINT", "EXPO", "CIRC", "BACK",
                    "BOUNCE", "ELASTIC"]
    first_key = get_first_key(bpy.context)
    if first_key and first_key.interpolation in interpolations:
        return interpolations.index(first_key.interpolation)
    else:
        return 0

def get_easing(self):
    easings = ["AUTO", "EASE_IN", "EASE_OUT", "EASE_IN_OUT"]

    first_key = get_first_key(bpy.context)
    if first_key and first_key.easing:
        return easings.index(first_key.easing)
    else:
        return 0

def get_key_frame(self):
    first_key = get_first_key(bpy.context)
    key = 0.0

    if first_key:
        if bpy.context.scene.key_props.sub_frames:
            key = first_key.co[0]
        else:
            key = int(first_key.co[0])

    return key

def get_key_value(self):
    first_key = get_first_key(bpy.context)
    value = 0.0
    if first_key:
        value = first_key.co[1]
    return value

def get_left_handle_type(self):
    types = ["FREE", "ALIGNED", "VECTOR", "AUTO", "AUTO_CLAMPED"]
    default = 0
    first_key = get_first_key(bpy.context)
    if first_key:
        return types.index(first_key.handle_left_type)
    else:
        return default

def get_left_frame(self):
    first_key = get_first_key(bpy.context)
    handle = 0
    if first_key:
        handle = first_key.handle_left[0]
    return handle

def get_left_value(self):
    first_key = get_first_key(bpy.context)
    value = 0.0
    if first_key:
        value = first_key.handle_left[1]
    return value

def get_left_length(self):
    return get_handle_length(get_key_frame(self),
        get_left_frame(self),get_key_value(self),
        get_left_value(self))

def get_right_handle_type(self):
    types = ["FREE", "ALIGNED", "VECTOR", "AUTO", "AUTO_CLAMPED"]
    default = 0
    first_key = get_first_key(bpy.context)
    if first_key:
        return types.index(first_key.handle_right_type)
    else:
        return default

def get_right_frame(self):
    first_key = get_first_key(bpy.context)
    handle = 0
    if first_key:
        handle = first_key.handle_right[0]
    return handle

def get_right_value(self):
    first_key = get_first_key(bpy.context)
    value = 0.0
    if first_key:
        value = first_key.handle_right[1]
    return value

def get_right_length(self):
    return get_handle_length(get_key_frame(self),
        get_right_frame(self),get_key_value(self),
        get_right_value(self))

def set_interpolation(self, value):
    interpolations = ["CONSTANT", "LINEAR", "BEZIER", "SINE", "QUAD",
                      "CUBIC", "QUART", "QUINT", "EXPO", "CIRC", "BACK",
                      "BOUNCE", "ELASTIC"]

    for sk in get_selected_keys():
        sk.interpolation = interpolations[value]
    update_fcurves()

def set_easing(self,value):
    easings = ["AUTO", "EASE_IN", "EASE_OUT", "EASE_IN_OUT"]
    for sk in get_selected_keys():
        sk.easing = easings[value]
    update_fcurves()

def set_key_frame(self, value):
    selected_keys = get_selected_keys()

    if get_prefs().absolute:
        for i, sk in enumerate(selected_keys):
            left_frame_offset = float(sk.co[0]-sk.handle_left[0])
            right_frame_offset = float(sk.co[0]-sk.handle_right[0])
            sk.co[0] = float(value)
            sk.handle_left[0] = value-left_frame_offset
            sk.handle_right[0] = value-right_frame_offset
    else:
        first_key = selected_keys[0]

        act_prev_frame = first_key.co[0]

        left_frame_offset = float(first_key.co[0]-first_key.handle_left[0])
        right_frame_offset = float(first_key.co[0]-first_key.handle_right[0])
        first_key.co[0] = float(value)
        first_key.handle_left[0] = value-left_frame_offset
        first_key.handle_right[0] = value-right_frame_offset

        amt = value-act_prev_frame

        for i, sk in enumerate(selected_keys):
            if i == 0:
                continue
            sk.co[0] += amt
            sk.handle_left[0] += amt
            sk.handle_right[0] += amt

    update_fcurves()

def set_key_value(self,value):
    selected_keys = get_selected_keys()
    if get_prefs().absolute:
        for i, sk in enumerate(selected_keys):
            left_frame_offset = float(sk.co[1]-sk.handle_left[1])
            right_frame_offset = float(sk.co[1]-sk.handle_right[1])
            sk.co[1] = float(value)
            sk.handle_left[1] = value-left_frame_offset
            sk.handle_right[1] = value-right_frame_offset
    else:
        first_key = selected_keys[0]

        act_prev_value = first_key.co[1]

        left_value_offset = first_key.co[1]-first_key.handle_left[1]
        right_value_offset = first_key.co[1]-first_key.handle_right[1]
        first_key.co[1] = value
        first_key.handle_left[1] = value-left_value_offset
        first_key.handle_right[1] = value-right_value_offset

        amt = value-act_prev_value

        for i, sk in enumerate(selected_keys):
            if i == 0:
                continue
            sk.co[1] += amt
            sk.handle_left[1] += amt
            sk.handle_right[1] += amt

    update_fcurves()

def set_left_handle_type(self, value):
    types = ["FREE", "ALIGNED", "VECTOR", "AUTO", "AUTO_CLAMPED"]
    selected_keys = get_selected_keys()

    for sk in selected_keys:
        if sk.handle_left_type in ("AUTO","AUTO_CLAMPED"):
            sk.handle_left_type = "ALIGNED"
            sk.handle_right_type = "ALIGNED"
        sk.handle_left_type = types[value]
    update_fcurves()

def set_left_frame(self, value):
    selected_keys = get_selected_keys()
    if get_prefs().absolute:
        for i, sk in enumerate(selected_keys):
            if sk.handle_left_type in ("AUTO","AUTO_CLAMPED"):
                sk.handle_left_type = "FREE"
                sk.handle_right_type = "FREE"
            sk.handle_left[0] = float(value)
    else:
        first_key = selected_keys[0]

        act_prev_left_frame = first_key.handle_left[0]
        if first_key.handle_left_type in ("AUTO","AUTO_CLAMPED"):
            first_key.handle_left_type = "FREE"
            first_key.handle_right_type = "FREE"
        first_key.handle_left[0] = float(value)

        for i, sk in enumerate(selected_keys):
            if i == 0:
                continue
            offset = sk.handle_left[0] - act_prev_left_frame
            if sk.handle_left_type in ("AUTO","AUTO_CLAMPED"):
                sk.handle_left_type = "FREE"
                sk.handle_right_type = "FREE"
            sk.handle_left[0] = offset + value

    update_fcurves()

def set_left_value(self,value):
    selected_keys = get_selected_keys()
    if get_prefs().absolute:
        for i, sk in enumerate(selected_keys):
            if sk.handle_left_type in ("AUTO","AUTO_CLAMPED"):
                sk.handle_left_type = "FREE"
                sk.handle_right_type = "FREE"
            sk.handle_left[1] = float(value)
    else:
        first_key = selected_keys[0]

        act_prev_value = first_key.handle_left[1]
        first_key.handle_left[1] = value

        for i, sk in enumerate(selected_keys):
            if i == 0:
                continue
            offset = sk.handle_left[1] - act_prev_value
            sk.handle_left[1] = offset + value

    update_fcurves()

def set_left_length(sk,handle_length):
    selected_keys = get_selected_keys()

    for i, sk in enumerate(selected_keys):
        if sk.handle_left_type in ("AUTO","AUTO_CLAMPED"):
            sk.handle_left_type = "FREE"
            sk.handle_right_type = "FREE"

        handle_direction = sub_v2_v2v2(sk.handle_left, sk.co)
        normalize_v2_length(handle_direction, handle_length)
        handle_direction = add_v2_v2v2(sk.handle_left,sk.co,handle_direction)

def set_right_length(sk,handle_length):
    selected_keys = get_selected_keys()
    for i, sk in enumerate(selected_keys):
        if sk.handle_right_type in ("AUTO","AUTO_CLAMPED"):
            sk.handle_left_type = "FREE"
            sk.handle_right_type = "FREE"

        handle_direction = sub_v2_v2v2(sk.handle_right, sk.co)
        normalize_v2_length(handle_direction, handle_length)
        add_v2_v2v2(sk.handle_right,sk.co,handle_direction)

def set_right_type(self, value):
    types = ["FREE", "ALIGNED", "VECTOR", "AUTO", "AUTO_CLAMPED"]
    selected_keys = get_selected_keys()
    for i, sk in enumerate(selected_keys):
        sk.handle_right_type = types[value]
    update_fcurves()

def set_right_frame(self, value):
    selected_keys = get_selected_keys()
    if get_prefs().absolute:
        for i, sk in enumerate(selected_keys):
            if sk.handle_right_type == "ALIGNED" or sk.handle_right_type in ("AUTO","AUTO_CLAMPED"):
                sk.handle_right_type = "FREE"
            sk.handle_right[0] = float(value)
    else:
        first_key = selected_keys[0]

        act_prev_right_frame = first_key.handle_right[0]
        if first_key.handle_right_type == "ALIGNED" or first_key.handle_right_type in ("AUTO","AUTO_CLAMPED"):
            first_key.handle_right_type = "FREE"
        first_key.handle_right[0] = float(value)

        for i, sk in enumerate(selected_keys):
            if i == 0:
                continue
            offset = sk.handle_right[0] - act_prev_right_frame
            if sk.handle_right_type == "ALIGNED" or sk.handle_right_type in ("AUTO","AUTO_CLAMPED"):
                sk.handle_right_type = "FREE"
            sk.handle_right[0] = offset + value

    update_fcurves()

def set_right_value(self,value):
    selected_keys = get_selected_keys()
    if get_prefs().absolute:
        for i, sk in enumerate(selected_keys):
            if sk.handle_right_type == "ALIGNED" or sk.handle_right_type in ("AUTO","AUTO_CLAMPED"):
                sk.handle_right_type = "FREE"
            sk.handle_right[1] = value
    else:
        first_key = selected_keys[0]

        act_prev_value = first_key.handle_right[1]
        if first_key.handle_right_type == "ALIGNED" or first_key.handle_right_type in ("AUTO","AUTO_CLAMPED"):
            first_key.handle_right_type = "FREE"
        first_key.handle_right[1] = value

        for i, sk in enumerate(selected_keys):
            if i == 0:
                continue
            offset = sk.handle_right[1] - act_prev_value
            if sk.handle_right_type == "ALIGNED" or sk.handle_right_type in ("AUTO","AUTO_CLAMPED"):
                sk.handle_right_type = "FREE"
            sk.handle_right[1] = offset + value

    update_fcurves()

def angle(cx, cy, ex, ey):
    dy = ey - cy
    dx = ex - cx
    theta = math.atan2(dy, dx) # range (-PI, PI]
    #theta *= 180 / math.pi # rads to degs, range (-180, 180]
    return theta

class GRAPH_PT_Key_Preferences_Panel(Panel):
    """Creates a Panel in the Graph Editor context"""
    bl_label = ""
    bl_idname = "GRAPH_PT_Key_Preferences_Panel"
    bl_space_type = "GRAPH_EDITOR"
    bl_region_type = "UI"
    bl_category = "Keys"
    bl_options = {'DEFAULT_CLOSED'}

    def draw_header(self, context):
        self.layout.label(text="Preferences", icon="PREFERENCES")

    @classmethod
    def poll(cls, context):
        return check_selected_keys(context)

    def draw(self, context):
        layout = self.layout
        row = layout.row(align=False)
        row.use_property_split = True
        row.prop(context.preferences.edit, "keyframe_new_handle_type", icon_only=True, expand=True)

class GRAPH_PT_Keys_Panel(Panel):
    """Creates a Panel in the Graph Editor context"""
    bl_label = ""
    bl_category = "Keys"
    bl_idname = "GRAPH_PT_Selected_Keys_Panel"
    bl_space_type = "GRAPH_EDITOR"
    bl_region_type = "UI"
    bl_context = "scene"

    def draw_header(self, context):
        self.layout.label(text="Keys", icon="KEYINGSET")

    @classmethod
    def poll(cls, context):
        return check_selected_keys(context)

    def draw(self, context):
        layout = self.layout
        scene = context.scene
        key_props = scene.key_props

        layout.use_property_split = True
        layout.use_property_decorate = False

        layout.separator()

        col = layout.column(align=False)
        row = col.row(align=True)
        row.prop( key_props, "interpolation", text="Interpolation" )

        if key_props.interpolation not in ["CONSTANT","LINEAR","BEZIER"]:
            row = col.row(align=True)
            row.prop( key_props, "easing", text="Easing" )

        col = layout.column(align=True)
        prefs = get_prefs()
        col.prop( prefs, "absolute", text="Absolute")
        col.prop( key_props, "sub_frames", text="Subframes")

        if key_props.sub_frames:
            col.prop( key_props, "key_frame", text= "Key Frame" )
        else:
            col.prop( key_props, "key_frame_int", text= "Key Frame")

        show_angles = is_rotation(context)
        key_value_data_path = get_data_path(context)
        if key_value_data_path:
            subtype,unit = get_subtype_and_unit(context.object,key_value_data_path)
            if unit and hasattr(key_props,"key_value_%s"%unit.lower()):
                col.prop( key_props, "key_value_%s"%unit.lower(), text = "Value" )
            else:
                col.prop( key_props, "key_value", text = "Value" )
        else:
            col.prop( key_props, "key_value", text = "Value" )
        col = layout.column(align=True)
        col.prop( key_props, "left_handle_type", text = "Left Handle Type" )
        col.prop( key_props, "left_frame", text = "Frame" )
        left_value_data_path = get_data_path(context)
        if left_value_data_path:
            subtype,unit = get_subtype_and_unit(context.object,key_value_data_path)
            if unit and hasattr(key_props,"left_value_%s"%unit.lower()):
                col.prop( key_props, "left_value_%s"%unit.lower(), text = "Value" )
            else:
                col.prop( key_props, "left_value", text = "Value" )
        else:
            col.prop( key_props, "left_value", text = "Value" )            
        col.prop( key_props, "left_length", text = "Length" )
        col = layout.column(align=True)
        col.prop( key_props, "right_handle_type", text = "Right Handle Type" )
        col.prop( key_props, "right_frame", text = "Frame" )

        right_value_data_path = get_data_path(context)
        if right_value_data_path:
            subtype,unit = get_subtype_and_unit(context.object,key_value_data_path)
            if unit and hasattr(key_props,"right_value_%s"%unit.lower()):
                col.prop( key_props, "right_value_%s"%unit.lower(), text = "Value" )
            else:
                col.prop( key_props, "right_value", text = "Value" )
        else:
            col.prop( key_props, "right_value", text = "Value" )            

        col.prop( key_props, "right_length", text = "Length" )
        layout.separator()
        if bpy.app.version >= (3, 1, 0):
            handles = layout.operator("graph.equalize_handles", icon="HANDLE_ALIGNED")
            handles.side = "BOTH"


def get_handle_types(self,context):
    return [("FREE","Free","Completely independent manually set handle",
            "HANDLE_FREE", 0),
            ("ALIGNED","Aligned","Manually set handle with rotation locked"\
            " together with its pair",
            "HANDLE_ALIGNED", 1),
            ("VECTOR","Vector","Automatic handles that create straight lines",
            "HANDLE_VECTOR", 2),
            ("AUTO","Automatic","Automatic handles that create smooth curves",
            "HANDLE_AUTO", 3),
            ("AUTO_CLAMPED","Auto Clamped","Automatic handles that create "\
            "smooth curves which only change direction at keyframes",
            "HANDLE_AUTOCLAMPED", 4),
        ]

key_value_desc = "Control Point\nCoordinates of the Control Point. "\
            "Note: Changing this value also updates the handles similar to "\
            "using the graph editor transform operator."

handle_desc = "%s Handle.\nCoordinates of the %s handle "\
            "(%s the control point)"

class Keys_Props(PropertyGroup):
    interpolation: EnumProperty(
        name= "",
        description= "Interpolation method to use for segment of the F-Curve "\
            "from this Keyframe until the next Keyframe.",
        items= [
            ("","Interpolation",""),
            ("CONSTANT","Constant","No interpolation, value of A gets held" \
            " until B is encountered", "IPO_CONSTANT", 0),
            ("LINEAR","Linear","Straight line interpolation between A and "\
            "B (i.e. no ease in/out)", "IPO_LINEAR", 1),
            ("BEZIER","Bezier","Smooth interpolation between A and B, with"\
            " some control over curve shape", "IPO_BEZIER", 2),
            ("","Easing (by strength)",""),
            ("SINE","Sine","Sine","IPO_SINE",3),
            ("QUAD","Quad","Quad","IPO_QUAD",4),
            ("CUBIC","Cubic","Cubic","IPO_CUBIC",5),
            ("QUART","Quart","Quart","IPO_QUART",6),
            ("QUINT","Quint","Quint","IPO_QUINT",7),
            ("EXPO","Expo","Expo","IPO_EXPO",8),
            ("CIRC","Circ","Circ","IPO_CIRC",9),
            ("","Dynamic Effects",""),
            ("BACK","Back","Back","IPO_BACK",10),
            ("BOUNCE","Bounce","Bounce","IPO_BOUNCE",11),
            ("ELASTIC","Elastic","Elastic","IPO_ELASTIC",12),
        ],
        options={'HIDDEN'},
        get = get_interpolation,
        set = set_interpolation
    )

    easing: EnumProperty (
        name= "Easing",
        description= "Which ends of the segment between this and the next keyframe easing interpolation is applied to.",
        default = "EASE_IN_OUT",
        items = [
            ("AUTO","Auto","Easing type is chosen automatically "\
             "based on what the type of interpolation used (e.g. "\
             "Ease In for transitional types, and Ease Out for dynamic effects."),
            ("EASE_IN","Ease In","Only on the end closest to the next keyframe"),
            ("EASE_OUT","Ease Out","Only on the end closest to the first keyframe"),
            ("EASE_IN_OUT","Ease In and Out","Segment between both keyframes"),
        ],
        options={'HIDDEN'},
        get = get_easing,
        set = set_easing
    )

    sub_frames : BoolProperty(
        name="Subframes",
        description="Sub Frame Increments",
        options={'HIDDEN'},
        default = False)

    key_frame: FloatProperty(
        name = "",
        description = key_value_desc,
        options={'HIDDEN'},
        soft_min = -100, soft_max = 1000,
        get = get_key_frame, set = set_key_frame
    )

    key_frame_int: IntProperty(
        name = "",
        description = key_value_desc,
        options={'HIDDEN'},
        soft_min = -100, soft_max = 100,
        get = get_key_frame, set = set_key_frame
    )

    key_value: FloatProperty(
        name = "",
        description = key_value_desc,
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        options={'HIDDEN'},
        get = get_key_value,
        set = set_key_value )

    key_value_none: FloatProperty(
        name = "",
        description = key_value_desc,
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        options={'HIDDEN'},
        unit="NONE",
        get = get_key_value,
        set = set_key_value )

    key_value_length: FloatProperty(
        name = "",
        description = key_value_desc,
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        options={'HIDDEN'},
        unit="LENGTH",
        get = get_key_value,
        set = set_key_value )

    key_value_area: FloatProperty(
        name = "",
        description = key_value_desc,
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        options={'HIDDEN'},
        unit="AREA",
        get = get_key_value,
        set = set_key_value )

    key_value_volume: FloatProperty(
        name = "",
        description = key_value_desc,
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        options={'HIDDEN'},
        unit="VOLUME",
        get = get_key_value,
        set = set_key_value )

    key_value_rotation: FloatProperty(
        name = "",
        description = key_value_desc,
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        options={'HIDDEN'},
        unit="ROTATION",
        get = get_key_value,
        set = set_key_value )

    key_value_time: FloatProperty(
        name = "",
        description = key_value_desc,
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        options={'HIDDEN'},
        unit="TIME",
        get = get_key_value,
        set = set_key_value )

    if (3, 0, 0) > bpy.app.version:
        key_value_time_absolute: FloatProperty(
            name = "",
            description = key_value_desc,
            default=0.0,
            soft_min = -1000, soft_max = 1000,
            options={'HIDDEN'},
            unit="TIME",
            get = get_key_value,
            set = set_key_value )
    else:
        key_value_time_absolute: FloatProperty(
            name = "",
            description = key_value_desc,
            default=0.0,
            soft_min = -1000, soft_max = 1000,
            options={'HIDDEN'},
            unit="TIME_ABSOLUTE",
            get = get_key_value,
            set = set_key_value )

    key_value_velocity: FloatProperty(
        name = "",
        description = key_value_desc,
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        options={'HIDDEN'},
        unit="VELOCITY",
        get = get_key_value,
        set = set_key_value )

    key_value_acceleration: FloatProperty(
        name = "",
        description = key_value_desc,
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        options={'HIDDEN'},
        unit="ACCELERATION",
        get = get_key_value,
        set = set_key_value )

    key_value_mass: FloatProperty(
        name = "",
        description = key_value_desc,
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        options={'HIDDEN'},
        unit="MASS",
        get = get_key_value,
        set = set_key_value )

    key_value_camera: FloatProperty(
        name = "",
        description = key_value_desc,
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        options={'HIDDEN'},
        unit="CAMERA",
        get = get_key_value,
        set = set_key_value )

    key_value_power: FloatProperty(
        name = "",
        description = key_value_desc,
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        options={'HIDDEN'},
        unit="POWER",
        get = get_key_value,
        set = set_key_value )

    key_value_temperature: FloatProperty(
        name = "",
        description = key_value_desc,
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        options={'HIDDEN'},
        unit="TEMPERATURE",
        get = get_key_value,
        set = set_key_value )

    left_handle_type: EnumProperty(
        name= "",
        description= "Type of left handle",
        items= get_handle_types,
        options={'HIDDEN'},
        get = get_left_handle_type,
        set = set_left_handle_type )

    left_frame: FloatProperty(
        name = "",
        description = handle_desc%("Left","left","before"),
        soft_min = -100, soft_max = 1000,
        options={'HIDDEN'},
        get = get_left_frame,
        set = set_left_frame )

    left_value: FloatProperty(
        name = "",
        description = handle_desc%("Left","left","before"),
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        get = get_left_value,
        set = set_left_value )
    
    left_value_none: FloatProperty(
        name = "",
        description = handle_desc%("Left","left","before"),
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        unit="NONE",
        precision=4,
        step=0.01,
        get = get_left_value,
        set = set_left_value )

    left_value_length: FloatProperty(
        name = "",
        description = handle_desc%("Left","left","before"),
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        unit="LENGTH",
        precision=4,
        step=0.01,
        get = get_left_value,
        set = set_left_value )

    left_value_area: FloatProperty(
        name = "",
        description = handle_desc%("Left","left","before"),
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        unit="AREA",
        precision=4,
        step=0.01,
        get = get_left_value,
        set = set_left_value )

    left_value_volume: FloatProperty(
        name = "",
        description = handle_desc%("Left","left","before"),
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        unit="VOLUME",
        precision=4,
        step=0.01,
        get = get_left_value,
        set = set_left_value )

    left_value_rotation: FloatProperty(
        name = "",
        description = handle_desc%("Left","left","before"),
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        unit="ROTATION",
        precision=4,
        step=0.01,
        get = get_left_value,
        set = set_left_value )

    left_value_time: FloatProperty(
        name = "",
        description = handle_desc%("Left","left","before"),
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        unit="TIME",
        precision=4,
        step=0.01,
        get = get_left_value,
        set = set_left_value )

    if (3, 0, 0) > bpy.app.version:
        left_value_time_absolute: FloatProperty(
            name = "",
            description = handle_desc%("Left","left","before"),
            default=0.0,
            soft_min = -1000, soft_max = 1000,
            unit="TIME",
            precision=4,
            step=0.01,
            get = get_left_value,
            set = set_left_value )
    else:
        left_value_time_absolute: FloatProperty(
            name = "",
            description = handle_desc%("Left","left","before"),
            default=0.0,
            soft_min = -1000, soft_max = 1000,
            unit="TIME_ABSOLUTE",
            precision=4,
            step=0.01,
            get = get_left_value,
            set = set_left_value )

    left_value_velocity: FloatProperty(
        name = "",
        description = handle_desc%("Left","left","before"),
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        unit="VELOCITY",
        precision=4,
        step=0.01,
        get = get_left_value,
        set = set_left_value )

    left_value_acceleration: FloatProperty(
        name = "",
        description = handle_desc%("Left","left","before"),
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        unit="ACCELERATION",
        precision=4,
        step=0.01,
        get = get_left_value,
        set = set_left_value )

    left_value_mass: FloatProperty(
        name = "",
        description = handle_desc%("Left","left","before"),
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        unit="MASS",
        precision=4,
        step=0.01,
        get = get_left_value,
        set = set_left_value )

    left_value_camera: FloatProperty(
        name = "",
        description = handle_desc%("Left","left","before"),
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        unit="CAMERA",
        precision=4,
        step=0.01,
        get = get_left_value,
        set = set_left_value )

    left_value_power: FloatProperty(
        name = "",
        description = handle_desc%("Left","left","before"),
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        unit="POWER",
        precision=4,
        step=0.01,
        get = get_left_value,
        set = set_left_value )

    left_value_temperature: FloatProperty(
        name = "",
        description = handle_desc%("Left","left","before"),
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        unit="TEMPERATURE",
        precision=4,
        step=0.01,
        get = get_left_value,
        set = set_left_value )

    left_length: FloatProperty(
        name = "",
        description = handle_desc%("Left","left","before"),
        default=0.0,
        min=0.01,
        options={'HIDDEN'},
        soft_min = -100, soft_max = 100,
        get = get_left_length,
        set = set_left_length)

    right_handle_type: EnumProperty(
        name= "",
        description = "Type of right handle",
        items= get_handle_types, options={'HIDDEN'},
        get = get_right_handle_type, set = set_right_type )
    right_frame: FloatProperty(
        name = "",
        description = handle_desc%("Right","right","after"),
        soft_min = -100, soft_max = 1000,
        options={'HIDDEN'},
        get = get_right_frame, set = set_right_frame )
    right_value: FloatProperty(
        name = "",
        description = handle_desc%("Right","right","after"),
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        options={'HIDDEN'},
        get = get_right_value,
        set = set_right_value )

    right_value_none: FloatProperty(
        name = "",
        description = handle_desc%("Right","right","after"),
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        unit="NONE",
        precision=4,
        step=0.01,
        get = get_right_value,
        set = set_right_value )

    right_value_length: FloatProperty(
        name = "",
        description = handle_desc%("Right","right","after"),
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        unit="LENGTH",
        precision=4,
        step=0.01,
        get = get_right_value,
        set = set_right_value )

    right_value_area: FloatProperty(
        name = "",
        description = handle_desc%("Right","right","after"),
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        unit="AREA",
        precision=4,
        step=0.01,
        get = get_right_value,
        set = set_right_value )

    right_value_volume: FloatProperty(
        name = "",
        description = handle_desc%("Right","right","after"),
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        unit="VOLUME",
        precision=4,
        step=0.01,
        get = get_right_value,
        set = set_right_value )

    right_value_rotation: FloatProperty(
        name = "",
        description = handle_desc%("Right","right","after"),
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        unit="ROTATION",
        precision=4,
        step=0.01,
        get = get_right_value,
        set = set_right_value )

    right_value_time: FloatProperty(
        name = "",
        description = handle_desc%("Right","right","after"),
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        unit="TIME",
        precision=4,
        step=0.01,
        get = get_right_value,
        set = set_right_value )

    if (3, 0, 0) > bpy.app.version:
        right_value_time_absolute: FloatProperty(
            name = "",
            description = handle_desc%("Right","right","after"),
            default=0.0,
            soft_min = -1000, soft_max = 1000,
            unit="TIME",
            precision=4,
            step=0.01,
            get = get_right_value,
            set = set_right_value )
    else:
        right_value_time_absolute: FloatProperty(
            name = "",
            description = handle_desc%("Right","right","after"),
            default=0.0,
            soft_min = -1000, soft_max = 1000,
            unit="TIME_ABSOLUTE",
            precision=4,
            step=0.01,
            get = get_right_value,
            set = set_right_value )

    right_value_velocity: FloatProperty(
        name = "",
        description = handle_desc%("Right","right","after"),
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        unit="VELOCITY",
        precision=4,
        step=0.01,
        get = get_right_value,
        set = set_right_value )

    right_value_acceleration: FloatProperty(
        name = "",
        description = handle_desc%("Right","right","after"),
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        unit="ACCELERATION",
        precision=4,
        step=0.01,
        get = get_right_value,
        set = set_right_value )

    right_value_mass: FloatProperty(
        name = "",
        description = handle_desc%("Right","right","after"),
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        unit="MASS",
        precision=4,
        step=0.01,
        get = get_right_value,
        set = set_right_value )

    right_value_camera: FloatProperty(
        name = "",
        description = handle_desc%("Right","right","after"),
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        unit="CAMERA",
        precision=4,
        step=0.01,
        get = get_right_value,
        set = set_right_value )

    right_value_power: FloatProperty(
        name = "",
        description = handle_desc%("Right","right","after"),
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        unit="POWER",
        precision=4,
        step=0.01,
        get = get_right_value,
        set = set_right_value )

    right_value_temperature: FloatProperty(
        name = "",
        description = handle_desc%("Right","right","after"),
        default=0.0,
        soft_min = -1000, soft_max = 1000,
        unit="TEMPERATURE",
        precision=4,
        step=0.01,
        get = get_right_value,
        set = set_right_value )

    right_length: FloatProperty(
        name = "",
        description = handle_desc%("Right","right","after"),
        default=0.0,
        min=0.01,
        options={'HIDDEN'},
        soft_min = -100, soft_max = 100,
        get = get_right_length,
        set = set_right_length)

def register():
    bpy.utils.register_class(Keys_Props)
    bpy.types.Scene.key_props = PointerProperty(type= Keys_Props)
    #bpy.utils.register_class(GRAPH_PT_Key_Preferences_Panel)
    bpy.utils.register_class(GRAPH_PT_Keys_Panel)

def unregister():
    #bpy.utils.unregister_class(GRAPH_PT_Key_Preferences_Panel)
    bpy.utils.unregister_class(GRAPH_PT_Keys_Panel)
    bpy.utils.unregister_class(Keys_Props)
    del bpy.types.Scene.key_props
