import bpy, math

def refresh():
    # Updates all animation related UI
    for area in bpy.context.screen.areas:
        if area.type in {'VIEW_3D', 'DOPESHEET_EDITOR', 'GRAPH_EDITOR', 'NLA_EDITOR', 'OUTLINER', 'PROPERTIES'}:
            for region in area.regions:
                if (area.type == 'VIEW_3D' and region.type == 'UI') or \
                   (area.type != 'VIEW_3D' and region.type in {'WINDOW', 'CHANNELS'}):
                    region.tag_redraw()

def get_prefs():
    # Attempt to determine the add-on name dynamically from available addons
    addon_name = 'keys'  # Replace with the unique part of your add-on's name
    addons = bpy.context.preferences.addons
    
    # Loop through all add-ons to find an exact match with the last part of each key
    for key in addons.keys():
        # Split the key by '.' and check the last element
        if key.split('.')[-1].replace(' ', '_').lower() == addon_name.lower():
            return addons[key].preferences  # Return preferences if exact match is found
    
    return None  # Return None if no exact match is found

def magnitude(p):
  return math.sqrt(p[0]**2 + p[1]**2)

def normalize(p):
  m = magnitude(p)
  if (m > 0):
    return [p[0]/m,p[1]/m]

def line_length(pt1,pt2):
    a = pt2[0] - pt1[0]
    b = pt2[1] - pt1[1]
    return math.sqrt(a**2 + b**2)

def dot(pt1, pt2):
    return pt1[0]*pt2[0] + pt1[1]*pt2[1]

def diff(pt1,pt2):
  return [pt1[0]-pt2[0],pt1[1]-pt2[1]]

def sqr(pt):
  return [pt[0]**2,pt[1]**2]

def prod(pt1,pt2):
  return [pt1[0]*pt2[0],pt1[1]*pt2[1]]

def mult(pt,f):
  return [pt[0]*f,pt[1]*f]

def add(pt1,pt2):
  return [pt1[0]+pt2[0],pt1[1]+pt2[1]]

def div(pt,q):
  return [pt[0]/q,pt[1]/q]

def get_handle_length(x1, x2, y1, y2):
    a = x1 - x2
    b = y1 - y2

    c = math.sqrt( a*a + b*b )
    return c

def sub_v2_v2v2(a, b):
    r = [0,0]
    r[0] = a[0] - b[0]
    r[1] = a[1] - b[1]
    return r

def dot_v2v2(a, b):
    return a[0] * b[0] + a[1] * b[1]

def mul_v2_v2fl(r, a, f):
    r[0] = a[0] * f
    r[1] = a[1] * f

def zero_v2(r):
    r[0] = 0.0
    r[1] = 0.0

def normalize_v2_v2_length(r, a, unit_length):
    d = dot_v2v2(a, a)
    if (d > 1.0e-35):
        d = math.sqrt(d)
        mul_v2_v2fl(r, a, unit_length / d)
    else:
        # Either the vector is small or one of it's values contained `nan`.
        zero_v2(r)
        d = 0.0
    return d

def normalize_v2_length(n, unit_length):
    return normalize_v2_v2_length(n, n, unit_length)

def add_v2_v2v2(r, a, b):
    r[0] = a[0] + b[0]
    r[1] = a[1] + b[1]
    return r