# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTIBILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

bl_info = {
    "name" : "Lazy Bones",
    "author" : "Blue Nile 3D", 
    "description" : "",
    "blender" : (3, 6, 0),
    "version" : (1, 0, 2),
    "location" : "",
    "warning" : "",
    "doc_url": "", 
    "tracker_url": "", 
    "category" : "Rigging" 
}


import bpy
import bpy.utils.previews
import os
import math
import mathutils


addon_keymaps = {}
_icons = None
lazy_bone_bone_target_serpens = {'sna_lazyboneselectionsettempvar': [], }
lazy_bone_symmetrize_serpens = {'sna_lazybones0bonesdeletevar': [], }


def property_exists(prop_path, glob, loc):
    try:
        eval(prop_path, glob, loc)
        return True
    except:
        return False


def load_preview_icon(path):
    global _icons
    if not path in _icons:
        if os.path.exists(path):
            _icons.load(path, path, "IMAGE")
        else:
            return 0
    return _icons[path].icon_id


class SNA_OT_Lazy_Bones_Select_Visible_Armature_Operator_0Cdc9(bpy.types.Operator):
    bl_idname = "sna.lazy_bones_select_visible_armature_operator_0cdc9"
    bl_label = "Lazy Bones Select Visible Armature Operator"
    bl_description = "Selects all armatures objects visible in the viewport"
    bl_options = {"REGISTER", "UNDO"}

    @classmethod
    def poll(cls, context):
        if bpy.app.version >= (3, 0, 0) and True:
            cls.poll_message_set('')
        return not False

    def execute(self, context):
        for i_C557D in range(len(bpy.context.visible_objects)):
            if bpy.context.visible_objects[i_C557D].type == 'ARMATURE':
                bpy.context.visible_objects[i_C557D].select_set(state=True, view_layer=bpy.context.view_layer, )
                bpy.context.view_layer.objects.active = bpy.context.visible_objects[i_C557D]
        return {"FINISHED"}

    def invoke(self, context, event):
        return self.execute(context)


class SNA_OT_Lazy_Bones_Dissolve_To_1_0D25F(bpy.types.Operator):
    bl_idname = "sna.lazy_bones_dissolve_to_1_0d25f"
    bl_label = "Lazy Bones Dissolve to 1"
    bl_description = "Dissolves all bones in the selected armature to 1 bones. May fail with closed loops of bones."
    bl_options = {"REGISTER", "UNDO"}

    @classmethod
    def poll(cls, context):
        if bpy.app.version >= (3, 0, 0) and True:
            cls.poll_message_set('')
        return not False

    def execute(self, context):
        if (bpy.context.view_layer.objects.active == None):
            self.report({'ERROR'}, message='No active object.')
        else:
            if bpy.context.view_layer.objects.active.type == 'ARMATURE':
                bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='EDIT')
                bpy.ops.armature.select_all('INVOKE_DEFAULT', action='SELECT')
                bpy.ops.armature.dissolve('INVOKE_DEFAULT', )
                bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='OBJECT')
        return {"FINISHED"}

    def invoke(self, context, event):
        return self.execute(context)


class SNA_PT_BONE_DOCTOR_7679E(bpy.types.Panel):
    bl_label = 'Bone Doctor'
    bl_idname = 'SNA_PT_BONE_DOCTOR_7679E'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_context = ''
    bl_category = 'Lazy Bones'
    bl_order = 0
    bl_ui_units_x=0

    @classmethod
    def poll(cls, context):
        return not (((not bpy.context.view_layer.objects.active.type == 'ARMATURE') if (bpy.context.view_layer.objects.active != None) else True))

    def draw_header(self, context):
        layout = self.layout

    def draw(self, context):
        layout = self.layout
        layout.template_icon(icon_value=load_preview_icon(os.path.join(os.path.dirname(__file__), 'assets', 'Bone Doctor Icon.png')), scale=5.0)
        box_0C0B5 = layout.box()
        box_0C0B5.alert = False
        box_0C0B5.enabled = True
        box_0C0B5.active = True
        box_0C0B5.use_property_split = False
        box_0C0B5.use_property_decorate = False
        box_0C0B5.alignment = 'Expand'.upper()
        box_0C0B5.scale_x = 1.0
        box_0C0B5.scale_y = 1.0
        if not True: box_0C0B5.operator_context = "EXEC_DEFAULT"
        op = box_0C0B5.operator('sna.lazy_bones_select_visible_armature_operator_0cdc9', text='Select visible armatures', icon_value=256, emboss=True, depress=False)
        layout.separator(factor=1.0)
        box_21AE9 = layout.box()
        box_21AE9.alert = False
        box_21AE9.enabled = True
        box_21AE9.active = True
        box_21AE9.use_property_split = False
        box_21AE9.use_property_decorate = False
        box_21AE9.alignment = 'Expand'.upper()
        box_21AE9.scale_x = 1.0
        box_21AE9.scale_y = 1.0
        if not True: box_21AE9.operator_context = "EXEC_DEFAULT"
        row_3E9C1 = box_21AE9.row(heading='', align=False)
        row_3E9C1.alert = False
        row_3E9C1.enabled = True
        row_3E9C1.active = True
        row_3E9C1.use_property_split = False
        row_3E9C1.use_property_decorate = False
        row_3E9C1.scale_x = 1.0
        row_3E9C1.scale_y = 1.0
        row_3E9C1.alignment = 'Expand'.upper()
        row_3E9C1.operator_context = "INVOKE_DEFAULT" if True else "EXEC_DEFAULT"
        op = row_3E9C1.operator('sna.lazy_bones_show_bone_axis_operator_026a7', text='Axes', icon_value=254, emboss=True, depress=False)
        op = row_3E9C1.operator('sna.lazy_bones_hide_bone_axis_operator_02f86', text='Axes', icon_value=253, emboss=True, depress=False)
        layout.separator(factor=1.0)
        box_D2112 = layout.box()
        box_D2112.alert = False
        box_D2112.enabled = True
        box_D2112.active = True
        box_D2112.use_property_split = False
        box_D2112.use_property_decorate = False
        box_D2112.alignment = 'Expand'.upper()
        box_D2112.scale_x = 1.0
        box_D2112.scale_y = 1.0
        if not True: box_D2112.operator_context = "EXEC_DEFAULT"
        row_B8D8C = box_D2112.row(heading='', align=True)
        row_B8D8C.alert = False
        row_B8D8C.enabled = True
        row_B8D8C.active = True
        row_B8D8C.use_property_split = False
        row_B8D8C.use_property_decorate = False
        row_B8D8C.scale_x = 1.0
        row_B8D8C.scale_y = 1.0
        row_B8D8C.alignment = 'Expand'.upper()
        row_B8D8C.operator_context = "INVOKE_DEFAULT" if True else "EXEC_DEFAULT"
        op = row_B8D8C.operator('sna.lazy_bones_dissolve_by_fraction_operator_df537', text='Dissolve by', icon_value=0, emboss=True, depress=False)
        row_B8D8C.prop(bpy.context.scene, 'sna_lb_dissolveby', text='', icon_value=0, emboss=True)
        op = box_D2112.operator('sna.lazy_bones_dissolve_to_1_0d25f', text='Dissolve to one', icon_value=0, emboss=True, depress=False)
        layout.separator(factor=1.0)
        box_C763F = layout.box()
        box_C763F.alert = False
        box_C763F.enabled = True
        box_C763F.active = True
        box_C763F.use_property_split = False
        box_C763F.use_property_decorate = False
        box_C763F.alignment = 'Expand'.upper()
        box_C763F.scale_x = 1.0
        box_C763F.scale_y = 1.0
        if not True: box_C763F.operator_context = "EXEC_DEFAULT"
        box_C763F.label(text='Bone Roll - Global', icon_value=0)
        grid_BA5D9 = box_C763F.grid_flow(columns=4, row_major=False, even_columns=True, even_rows=False, align=True)
        grid_BA5D9.enabled = True
        grid_BA5D9.active = True
        grid_BA5D9.use_property_split = False
        grid_BA5D9.use_property_decorate = False
        grid_BA5D9.alignment = 'Expand'.upper()
        grid_BA5D9.scale_x = 1.0
        grid_BA5D9.scale_y = 1.0
        if not True: grid_BA5D9.operator_context = "EXEC_DEFAULT"
        op = grid_BA5D9.operator('sna.lazy_bones_bone_roll_global_x_operator_168ea', text='+', icon_value=912, emboss=True, depress=False)
        op = grid_BA5D9.operator('sna.lazy_bones_bone_roll_global_y_operator_50848', text='+', icon_value=913, emboss=True, depress=False)
        op = grid_BA5D9.operator('sna.lazy_bones_bone_roll_global_z_operator_2ef8d', text='+', icon_value=914, emboss=True, depress=False)
        grid_65D5D = box_C763F.grid_flow(columns=4, row_major=False, even_columns=True, even_rows=False, align=True)
        grid_65D5D.enabled = True
        grid_65D5D.active = True
        grid_65D5D.use_property_split = False
        grid_65D5D.use_property_decorate = False
        grid_65D5D.alignment = 'Expand'.upper()
        grid_65D5D.scale_x = 1.0
        grid_65D5D.scale_y = 1.0
        if not True: grid_65D5D.operator_context = "EXEC_DEFAULT"
        op = grid_65D5D.operator('sna.lazy_bones_bone_roll_global_x_operator_eaa28', text='-', icon_value=912, emboss=True, depress=False)
        op = grid_65D5D.operator('sna.lazy_bones_bone_roll_global_y_operator_96065', text='-', icon_value=913, emboss=True, depress=False)
        op = grid_65D5D.operator('sna.lazy_bones_bone_roll_global_z_operator_d986f', text='-', icon_value=914, emboss=True, depress=False)
        box_C763F.label(text='Bone Roll - Local', icon_value=0)
        grid_6C6D0 = box_C763F.grid_flow(columns=4, row_major=False, even_columns=True, even_rows=False, align=True)
        grid_6C6D0.enabled = True
        grid_6C6D0.active = True
        grid_6C6D0.use_property_split = False
        grid_6C6D0.use_property_decorate = False
        grid_6C6D0.alignment = 'Expand'.upper()
        grid_6C6D0.scale_x = 1.0
        grid_6C6D0.scale_y = 1.0
        if not True: grid_6C6D0.operator_context = "EXEC_DEFAULT"
        op = grid_6C6D0.operator('sna.lazy_bones_bone_roll_local_x_operator_10200', text='+', icon_value=912, emboss=True, depress=False)
        op = grid_6C6D0.operator('sna.lazy_bones_bone_roll_local_z_operator_c377a', text='+', icon_value=914, emboss=True, depress=False)
        grid_6666D = box_C763F.grid_flow(columns=4, row_major=False, even_columns=True, even_rows=False, align=True)
        grid_6666D.enabled = True
        grid_6666D.active = True
        grid_6666D.use_property_split = False
        grid_6666D.use_property_decorate = False
        grid_6666D.alignment = 'Expand'.upper()
        grid_6666D.scale_x = 1.0
        grid_6666D.scale_y = 1.0
        if not True: grid_6666D.operator_context = "EXEC_DEFAULT"
        op = grid_6666D.operator('sna.lazy_bones_bone_roll_local_x_operator_d57fd', text='-', icon_value=912, emboss=True, depress=False)
        op = grid_6666D.operator('sna.lazy_bones_bone_roll_local_z_operator_dc8d7', text='-', icon_value=914, emboss=True, depress=False)
        layout.separator(factor=1.0)
        box_9D962 = layout.box()
        box_9D962.alert = False
        box_9D962.enabled = True
        box_9D962.active = True
        box_9D962.use_property_split = False
        box_9D962.use_property_decorate = False
        box_9D962.alignment = 'Expand'.upper()
        box_9D962.scale_x = 1.0
        box_9D962.scale_y = 1.0
        if not True: box_9D962.operator_context = "EXEC_DEFAULT"
        op = box_9D962.operator('sna.lazy_bones_switch_bone_direction_operator_10514', text='Switch Direction', icon_value=320, emboss=True, depress=False)
        op = box_9D962.operator('sna.bone_target_whole_817da', text='Bone Target (Whole Armature)', icon_value=552, emboss=True, depress=False)
        op = box_9D962.operator('sna.bone_target_individual_e2333', text='Bone Target (Individual Bones)', icon_value=620, emboss=True, depress=False)
        layout.separator(factor=1.0)
        box_AFE7E = layout.box()
        box_AFE7E.alert = False
        box_AFE7E.enabled = True
        box_AFE7E.active = True
        box_AFE7E.use_property_split = False
        box_AFE7E.use_property_decorate = False
        box_AFE7E.alignment = 'Expand'.upper()
        box_AFE7E.scale_x = 1.0
        box_AFE7E.scale_y = 1.0
        if not True: box_AFE7E.operator_context = "EXEC_DEFAULT"
        op = box_AFE7E.operator('sna.delete_half_world_space_f888f', text='Delete Half (World Space)', icon_value=21, emboss=True, depress=False)
        op.sna_use_existing_symmetry_suffixes = False
        op = box_AFE7E.operator('sna.symmetrize_world_space_68cc8', text='Symmetrize (World Space)', icon_value=446, emboss=True, depress=False)
        op.sna_use_existing_symmetry_suffixes = False


class SNA_OT_Bone_Target_Individual_E2333(bpy.types.Operator):
    bl_idname = "sna.bone_target_individual_e2333"
    bl_label = "Bone Target (Individual)"
    bl_description = "Points each individual bone either toward/away from the 3d cursor. May break bone relations."
    bl_options = {"REGISTER", "UNDO"}

    def sna_target_direction_enum_items(self, context):
        return [("No Items", "No Items", "No generate enum items node found to create items!", "ERROR", 0)]
    sna_target_direction: bpy.props.EnumProperty(name='Target Direction?', description='', items=[('Point towards target', 'Point towards target', '', 0, 0), ('Point away from target', 'Point away from target', '', 0, 1)])

    @classmethod
    def poll(cls, context):
        if bpy.app.version >= (3, 0, 0) and True:
            cls.poll_message_set('')
        return not False

    def execute(self, context):
        sna_lazy_bones_bone_target_indiv_func_AA6CD(bpy.context.scene.cursor.location, self.sna_target_direction)
        return {"FINISHED"}

    def draw(self, context):
        layout = self.layout
        if bpy.context.preferences.addons['lazy_bones'].preferences.sna_lazy_bones_show_tips:
            box_2492F = layout.box()
            box_2492F.alert = False
            box_2492F.enabled = True
            box_2492F.active = True
            box_2492F.use_property_split = False
            box_2492F.use_property_decorate = False
            box_2492F.alignment = 'Expand'.upper()
            box_2492F.scale_x = 1.0
            box_2492F.scale_y = 1.0
            if not True: box_2492F.operator_context = "EXEC_DEFAULT"
            box_2492F.label(text='The target is the 3d cursor.', icon_value=4)
            box_2492F.label(text='Individual bones will be flipped/not.', icon_value=4)
            box_2492F.label(text='This method is much more accurate than the (Whole Armature) method.', icon_value=4)
            box_2492F.label(text='Great for splitting directions in the middle of a complex armature chain/loop.', icon_value=4)
            box_2492F.separator(factor=1.0)
            if bpy.context.preferences.addons['lazy_bones'].preferences.sna_lazy_bones_show_tips:
                box_5B366 = box_2492F.box()
                box_5B366.alert = True
                box_5B366.enabled = True
                box_5B366.active = True
                box_5B366.use_property_split = False
                box_5B366.use_property_decorate = False
                box_5B366.alignment = 'Expand'.upper()
                box_5B366.scale_x = 1.0
                box_5B366.scale_y = 1.0
                if not True: box_5B366.operator_context = "EXEC_DEFAULT"
                box_5B366.label(text='Bone relationships will not be maintained with this method.', icon_value=2)
                box_5B366.label(text='         Flipped bones will need to be manually reparented.', icon_value=0)
                box_5B366.label(text='         Solving this is a focus of the next update.', icon_value=0)
        layout.separator(factor=1.0)
        layout.prop(self, 'sna_target_direction', text='Force Bone Direction?', icon_value=0, emboss=True)

    def invoke(self, context, event):
        return context.window_manager.invoke_props_dialog(self, width=500)


class SNA_OT_Lazy_Bones_Hide_Bone_Axis_Operator_02F86(bpy.types.Operator):
    bl_idname = "sna.lazy_bones_hide_bone_axis_operator_02f86"
    bl_label = "Lazy Bones Hide Bone Axis Operator"
    bl_description = "Hides bone axes for selected armatures."
    bl_options = {"REGISTER", "UNDO"}

    @classmethod
    def poll(cls, context):
        if bpy.app.version >= (3, 0, 0) and True:
            cls.poll_message_set('')
        return not False

    def execute(self, context):
        sna_lazy_bone_show_bone_axis_func_2C119(False)
        return {"FINISHED"}

    def invoke(self, context, event):
        return self.execute(context)


class SNA_OT_Bone_Target_Whole_817Da(bpy.types.Operator):
    bl_idname = "sna.bone_target_whole_817da"
    bl_label = "Bone Target (Whole)"
    bl_description = "Gets the average direction of all bones in an armature. Then points all bones together toward/away from the 3d cursor."
    bl_options = {"REGISTER", "UNDO"}

    def sna_target_direction_enum_items(self, context):
        return [("No Items", "No Items", "No generate enum items node found to create items!", "ERROR", 0)]
    sna_target_direction: bpy.props.EnumProperty(name='Target Direction?', description='', items=[('Point towards target', 'Point towards target', '', 0, 0), ('Point Away From Target', 'Point Away From Target', '', 0, 1)])

    @classmethod
    def poll(cls, context):
        if bpy.app.version >= (3, 0, 0) and True:
            cls.poll_message_set('')
        return not False

    def execute(self, context):
        sna_lazy_bones_bone_target_whole_func_2AE3D(bpy.context.scene.cursor.location, self.sna_target_direction)
        return {"FINISHED"}

    def draw(self, context):
        layout = self.layout
        if bpy.context.preferences.addons['lazy_bones'].preferences.sna_lazy_bones_show_tips:
            box_BE64E = layout.box()
            box_BE64E.alert = False
            box_BE64E.enabled = True
            box_BE64E.active = True
            box_BE64E.use_property_split = False
            box_BE64E.use_property_decorate = False
            box_BE64E.alignment = 'Expand'.upper()
            box_BE64E.scale_x = 1.0
            box_BE64E.scale_y = 1.0
            if not True: box_BE64E.operator_context = "EXEC_DEFAULT"
            box_BE64E.label(text='The target is the 3d cursor.', icon_value=4)
            box_BE64E.label(text='The whole armature will be flipped/not flipped. ', icon_value=4)
            box_BE64E.label(text='Good for flipping single straight armature chains.', icon_value=4)
            box_BE64E.label(text='Accuracy is not great for curved/complex bone chains.', icon_value=4)
            box_BE64E.label(text='Bone parenting will be retained.', icon_value=4)
        layout.separator(factor=1.0)
        layout.prop(self, 'sna_target_direction', text='Force Bone Direction?', icon_value=0, emboss=True)

    def invoke(self, context, event):
        return context.window_manager.invoke_props_dialog(self, width=500)


class SNA_OT_Lazy_Bones_Bone_Roll_Global_Z_Operator_D986F(bpy.types.Operator):
    bl_idname = "sna.lazy_bones_bone_roll_global_z_operator_d986f"
    bl_label = "Lazy Bones Bone Roll Global Z- Operator"
    bl_description = "Sets the bones roll of all bones in selected armatures to -Z (World Space)"
    bl_options = {"REGISTER", "UNDO"}

    @classmethod
    def poll(cls, context):
        if bpy.app.version >= (3, 0, 0) and True:
            cls.poll_message_set('')
        return not False

    def execute(self, context):
        sna_lazy_bones_set_bone_roll_func_7F104('GLOBAL_NEG_Z', False)
        return {"FINISHED"}

    def invoke(self, context, event):
        return self.execute(context)


class SNA_OT_Lazy_Bones_Bone_Roll_Global_Y_Operator_50848(bpy.types.Operator):
    bl_idname = "sna.lazy_bones_bone_roll_global_y_operator_50848"
    bl_label = "Lazy Bones Bone Roll Global Y+ Operator"
    bl_description = "Sets the bones roll of all bones in selected armatures to +Y (World Space)"
    bl_options = {"REGISTER", "UNDO"}

    @classmethod
    def poll(cls, context):
        if bpy.app.version >= (3, 0, 0) and True:
            cls.poll_message_set('')
        return not False

    def execute(self, context):
        sna_lazy_bones_set_bone_roll_func_7F104('GLOBAL_POS_Y', False)
        return {"FINISHED"}

    def invoke(self, context, event):
        return self.execute(context)


class SNA_OT_Lazy_Bones_Bone_Roll_Global_Z_Operator_2Ef8D(bpy.types.Operator):
    bl_idname = "sna.lazy_bones_bone_roll_global_z_operator_2ef8d"
    bl_label = "Lazy Bones Bone Roll Global Z+ Operator"
    bl_description = "Sets the bones roll of all bones in selected armatures to +Z (World Space)"
    bl_options = {"REGISTER", "UNDO"}

    @classmethod
    def poll(cls, context):
        if bpy.app.version >= (3, 0, 0) and True:
            cls.poll_message_set('')
        return not False

    def execute(self, context):
        sna_lazy_bones_set_bone_roll_func_7F104('GLOBAL_POS_Z', False)
        return {"FINISHED"}

    def invoke(self, context, event):
        return self.execute(context)


class SNA_OT_Lazy_Bones_Bone_Roll_Global_X_Operator_Eaa28(bpy.types.Operator):
    bl_idname = "sna.lazy_bones_bone_roll_global_x_operator_eaa28"
    bl_label = "Lazy Bones Bone Roll Global X- Operator"
    bl_description = "Sets the bones roll of all bones in selected armatures to -X (World Space)"
    bl_options = {"REGISTER", "UNDO"}

    @classmethod
    def poll(cls, context):
        if bpy.app.version >= (3, 0, 0) and True:
            cls.poll_message_set('')
        return not False

    def execute(self, context):
        sna_lazy_bones_set_bone_roll_func_7F104('GLOBAL_NEG_X', False)
        return {"FINISHED"}

    def invoke(self, context, event):
        return self.execute(context)


class SNA_OT_Lazy_Bones_Bone_Roll_Global_Y_Operator_96065(bpy.types.Operator):
    bl_idname = "sna.lazy_bones_bone_roll_global_y_operator_96065"
    bl_label = "Lazy Bones Bone Roll Global Y- Operator"
    bl_description = "Sets the bones roll of all bones in selected armatures to -Y (World Space)"
    bl_options = {"REGISTER", "UNDO"}

    @classmethod
    def poll(cls, context):
        if bpy.app.version >= (3, 0, 0) and True:
            cls.poll_message_set('')
        return not False

    def execute(self, context):
        sna_lazy_bones_set_bone_roll_func_7F104('GLOBAL_NEG_Y', False)
        return {"FINISHED"}

    def invoke(self, context, event):
        return self.execute(context)


class SNA_OT_Lazy_Bones_Bone_Roll_Local_Z_Operator_Dc8D7(bpy.types.Operator):
    bl_idname = "sna.lazy_bones_bone_roll_local_z_operator_dc8d7"
    bl_label = "Lazy Bones Bone Roll Local Z- Operator"
    bl_description = "Sets the bones roll of all bones in selected armatures to -Z (Local Space)"
    bl_options = {"REGISTER", "UNDO"}

    @classmethod
    def poll(cls, context):
        if bpy.app.version >= (3, 0, 0) and True:
            cls.poll_message_set('')
        return not False

    def execute(self, context):
        sna_lazy_bones_set_bone_roll_func_7F104('NEG_Z', False)
        return {"FINISHED"}

    def invoke(self, context, event):
        return self.execute(context)


class SNA_OT_Lazy_Bones_Bone_Roll_Local_X_Operator_10200(bpy.types.Operator):
    bl_idname = "sna.lazy_bones_bone_roll_local_x_operator_10200"
    bl_label = "Lazy Bones Bone Roll Local X+ Operator"
    bl_description = "Sets the bones roll of all bones in selected armatures to +X (Local Space)"
    bl_options = {"REGISTER", "UNDO"}

    @classmethod
    def poll(cls, context):
        if bpy.app.version >= (3, 0, 0) and True:
            cls.poll_message_set('')
        return not False

    def execute(self, context):
        sna_lazy_bones_set_bone_roll_func_7F104('POS_X', False)
        return {"FINISHED"}

    def invoke(self, context, event):
        return self.execute(context)


class SNA_OT_Lazy_Bones_Bone_Roll_Local_X_Operator_D57Fd(bpy.types.Operator):
    bl_idname = "sna.lazy_bones_bone_roll_local_x_operator_d57fd"
    bl_label = "Lazy Bones Bone Roll Local X- Operator"
    bl_description = "Sets the bones roll of all bones in selected armatures to -X (Local Space)"
    bl_options = {"REGISTER", "UNDO"}

    @classmethod
    def poll(cls, context):
        if bpy.app.version >= (3, 0, 0) and True:
            cls.poll_message_set('')
        return not False

    def execute(self, context):
        sna_lazy_bones_set_bone_roll_func_7F104('NEG_X', False)
        return {"FINISHED"}

    def invoke(self, context, event):
        return self.execute(context)


class SNA_OT_Lazy_Bones_Bone_Roll_Global_X_Operator_168Ea(bpy.types.Operator):
    bl_idname = "sna.lazy_bones_bone_roll_global_x_operator_168ea"
    bl_label = "Lazy Bones Bone Roll Global X+ Operator"
    bl_description = "Sets the bones roll of all bones in selected armatures to +X (World Space)"
    bl_options = {"REGISTER", "UNDO"}

    @classmethod
    def poll(cls, context):
        if bpy.app.version >= (3, 0, 0) and True:
            cls.poll_message_set('')
        return not False

    def execute(self, context):
        sna_lazy_bones_set_bone_roll_func_7F104('GLOBAL_POS_X', False)
        return {"FINISHED"}

    def invoke(self, context, event):
        return self.execute(context)


class SNA_OT_Lazy_Bones_Bone_Roll_Local_Z_Operator_C377A(bpy.types.Operator):
    bl_idname = "sna.lazy_bones_bone_roll_local_z_operator_c377a"
    bl_label = "Lazy Bones Bone Roll Local Z+ Operator"
    bl_description = "Sets the bones roll of all bones in selected armatures to +Z (Local Space)"
    bl_options = {"REGISTER", "UNDO"}

    @classmethod
    def poll(cls, context):
        if bpy.app.version >= (3, 0, 0) and True:
            cls.poll_message_set('')
        return not False

    def execute(self, context):
        sna_lazy_bones_set_bone_roll_func_7F104('POS_Z', False)
        return {"FINISHED"}

    def invoke(self, context, event):
        return self.execute(context)


def sna_lazy_bones_set_bone_roll_func_7F104(Type, Axis_Flip):
    if (bpy.context.view_layer.objects.active == None):
        pass
    else:
        if bpy.context.view_layer.objects.active.type == 'ARMATURE':
            bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='EDIT', toggle=False)
            bpy.ops.armature.select_all('INVOKE_DEFAULT', action='SELECT')
            bpy.ops.armature.calculate_roll('INVOKE_DEFAULT', type=Type, axis_flip=Axis_Flip, axis_only=False)
            bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='OBJECT', toggle=False)


def sna_lazy_bone_show_bone_axis_func_2C119(showhide_bone_axis):
    if (len(bpy.context.view_layer.objects.selected) > 0):
        for i_7AD9C in range(len(bpy.context.view_layer.objects.selected)):
            if bpy.context.view_layer.objects.selected[i_7AD9C].type == 'ARMATURE':
                bpy.context.view_layer.objects.selected[i_7AD9C].data.show_axes = showhide_bone_axis


class SNA_OT_Lazy_Bones_Show_Bone_Axis_Operator_026A7(bpy.types.Operator):
    bl_idname = "sna.lazy_bones_show_bone_axis_operator_026a7"
    bl_label = "Lazy Bones Show Bone Axis Operator"
    bl_description = "Displays bone axes for selected armatures."
    bl_options = {"REGISTER", "UNDO"}

    @classmethod
    def poll(cls, context):
        if bpy.app.version >= (3, 0, 0) and True:
            cls.poll_message_set('')
        return not False

    def execute(self, context):
        sna_lazy_bone_show_bone_axis_func_2C119(True)
        return {"FINISHED"}

    def invoke(self, context, event):
        return self.execute(context)


class SNA_OT_Lazy_Bones_Dissolve_By_Fraction_Operator_Df537(bpy.types.Operator):
    bl_idname = "sna.lazy_bones_dissolve_by_fraction_operator_df537"
    bl_label = "Lazy Bones Dissolve by Fraction Operator"
    bl_description = "Dissolves every Nth bone for the selected armatures. (Dissolve by 2 halves the number of bones)"
    bl_options = {"REGISTER", "UNDO"}

    @classmethod
    def poll(cls, context):
        if bpy.app.version >= (3, 0, 0) and True:
            cls.poll_message_set('')
        return not False

    def execute(self, context):
        bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='EDIT')
        for i_AA0E5 in range(len(bpy.context.view_layer.objects.selected)):
            if (bpy.context.view_layer.objects.active == None):
                pass
            else:
                if bpy.context.view_layer.objects.selected[i_AA0E5].type == 'ARMATURE':
                    bpy.ops.armature.select_all('INVOKE_DEFAULT', action='DESELECT')
                    for i_D20A1 in range(len(bpy.context.view_layer.objects.selected[i_AA0E5].data.edit_bones)):
                        bpy.context.view_layer.objects.selected[i_AA0E5].data.edit_bones[i_D20A1].select_tail = (int(eval("i_D20A1 % bpy.context.scene.sna_lb_dissolveby")) == 0)
                    bpy.ops.armature.dissolve('INVOKE_DEFAULT', )
        bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='OBJECT', toggle=False)
        return {"FINISHED"}

    def invoke(self, context, event):
        return self.execute(context)


class SNA_OT_Lazy_Bones_Switch_Bone_Direction_Operator_10514(bpy.types.Operator):
    bl_idname = "sna.lazy_bones_switch_bone_direction_operator_10514"
    bl_label = "Lazy Bones Switch Bone Direction Operator"
    bl_description = "Switches the direction of all bones in selected armatures."
    bl_options = {"REGISTER", "UNDO"}

    @classmethod
    def poll(cls, context):
        if bpy.app.version >= (3, 0, 0) and True:
            cls.poll_message_set('')
        return not False

    def execute(self, context):
        if (bpy.context.view_layer.objects.active == None):
            self.report({'ERROR'}, message='No active object.')
        else:
            if bpy.context.view_layer.objects.active.type == 'ARMATURE':
                if (len(bpy.context.view_layer.objects.selected) > 0):
                    sna_lazy_bones_switch_direction_func_80DCF()
        return {"FINISHED"}

    def invoke(self, context, event):
        return self.execute(context)


def sna_lazy_bones_switch_direction_func_80DCF():
    for i_7DF14 in range(len(bpy.context.view_layer.objects.selected)):
        if bpy.context.view_layer.objects.selected[i_7DF14].type == 'ARMATURE':
            bpy.context.view_layer.objects.active = bpy.context.view_layer.objects.selected[i_7DF14]
    bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='EDIT')
    bpy.ops.armature.select_all('INVOKE_DEFAULT', action='SELECT')
    bpy.ops.armature.switch_direction('INVOKE_DEFAULT', )
    bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='OBJECT')


def sna_lazy_bones_bone_target_indiv_func_AA6CD(Bone_Target_Vector, Force_Bone_Direction):
    for i_0DC59 in range(len(bpy.context.view_layer.objects.selected)):
        if bpy.context.view_layer.objects.selected[i_0DC59].type == 'ARMATURE':
            bpy.context.view_layer.objects.selected[i_0DC59].select_set(state=True, view_layer=bpy.context.view_layer, )
            bpy.context.view_layer.objects.active = bpy.context.view_layer.objects.selected[i_0DC59]
            bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='EDIT')
            for i_EAB15 in range(len(bpy.context.view_layer.objects.selected[i_0DC59].data.edit_bones)):
                if (len(bpy.context.view_layer.objects.selected[i_0DC59].data.edit_bones) > 0):
                    bpy.ops.armature.select_all('INVOKE_DEFAULT', action='DESELECT')
                    bpy.context.view_layer.objects.selected[i_0DC59].data.edit_bones[i_EAB15].select = True
                    bonenamevar = bpy.context.view_layer.objects.selected[i_0DC59].data.edit_bones[i_EAB15].name
                    targetlocvar = Bone_Target_Vector
                    thresholdvar = 90
                    is_pointing = None
                    import math

                    def is_edit_bone_pointing_at_target(bone_name, target_location, threshold_angle_degrees):
                        obj = bpy.context.object
                        armature = obj.data  # Access the armature's data
                        edit_bone = armature.edit_bones[bone_name]  # Access the edit bone
                        # Get the vector from the bone's head to the target
                        bone_to_target = target_location - obj.matrix_world @ edit_bone.head  # Convert bone head to world space
                        # Get the vector representing the direction of the bone
                        bone_direction = obj.matrix_world @ edit_bone.tail - obj.matrix_world @ edit_bone.head  # Convert bone tail to world space
                        # Normalize the vectors
                        bone_to_target.normalize()
                        bone_direction.normalize()
                        # Calculate the dot product
                        dot_product = bone_to_target.dot(bone_direction)
                        # Ensure dot product is within the valid range
                        dot_product = min(max(dot_product, -1), 1)
                        # Calculate the angle in radians
                        angle = math.acos(dot_product)
                        # Convert angle to degrees and check against the threshold
                        angle_degrees = math.degrees(angle)
                        is_pointing_at_target = angle_degrees <= threshold_angle_degrees
                        return is_pointing_at_target
                    # Example usage with input variables
                    bone_name_input = bonenamevar
                    target_location_input = mathutils.Vector((targetlocvar))  # Replace with your target location as a float vector
                    threshold_angle_degrees_input = thresholdvar  # Threshold angle in degrees
                    is_pointing = is_edit_bone_pointing_at_target(bone_name_input, target_location_input, threshold_angle_degrees_input)
                    print(f"The edit bone '{bone_name_input}' is pointing at the target: {is_pointing}")
                    if (Force_Bone_Direction == 'Point towards target'):
                        if is_pointing:
                            pass
                        else:
                            bpy.ops.armature.switch_direction('INVOKE_DEFAULT', )
                    else:
                        if is_pointing:
                            bpy.ops.armature.switch_direction('INVOKE_DEFAULT', )
            bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='OBJECT')


def sna_lazy_bones_bone_target_whole_func_2AE3D(Bone_Target_Vector, Force_Bone_Direction):
    if (bpy.context.view_layer.objects.active == None):
        pass
    else:
        for i_4D5C4 in range(len(bpy.context.view_layer.objects.selected)):
            if bpy.context.view_layer.objects.selected[i_4D5C4].type == 'ARMATURE':
                lazy_bone_bone_target_serpens['sna_lazyboneselectionsettempvar'].append(bpy.context.view_layer.objects.selected[i_4D5C4].name)
        for i_37059 in range(len(lazy_bone_bone_target_serpens['sna_lazyboneselectionsettempvar'])):
            bpy.ops.object.select_all('INVOKE_DEFAULT', action='DESELECT')
            bpy.data.objects[lazy_bone_bone_target_serpens['sna_lazyboneselectionsettempvar'][i_37059]].select_set(state=True, view_layer=bpy.context.view_layer, )
            bpy.context.view_layer.objects.active = bpy.data.objects[lazy_bone_bone_target_serpens['sna_lazyboneselectionsettempvar'][i_37059]]
            targetlocvar = Bone_Target_Vector
            thresholdvar = 90
            is_pointing = None
            import math

            def is_bone_pointing_at_target(target_location, threshold_angle_degrees):
                obj = bpy.context.object
                armature = obj.data  # Access the armature's data
                # Check if the armature has bones
                if not armature.bones:
                    print("Armature has no bones.")
                    return False
                # List to store pointing results for each bone
                pointing_results = []
                # Iterate over all bones in the armature
                for bone in armature.bones:
                    # Get the vector from the bone's head to the target
                    bone_head_world = obj.matrix_world @ bone.head_local
                    bone_to_target = target_location - bone_head_world
                    # Get the vector representing the direction of the bone
                    bone_tail_world = obj.matrix_world @ bone.tail_local
                    bone_direction = bone_tail_world - bone_head_world
                    # Normalize the vectors
                    bone_to_target.normalize()
                    bone_direction.normalize()
                    # Calculate the dot product
                    dot_product = bone_to_target.dot(bone_direction)
                    # Ensure dot product is within the valid range
                    dot_product = min(max(dot_product, -1), 1)
                    # Calculate the angle in radians
                    angle = math.acos(dot_product)
                    # Convert angle to degrees and check against the threshold
                    angle_degrees = math.degrees(angle)
                    is_pointing_at_target = angle_degrees <= threshold_angle_degrees
                    # Append the pointing result to the list
                    pointing_results.append(is_pointing_at_target)
                # Calculate the total number of True and False results
                num_true = sum(pointing_results)
                num_false = len(pointing_results) - num_true
                # Print the total number of True and False results
                print(f"Total True results: {num_true}")
                print(f"Total False results: {num_false}")
                # Calculate the average of the pointing results
                average_pointing = sum(pointing_results) / len(pointing_results)
                # If the average is greater than or equal to 0.5, consider it True; otherwise, False
                is_pointing = average_pointing >= 0.5
                return is_pointing
            # Input Variables
            target_location = mathutils.Vector((targetlocvar))  # Replace with your target location
            threshold_angle_degrees = thresholdvar
            is_pointing = is_bone_pointing_at_target(target_location, threshold_angle_degrees)
            print(f"At least half of the bones are pointing at the target: {is_pointing}")
            if (Force_Bone_Direction == 'Point towards target'):
                if is_pointing:
                    pass
                else:
                    sna_lazy_bones_switch_direction_func_80DCF()
            else:
                if is_pointing:
                    sna_lazy_bones_switch_direction_func_80DCF()
        for i_6B779 in range(len(lazy_bone_bone_target_serpens['sna_lazyboneselectionsettempvar'])):
            bpy.data.objects[lazy_bone_bone_target_serpens['sna_lazyboneselectionsettempvar'][i_6B779]].select_set(state=True, view_layer=bpy.context.view_layer, )
        lazy_bone_bone_target_serpens['sna_lazyboneselectionsettempvar'] = []


def sna_lazy_bones_make_outliner_collection_func_3888C(CollectionName):
    if (property_exists("bpy.data.collections", globals(), locals()) and CollectionName in bpy.data.collections):
        pass
    else:
        collection_7033B = bpy.data.collections.new(name=CollectionName, )
        bpy.context.scene.collection.children.link(child=collection_7033B, )
        bpy.data.collections[CollectionName].color_tag = 'COLOR_06'


class SNA_MT_414F8(bpy.types.Menu):
    bl_idname = "SNA_MT_414F8"
    bl_label = ""

    @classmethod
    def poll(cls, context):
        return not (False)

    def draw(self, context):
        layout = self.layout.column_flow(columns=1)
        layout.operator_context = "INVOKE_DEFAULT"
        op = layout.operator('sna.edge2bone_2cf72', text='Edge-2-Bone', icon_value=174, emboss=True, depress=False)
        op.sna_decimate_angle_limit = 0.0
        op.sna_join_created_armatures = False
        op.sna_remove_intersections = True
        op.sna_hide_base_object = True


class SNA_OT_Edge2Bone_2Cf72(bpy.types.Operator):
    bl_idname = "sna.edge2bone_2cf72"
    bl_label = "Edge-2-Bone"
    bl_description = "Converts selected edges into armature bones."
    bl_options = {"REGISTER", "UNDO"}
    sna_decimate_angle_limit: bpy.props.FloatProperty(name='Decimate Angle Limit', description='', default=0.0, subtype='NONE', unit='NONE', min=0.0, step=3, precision=2)
    sna_join_created_armatures: bpy.props.BoolProperty(name='Join created armatures?', description='', default=True)
    sna_remove_intersections: bpy.props.BoolProperty(name='Remove intersections?', description='', default=False)
    sna_hide_base_object: bpy.props.BoolProperty(name='Hide base object?', description='', default=False)

    @classmethod
    def poll(cls, context):
        if bpy.app.version >= (3, 0, 0) and True:
            cls.poll_message_set('')
        return not False

    def execute(self, context):
        if (bpy.context.view_layer.objects.active == None):
            pass
        else:
            if (len(bpy.context.view_layer.objects.selected) > 1):
                self.report({'WARNING'}, message='Only select one object please')
            else:
                result_91194 = bpy.context.view_layer.objects.active.data.count_selected_items()
                if (result_91194[1] == 0):
                    self.report({'WARNING'}, message='No edges selected!')
                else:
                    bpy.context.scene.sna_e2bbaseedgecount = result_91194[1]
                    if False:
                        purge_count = 0
                        for a in range(len(bpy.data.actions)-1,-1,-1):
                            if (bpy.data.actions[a].users == 0):
                                bpy.data.actions.remove(action=bpy.data.actions[a], )
                                purge_count += 1
                    if True:
                        purge_count = 0
                        for ar in range(len(bpy.data.armatures)-1,-1,-1):
                            if (bpy.data.armatures[ar].users == 0):
                                bpy.data.armatures.remove(armature=bpy.data.armatures[ar], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for b in range(len(bpy.data.brushes)-1,-1,-1):
                            if (bpy.data.brushes[b].users == 0):
                                bpy.data.brushes.remove(brush=bpy.data.brushes[b], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for cf in range(len(bpy.data.cache_files)-1,-1,-1):
                            if (bpy.data.cache_files[cf].users == 0):
                                bpy.data.cache_files.remove(cache_file=bpy.data.brushes[cf], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for c in range(len(bpy.data.cameras)-1,-1,-1):
                            if (bpy.data.cameras[c].users == 0):
                                bpy.data.cameras.remove(camera=bpy.data.cameras[c], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for c in range(len(bpy.data.collections)-1,-1,-1):
                            if (bpy.data.collections[c].users == 0):
                                bpy.data.collections.remove(collection=bpy.data.collections[c], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for c in range(len(bpy.data.curves)-1,-1,-1):
                            if (bpy.data.curves[c].users == 0):
                                bpy.data.curves.remove(curve=bpy.data.curves[c], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for hc in range(len(bpy.data.hair_curves)-1,-1,-1):
                            if (bpy.data.hair_curves[hc].users == 0):
                                bpy.data.hair_curves.remove(curves=bpy.data.hair_curves[hc], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for f in range(len(bpy.data.fonts)-1,-1,-1):
                            if (bpy.data.fonts[f].users == 0):
                                bpy.data.fonts.remove(font=bpy.data.fonts[f], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for gp in range(len(bpy.data.grease_pencils)-1,-1,-1):
                            if (bpy.data.grease_pencils[gp].users == 0):
                                bpy.data.grease_pencils.remove(grease_pencil=bpy.data.grease_pencils[gp], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for i in range(len(bpy.data.images)-1,-1,-1):
                            if (bpy.data.images[i].users == 0):
                                bpy.data.images.remove(image=bpy.data.images[i], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for k in range(len(bpy.data.shape_keys)-1,-1,-1):
                            if (bpy.data.shape_keys[k].users == 0):
                                bpy.data.shape_keys.remove(key=bpy.data.shape_keys[k], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for l in range(len(bpy.data.lattices)-1,-1,-1):
                            if (bpy.data.lattices[l].users == 0):
                                bpy.data.lattices.remove(lattice=bpy.data.lattices[l], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for l in range(len(bpy.data.libraries)-1,-1,-1):
                            if (bpy.data.libraries[l].users == 0):
                                bpy.data.libraries.remove(library=bpy.data.libraries[l], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for l in range(len(bpy.data.lights)-1,-1,-1):
                            if (bpy.data.lights[l].users == 0):
                                bpy.data.lights.remove(light=bpy.data.lights[l], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for l in range(len(bpy.data.lightprobes)-1,-1,-1):
                            if (bpy.data.lightprobes[l].users == 0):
                                bpy.data.lightprobes.remove(lightprobe=bpy.data.lightprobes[l], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for l in range(len(bpy.data.linestyles)-1,-1,-1):
                            if (bpy.data.linestyles[l].users == 0):
                                bpy.data.linestyles.remove(linestyle=bpy.data.linestyles[l], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for m in range(len(bpy.data.masks)-1,-1,-1):
                            if (bpy.data.masks[m].users == 0):
                                bpy.data.masks.remove(mask=bpy.data.masks[m], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for m in range(len(bpy.data.materials)-1,-1,-1):
                            if (bpy.data.materials[m].users == 0):
                                bpy.data.materials.remove(material=bpy.data.materials[m], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for m in range(len(bpy.data.meshes)-1,-1,-1):
                            if (bpy.data.meshes[m].users == 0):
                                bpy.data.meshes.remove(mesh=bpy.data.meshes[m], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for m in range(len(bpy.data.metaballs)-1,-1,-1):
                            if (bpy.data.metaballs[m].users == 0):
                                bpy.data.metaballs.remove(metaball=bpy.data.metaballs[m], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for m in range(len(bpy.data.movieclips)-1,-1,-1):
                            if (bpy.data.movieclips[m].users == 0):
                                bpy.data.movieclips.remove(movieclip=bpy.data.movieclips[m], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for ng in range(len(bpy.data.node_groups)-1,-1,-1):
                            if (bpy.data.node_groups[ng].users == 0):
                                bpy.data.node_groups.remove(tree=bpy.data.node_groups[ng], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for o in range(len(bpy.data.objects)-1,-1,-1):
                            if (bpy.data.objects[o].users == 0):
                                bpy.data.objects.remove(object=bpy.data.objects[o], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for pc in range(len(bpy.data.paint_curves)-1,-1,-1):
                            if (bpy.data.paint_curves[pc].users == 0):
                                bpy.data.paint_curves.remove(paint_curve=bpy.data.paint_curves[pc], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for p in range(len(bpy.data.palettes)-1,-1,-1):
                            if (bpy.data.palettes[p].users == 0):
                                bpy.data.palettes.remove(palette=bpy.data.palettes[p], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for p in range(len(bpy.data.particles)-1,-1,-1):
                            if (bpy.data.particles[p].users == 0):
                                bpy.data.particles.remove(particle=bpy.data.particles[p], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for p in range(len(bpy.data.pointclouds)-1,-1,-1):
                            if (bpy.data.pointclouds[p].users == 0):
                                bpy.data.pointclouds.remove(pointcloud=bpy.data.pointclouds[p], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for s in range(len(bpy.data.scenes)-1,-1,-1):
                            if (bpy.data.scenes[s].users == 0):
                                bpy.data.scenes.remove(scene=bpy.data.scenes[s], )
                                purge_count += 1
                    # if False:
                    #     purge_count = 0
                    #     for s in range(len(bpy.data.simulations)-1,-1,-1):
                    #         if (bpy.data.simulations[s].users == 0):
                    #             bpy.data.simulations.remove(simulation=bpy.data.simulations[s], )
                    #             purge_count += 1
                    if False:
                        purge_count = 0
                        for s in range(len(bpy.data.sounds)-1,-1,-1):
                            if (bpy.data.sounds[s].users == 0):
                                bpy.data.sounds.remove(sound=bpy.data.sounds[s], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for s in range(len(bpy.data.speakers)-1,-1,-1):
                            if (bpy.data.speakers[s].users == 0):
                                bpy.data.speakers.remove(speaker=bpy.data.speakers[s], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for txt in range(len(bpy.data.texts)-1,-1,-1):
                            if (bpy.data.texts[txt].users == 0):
                                bpy.data.texts.remove(text=bpy.data.texts[txt], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for tex in range(len(bpy.data.textures)-1,-1,-1):
                            if (bpy.data.textures[tex].users == 0):
                                bpy.data.textures.remove(texture=bpy.data.textures[tex], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for v in range(len(bpy.data.volumes)-1,-1,-1):
                            if (bpy.data.volumes[v].users == 0):
                                bpy.data.volumes.remove(volume=bpy.data.volumes[v], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for wm in range(len(bpy.data.window_managers)-1,-1,-1):
                            if (bpy.data.window_managers[wm].users == 0):
                                bpy.data.window_managers.remove(window_manager=bpy.data.window_managers[wm], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for ws in range(len(bpy.data.workspaces)-1,-1,-1):
                            if (bpy.data.workspaces[ws].users == 0):
                                bpy.data.workspaces.remove(workspace=bpy.data.workspaces[ws], )
                                purge_count += 1
                    if False:
                        purge_count = 0
                        for w in range(len(bpy.data.worlds)-1,-1,-1):
                            if (bpy.data.worlds[w].users == 0):
                                bpy.data.worlds.remove(world=bpy.data.worlds[w], )
                                purge_count += 1
                    bpy.context.scene.sna_e2bcomponentscollection.clear()
                    bpy.context.scene.sna_e2barmaturescollection.clear()
                    bpy.context.scene.sna_e2bbaseobject = bpy.context.view_layer.objects.active
                    sna_lazy_bones_make_outliner_collection_func_3888C('Lazy_Bone_Armatures')
                    bpy.context.view_layer.layer_collection.children['Lazy_Bone_Armatures'].exclude = False
                    bpy.ops.mesh.duplicate('INVOKE_DEFAULT', mode=1)
                    bpy.ops.mesh.separate('INVOKE_DEFAULT', type='SELECTED')
                    bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='OBJECT', toggle=True)
                    bpy.context.scene.sna_e2bbaseobject.select_set(state=False, view_layer=bpy.context.view_layer, )
                    bpy.context.view_layer.objects.selected[0].select_set(state=True, view_layer=bpy.context.view_layer, )
                    bpy.context.view_layer.objects.active = bpy.context.view_layer.objects.selected[0]
                    if self.sna_remove_intersections:
                        sna_add_lazy_bones_remove_intersections_modifier_func_3F946()
                        bpy.context.view_layer.objects.active.select_set(state=True, view_layer=bpy.context.view_layer, )
                        bpy.ops.object.convert('INVOKE_DEFAULT', target='MESH')
                        bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='EDIT', toggle=True)
                        bpy.ops.mesh.select_all('INVOKE_DEFAULT', action='SELECT')
                        bpy.ops.mesh.unsubdivide('INVOKE_DEFAULT', iterations=2)
                        bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='OBJECT', toggle=True)
                    bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='EDIT', toggle=True)
                    bpy.ops.mesh.separate('INVOKE_DEFAULT', type='LOOSE')
                    bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='OBJECT', toggle=True)
                    bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='EDIT', toggle=True)
                    bpy.ops.mesh.select_all('INVOKE_DEFAULT', action='SELECT')
                    bpy.ops.mesh.dissolve_limited('INVOKE_DEFAULT', angle_limit=math.radians(self.sna_decimate_angle_limit), use_dissolve_boundaries=False, delimit={'NORMAL'})
                    bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='OBJECT', toggle=True)
                    for i_56468 in range(len(bpy.context.view_layer.objects.selected)):
                        item_72C03 = bpy.context.scene.sna_e2bcomponentscollection.add()
                        item_72C03.e2bcomponents = bpy.context.view_layer.objects.selected[i_56468]
                        item_72C03.e2bcomponentedgecount = len(bpy.context.view_layer.objects.selected[i_56468].data.edges)
                    for i_C4A2D in range(len(bpy.context.scene.sna_e2bcomponentscollection)):
                        bpy.ops.object.select_all('INVOKE_DEFAULT', action='DESELECT')
                        bpy.context.scene.sna_e2bcomponentscollection[i_C4A2D].e2bcomponents.select_set(state=True, view_layer=bpy.context.view_layer, )
                        bpy.context.view_layer.objects.active = bpy.context.scene.sna_e2bcomponentscollection[i_C4A2D].e2bcomponents
                        bpy.ops.object.convert('INVOKE_DEFAULT', target='CURVE')
                        bpy.ops.object.convert('INVOKE_DEFAULT', target='MESH')
                    for i_8EA61 in range(len(bpy.context.scene.sna_e2bcomponentscollection)):
                        bpy.ops.object.select_all('INVOKE_DEFAULT', action='DESELECT')
                        bpy.context.scene.sna_e2bcomponentscollection[i_8EA61].e2bcomponents.select_set(state=True, view_layer=bpy.context.view_layer, )
                        bpy.context.view_layer.objects.active = bpy.context.scene.sna_e2bcomponentscollection[i_8EA61].e2bcomponents
                        bpy.ops.object.modifier_add('INVOKE_DEFAULT', type='SKIN')
                        bpy.ops.object.skin_armature_create('INVOKE_DEFAULT', modifier='Skin')
                        item_14E12 = bpy.context.scene.sna_e2barmaturescollection.add()
                        item_14E12.e2barmatures = bpy.context.view_layer.objects.active
                        bpy.context.view_layer.objects.active.data.display_type = 'OCTAHEDRAL'
                        bpy.context.view_layer.objects.active.name = 'E2B_Armature'
                        bpy.context.active_object.data.name = bpy.context.active_object.name
                    for i_07E49 in range(len(bpy.context.scene.sna_e2barmaturescollection)):
                        bpy.ops.object.select_all('INVOKE_DEFAULT', action='DESELECT')
                        bpy.context.scene.sna_e2barmaturescollection[i_07E49].e2barmatures.select_set(state=True, view_layer=bpy.context.view_layer, )
                        bpy.context.view_layer.objects.active = bpy.context.scene.sna_e2barmaturescollection[i_07E49].e2barmatures
                        if (bpy.context.scene.sna_e2bcomponentscollection[i_07E49].e2bcomponentedgecount == len(bpy.context.scene.sna_e2barmaturescollection[i_07E49].e2barmatures.data.bones)):
                            pass
                        else:
                            bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='EDIT', toggle=True)
                            bpy.ops.armature.select_all('INVOKE_DEFAULT', action='DESELECT')
                            bpy.context.scene.sna_e2barmaturescollection[i_07E49].e2barmatures.data.edit_bones[0].select = True
                            bpy.ops.armature.delete('INVOKE_DEFAULT', confirm=False)
                            bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='OBJECT', toggle=True)
                    for i_3A03A in range(len(bpy.context.scene.sna_e2barmaturescollection)):
                        bpy.ops.object.select_all('INVOKE_DEFAULT', action='DESELECT')
                        bpy.context.scene.sna_e2barmaturescollection[i_3A03A].e2barmatures.select_set(state=True, view_layer=bpy.context.view_layer, )
                        bpy.context.view_layer.objects.active = bpy.context.scene.sna_e2barmaturescollection[i_3A03A].e2barmatures
                        if ((bpy.context.scene.sna_e2bbaseedgecount == 2) and (len(bpy.context.scene.sna_e2bcomponentscollection) == 1)):
                            pass
                        else:
                            if (bpy.context.scene.sna_e2bcomponentscollection[i_3A03A].e2bcomponentedgecount == 2):
                                bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='EDIT', toggle=True)
                                bpy.ops.armature.select_all('INVOKE_DEFAULT', action='SELECT')
                                bpy.ops.armature.dissolve('INVOKE_DEFAULT', )
                                bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='OBJECT', toggle=True)
                    for i_C4100 in range(len(bpy.context.scene.sna_e2bcomponentscollection)):
                        bpy.ops.object.select_all('INVOKE_DEFAULT', action='DESELECT')
                        bpy.context.scene.sna_e2bcomponentscollection[i_C4100].e2bcomponents.select_set(state=True, view_layer=bpy.context.view_layer, )
                        bpy.context.view_layer.objects.active = bpy.context.scene.sna_e2bcomponentscollection[i_C4100].e2bcomponents
                        bpy.ops.object.delete('INVOKE_DEFAULT', use_global=True, confirm=False)
                    for i_DB504 in range(len(bpy.context.scene.sna_e2barmaturescollection)):
                        bpy.context.collection.objects.unlink(object=bpy.context.scene.sna_e2barmaturescollection[i_DB504].e2barmatures, )
                        bpy.data.collections['Lazy_Bone_Armatures'].objects.link(object=bpy.context.scene.sna_e2barmaturescollection[i_DB504].e2barmatures, )
                    for i_9213F in range(len(bpy.context.scene.sna_e2barmaturescollection)):
                        bpy.context.scene.sna_e2barmaturescollection[i_9213F].e2barmatures.select_set(state=True, view_layer=bpy.context.view_layer, )
                        bpy.context.view_layer.objects.active = bpy.context.scene.sna_e2barmaturescollection[i_9213F].e2barmatures
                    sna_lazy_bones_bone_target_whole_func_2AE3D(bpy.context.scene.cursor.location, 'Point away from target')
                    if self.sna_join_created_armatures:
                        bpy.ops.object.join('INVOKE_DEFAULT', )
                    if self.sna_hide_base_object:
                        bpy.context.scene.sna_e2bbaseobject.hide_set(state=True, view_layer=bpy.context.view_layer, )
                        bpy.context.scene.sna_e2bcomponentscollection.clear()
                        bpy.context.scene.sna_e2barmaturescollection.clear()
        return {"FINISHED"}

    def draw(self, context):
        layout = self.layout
        box_FC013 = layout.box()
        box_FC013.alert = False
        box_FC013.enabled = True
        box_FC013.active = True
        box_FC013.use_property_split = False
        box_FC013.use_property_decorate = False
        box_FC013.alignment = 'Expand'.upper()
        box_FC013.scale_x = 1.0
        box_FC013.scale_y = 1.0
        if not True: box_FC013.operator_context = "EXEC_DEFAULT"
        box_FC013.template_icon(icon_value=load_preview_icon(os.path.join(os.path.dirname(__file__), 'assets', 'Edge 2 Bone Icon.png')), scale=5.0)
        box_8CE5D = box_FC013.box()
        box_8CE5D.alert = False
        box_8CE5D.enabled = True
        box_8CE5D.active = True
        box_8CE5D.use_property_split = False
        box_8CE5D.use_property_decorate = False
        box_8CE5D.alignment = 'Expand'.upper()
        box_8CE5D.scale_x = 1.0
        box_8CE5D.scale_y = 1.0
        if not True: box_8CE5D.operator_context = "EXEC_DEFAULT"
        box_8CE5D.prop(self, 'sna_hide_base_object', text='Hide base object?', icon_value=254, emboss=True)
        box_8CE5D.prop(self, 'sna_decimate_angle_limit', text='Decimate Angle Limit', icon_value=293, emboss=True)
        box_8CE5D.prop(self, 'sna_join_created_armatures', text='Join created armatures?', icon_value=0, emboss=True)
        if bpy.context.preferences.addons['lazy_bones'].preferences.sna_lazy_bones_show_tips:
            if self.sna_join_created_armatures:
                box_7E76B = box_8CE5D.box()
                box_7E76B.alert = True
                box_7E76B.enabled = True
                box_7E76B.active = True
                box_7E76B.use_property_split = False
                box_7E76B.use_property_decorate = False
                box_7E76B.alignment = 'Expand'.upper()
                box_7E76B.scale_x = 1.0
                box_7E76B.scale_y = 1.0
                if not True: box_7E76B.operator_context = "EXEC_DEFAULT"
                box_7E76B.label(text='Joining the armatures will prevent some clean up options', icon_value=4)
        box_8CE5D.prop(self, 'sna_remove_intersections', text='Remove intersections?', icon_value=0, emboss=True)
        if bpy.context.preferences.addons['lazy_bones'].preferences.sna_lazy_bones_show_tips:
            box_C2677 = box_8CE5D.box()
            box_C2677.alert = True
            box_C2677.enabled = True
            box_C2677.active = True
            box_C2677.use_property_split = False
            box_C2677.use_property_decorate = False
            box_C2677.alignment = 'Expand'.upper()
            box_C2677.scale_x = 1.0
            box_C2677.scale_y = 1.0
            if not True: box_C2677.operator_context = "EXEC_DEFAULT"
            box_C2677.label(text='Bones will attempt to point away from the 3d cursor', icon_value=4)
            box_C2677.label(text='This can be changed during clean up', icon_value=4)

    def invoke(self, context, event):
        return context.window_manager.invoke_props_dialog(self, width=400)


def sna_add_to_view3d_mt_edit_mesh_context_menu_325A3(self, context):
    if not (False):
        layout = self.layout
        if bpy.context.tool_settings.mesh_select_mode[1]:
            layout.menu('SNA_MT_414F8', text='Lazy Bones', icon_value=200)


def sna_add_to_view3d_mt_object_context_menu_5E1DB(self, context):
    if not (((not bpy.context.view_layer.objects.active.type == 'LATTICE') if (bpy.context.view_layer.objects.active != None) else True)):
        layout = self.layout
        layout.menu('SNA_MT_ACB99', text='Lazy Bones', icon_value=174)


class SNA_MT_ACB99(bpy.types.Menu):
    bl_idname = "SNA_MT_ACB99"
    bl_label = ""

    @classmethod
    def poll(cls, context):
        return not (False)

    def draw(self, context):
        layout = self.layout.column_flow(columns=1)
        layout.operator_context = "INVOKE_DEFAULT"
        op = layout.operator('sna.lattice_2_edges_badcf', text='Lattice-2-Edges', icon_value=0, emboss=True, depress=False)


class SNA_OT_Lattice_2_Edges_Badcf(bpy.types.Operator):
    bl_idname = "sna.lattice_2_edges_badcf"
    bl_label = "Lattice 2 Edges"
    bl_description = ""
    bl_options = {"REGISTER", "UNDO"}

    @classmethod
    def poll(cls, context):
        if bpy.app.version >= (3, 0, 0) and True:
            cls.poll_message_set('')
        return not False

    def execute(self, context):
        # Get the active object
        active_object = bpy.context.active_object
        # Check if the active object is a lattice
        if active_object and active_object.type == 'LATTICE':
            lattice = active_object
            u = lattice.data.points_u
            v = lattice.data.points_v
            w = lattice.data.points_w
            w_cap = u * v * (w - 1)
            v_cap = v * u
            verts = []
            edges = []
            # Get the world matrix of the lattice
            world_matrix = lattice.matrix_world
            for point in lattice.data.points:
                # Transform each lattice point to world space
                verts.append(world_matrix @ point.co)
            x = 0
            for i in range(0, len(verts)):
                if i % u:
                    edges.append((i - 1, i))  # creating u edges
                if (i - x) % (v_cap):
                    edges.append((i, i - u))  # creating v edges
                if (x + 1) % u:
                    x += 1
                else:
                    x = 0
                if i < w_cap:
                    edges.append((i, i + v_cap))  # creating w edges
            # Check if the object with name 'Lattice_2_Edges' already exists
            object_name = 'Lattice_2_Edges'
            suffix = 1
            while object_name in bpy.data.objects:
                object_name = f'Lattice_2_Edges_{suffix}'
                suffix += 1
            mesh = bpy.data.meshes.new(object_name)
            object = bpy.data.objects.new(object_name, mesh)
            mesh.from_pydata(verts, edges, [])
            bpy.context.scene.collection.objects.link(object)
            # Deselect all objects
            bpy.ops.object.select_all(action='DESELECT')
            # Select and activate the newly created mesh object
            bpy.context.view_layer.objects.active = object
            object.select_set(True)
            # Switch to object mode
            bpy.ops.object.mode_set(mode='OBJECT')
        else:
            print("No active lattice object found.")
        bpy.ops.object.origin_set('INVOKE_DEFAULT', type='ORIGIN_GEOMETRY', center='MEDIAN')
        bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='EDIT', toggle=False)
        return {"FINISHED"}

    def invoke(self, context, event):
        return self.execute(context)


class SNA_OT_Lazy_Bones_Apply_Sim_Operator_9020B(bpy.types.Operator):
    bl_idname = "sna.lazy_bones_apply_sim_operator_9020b"
    bl_label = "Lazy Bones: Apply Sim Operator"
    bl_description = "Applies the simulation and adds Sim Clean Up modifiers"
    bl_options = {"REGISTER", "UNDO"}

    @classmethod
    def poll(cls, context):
        if bpy.app.version >= (3, 0, 0) and True:
            cls.poll_message_set('')
        return not False

    def execute(self, context):
        if bpy.context.screen.is_animation_playing:
            bpy.ops.screen.animation_cancel('INVOKE_DEFAULT', restore_frame=False)
        bpy.ops.object.convert('INVOKE_DEFAULT', target='MESH')
        sna_add_lazy_bones_clean_up_modifiers_func_D2B53()
        return {"FINISHED"}

    def invoke(self, context, event):
        return self.execute(context)


class SNA_OT_Lazy_Bones_Cancel_Sim_Operator_32Fa6(bpy.types.Operator):
    bl_idname = "sna.lazy_bones_cancel_sim_operator_32fa6"
    bl_label = "Lazy Bones: Cancel Sim Operator"
    bl_description = "Cancels the simulation and removes the simulation object."
    bl_options = {"REGISTER", "UNDO"}

    @classmethod
    def poll(cls, context):
        if bpy.app.version >= (3, 0, 0) and True:
            cls.poll_message_set('')
        return not False

    def execute(self, context):
        if bpy.context.screen.is_animation_playing:
            bpy.ops.screen.animation_cancel('INVOKE_DEFAULT', restore_frame=True)
        bpy.ops.object.delete('INVOKE_DEFAULT', use_global=True, confirm=False)
        bpy.context.scene.frame_start = bpy.context.scene.sna_m2etimelinestart
        bpy.context.scene.frame_end = bpy.context.scene.sna_m2etimelineend
        if (bpy.context.scene.sna_m2ebasemesh == None):
            pass
        else:
            bpy.context.scene.sna_m2ebasemesh.hide_set(state=False, view_layer=bpy.context.view_layer, )
        return {"FINISHED"}

    def invoke(self, context, event):
        return self.execute(context)


class SNA_PT_MESH2BONE_SIM_4CCC3(bpy.types.Panel):
    bl_label = 'Mesh-2-Bone: Sim'
    bl_idname = 'SNA_PT_MESH2BONE_SIM_4CCC3'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_context = ''
    bl_category = 'Lazy Bones'
    bl_order = 0
    bl_ui_units_x=0

    @classmethod
    def poll(cls, context):
        return not ((not (property_exists("bpy.context.view_layer.objects.active.modifiers", globals(), locals()) and 'Lazy_Bones_Mesh_2_Edge_Sim' in bpy.context.view_layer.objects.active.modifiers)))

    def draw_header(self, context):
        layout = self.layout

    def draw(self, context):
        layout = self.layout
        layout.template_icon(icon_value=load_preview_icon(os.path.join(os.path.dirname(__file__), 'assets', 'Mesh-2-Bone Sim Icon.png')), scale=5.0)
        box_2EBB4 = layout.box()
        box_2EBB4.alert = False
        box_2EBB4.enabled = True
        box_2EBB4.active = True
        box_2EBB4.use_property_split = False
        box_2EBB4.use_property_decorate = False
        box_2EBB4.alignment = 'Expand'.upper()
        box_2EBB4.scale_x = 1.0
        box_2EBB4.scale_y = 1.0
        if not True: box_2EBB4.operator_context = "EXEC_DEFAULT"
        op = box_2EBB4.operator('sna.lazy_bones_apply_sim_operator_9020b', text='Apply Simulation', icon_value=0, emboss=True, depress=False)
        box_1F62C = box_2EBB4.box()
        box_1F62C.alert = True
        box_1F62C.enabled = True
        box_1F62C.active = True
        box_1F62C.use_property_split = False
        box_1F62C.use_property_decorate = False
        box_1F62C.alignment = 'Expand'.upper()
        box_1F62C.scale_x = 1.0
        box_1F62C.scale_y = 1.0
        if not True: box_1F62C.operator_context = "EXEC_DEFAULT"
        op = box_1F62C.operator('sna.lazy_bones_cancel_sim_operator_32fa6', text='Cancel', icon_value=21, emboss=True, depress=False)


def sna_add_lazy_bones_clean_up_modifiers_func_D2B53():
    sna_add_lazy_bones_center_snap_modifier_func_57415()
    bpy.ops.object.modifier_add('INVOKE_DEFAULT', type='MIRROR')
    bpy.context.view_layer.objects.active.modifiers['Mirror'].name = 'Lazy_Bones_Mirror'
    bpy.context.view_layer.objects.active.modifiers['Lazy_Bones_Mirror'].use_bisect_axis = (True, True, True)
    bpy.context.view_layer.objects.active.modifiers['Lazy_Bones_Mirror'].use_axis = (False, False, False)
    sna_add_lazy_bones_remove_intersections_modifier_func_3F946()
    sna_add_lazy_bones_resample_and_remove_small_edges_modifier_func_ACE77()
    sna_add_lazy_bones_bone_preview_func_E0CBC()
    bpy.context.scene.sna_m2edupemesh.select_set(state=True, view_layer=bpy.context.view_layer, )
    bpy.data.scenes[0].frame_start = bpy.context.scene.sna_m2etimelinestart
    bpy.context.scene.frame_end = bpy.context.scene.sna_m2etimelineend
    bpy.ops.wm.call_panel(name="SNA_PT_SIM_CLEAN_A2455", keep_open=True)


def sna_add_lazy_bones_center_snap_modifier_func_57415():
    if (property_exists("bpy.data.node_groups", globals(), locals()) and 'Lazy_Bones_Center_Snap' in bpy.data.node_groups):
        pass
    else:
        before_data = list(bpy.data.node_groups)
        bpy.ops.wm.append(directory=os.path.join(os.path.dirname(__file__), 'assets', 'Lazy_Bones_Assets.blend') + r'\NodeTree', filename='Lazy_Bones_Center_Snap', link=False)
        new_data = list(filter(lambda d: not d in before_data, list(bpy.data.node_groups)))
        appended_59976 = None if not new_data else new_data[0]
        sna_lazy_bones_select_and__set_active_dupe_func_4FC0C()
    if (property_exists("bpy.context.view_layer.objects.active.modifiers", globals(), locals()) and 'Lazy_Bones_Center_Snap' in bpy.context.view_layer.objects.active.modifiers):
        pass
    else:
        bpy.ops.object.modifier_add('INVOKE_DEFAULT', type='NODES')
        bpy.context.active_object.modifiers['GeometryNodes'].node_group = bpy.data.node_groups['Lazy_Bones_Center_Snap']
        bpy.context.view_layer.objects.active.modifiers['GeometryNodes'].name = 'Lazy_Bones_Center_Snap'


def sna_add_lazy_bones_remove_intersections_modifier_func_3F946():
    if (property_exists("bpy.data.node_groups", globals(), locals()) and 'Lazy_Bones_Remove_Intersections' in bpy.data.node_groups):
        pass
    else:
        before_data = list(bpy.data.node_groups)
        bpy.ops.wm.append(directory=os.path.join(os.path.dirname(__file__), 'assets', 'Lazy_Bones_Assets.blend') + r'\NodeTree', filename='Lazy_Bones_Remove_Intersections', link=False)
        new_data = list(filter(lambda d: not d in before_data, list(bpy.data.node_groups)))
        appended_DECE7 = None if not new_data else new_data[0]
        sna_lazy_bones_select_and__set_active_dupe_func_4FC0C()
    if (property_exists("bpy.context.view_layer.objects.active.modifiers", globals(), locals()) and 'Lazy_Bones_Remove_Intersections' in bpy.context.view_layer.objects.active.modifiers):
        pass
    else:
        bpy.ops.object.modifier_add('INVOKE_DEFAULT', type='NODES')
        bpy.context.active_object.modifiers['GeometryNodes'].node_group = bpy.data.node_groups['Lazy_Bones_Remove_Intersections']
        bpy.context.view_layer.objects.active.modifiers['GeometryNodes'].name = 'Lazy_Bones_Remove_Intersections'


def sna_add_lazy_bones_resample_and_remove_small_edges_modifier_func_ACE77():
    if (property_exists("bpy.data.node_groups", globals(), locals()) and 'Lazy_Bones_Resample_And_Remove_Small_Edges' in bpy.data.node_groups):
        pass
    else:
        before_data = list(bpy.data.node_groups)
        bpy.ops.wm.append(directory=os.path.join(os.path.dirname(__file__), 'assets', 'Lazy_Bones_Assets.blend') + r'\NodeTree', filename='Lazy_Bones_Resample_And_Remove_Small_Edges', link=False)
        new_data = list(filter(lambda d: not d in before_data, list(bpy.data.node_groups)))
        appended_D48D3 = None if not new_data else new_data[0]
        sna_lazy_bones_select_and__set_active_dupe_func_4FC0C()
    if (property_exists("bpy.context.view_layer.objects.active.modifiers", globals(), locals()) and 'Lazy_Bones_Resample_And_Remove_Small_Edges' in bpy.context.view_layer.objects.active.modifiers):
        pass
    else:
        bpy.ops.object.modifier_add('INVOKE_DEFAULT', type='NODES')
        bpy.context.active_object.modifiers['GeometryNodes'].node_group = bpy.data.node_groups['Lazy_Bones_Resample_And_Remove_Small_Edges']
        bpy.context.view_layer.objects.active.modifiers['GeometryNodes'].name = 'Lazy_Bones_Resample_And_Remove_Small_Edges'


def sna_add_lazy_bones_bone_preview_func_E0CBC():
    if (property_exists("bpy.data.node_groups", globals(), locals()) and 'Lazy_Bones_Bone_Preview' in bpy.data.node_groups):
        pass
    else:
        before_data = list(bpy.data.node_groups)
        bpy.ops.wm.append(directory=os.path.join(os.path.dirname(__file__), 'assets', 'Lazy_Bones_Assets.blend') + r'\NodeTree', filename='Lazy_Bones_Bone_Preview', link=False)
        new_data = list(filter(lambda d: not d in before_data, list(bpy.data.node_groups)))
        appended_48DAF = None if not new_data else new_data[0]
        sna_lazy_bones_select_and__set_active_dupe_func_4FC0C()
    if (property_exists("bpy.context.view_layer.objects.active.modifiers", globals(), locals()) and 'Lazy_Bones_Bone_Preview' in bpy.context.view_layer.objects.active.modifiers):
        pass
    else:
        bpy.ops.object.modifier_add('INVOKE_DEFAULT', type='NODES')
        bpy.context.active_object.modifiers['GeometryNodes'].node_group = bpy.data.node_groups['Lazy_Bones_Bone_Preview']
        bpy.context.view_layer.objects.active.modifiers['GeometryNodes'].name = 'Lazy_Bones_Bone_Preview'


class SNA_PT_SIM_CLEAN_A2455(bpy.types.Panel):
    bl_label = 'Sim Clean'
    bl_idname = 'SNA_PT_SIM_CLEAN_A2455'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_context = ''
    bl_category = 'Lazy Bones'
    bl_order = 1
    bl_ui_units_x=0

    @classmethod
    def poll(cls, context):
        return not ((not (((property_exists("bpy.context.view_layer.objects.active.modifiers", globals(), locals()) and 'Lazy_Bones_Remove_Intersections' in bpy.context.view_layer.objects.active.modifiers) or (property_exists("bpy.context.view_layer.objects.active.modifiers", globals(), locals()) and 'Lazy_Bones_Resample_And_Remove_Small_Edges' in bpy.context.view_layer.objects.active.modifiers) or (property_exists("bpy.context.view_layer.objects.active.modifiers", globals(), locals()) and 'Lazy_Bones_Center_Snap' in bpy.context.view_layer.objects.active.modifiers) or (property_exists("bpy.context.view_layer.objects.active.modifiers", globals(), locals()) and 'Lazy_Bones_Mirror' in bpy.context.view_layer.objects.active.modifiers) or (property_exists("bpy.context.view_layer.objects.active.modifiers", globals(), locals()) and 'Lazy_Bones_Bone_Preview' in bpy.context.view_layer.objects.active.modifiers)) and bpy.context.view_layer.objects.active.type == 'MESH' and (len(bpy.context.view_layer.objects.selected) > 0))))

    def draw_header(self, context):
        layout = self.layout

    def draw(self, context):
        layout = self.layout
        layout.template_icon(icon_value=load_preview_icon(os.path.join(os.path.dirname(__file__), 'assets', 'Sim Clean Icon.png')), scale=5.0)
        box_C601D = layout.box()
        box_C601D.alert = False
        box_C601D.enabled = True
        box_C601D.active = True
        box_C601D.use_property_split = False
        box_C601D.use_property_decorate = False
        box_C601D.alignment = 'Expand'.upper()
        box_C601D.scale_x = 1.2999999523162842
        box_C601D.scale_y = 1.2999999523162842
        if not True: box_C601D.operator_context = "EXEC_DEFAULT"
        box_C601D.label(text='Center Snap (World Space)', icon_value=0)
        attr_3F43B = '["' + str('Socket_2' + '"]') 
        box_C601D.prop(bpy.context.view_layer.objects.active.modifiers['Lazy_Bones_Center_Snap'], attr_3F43B, text='X snap threshold', icon_value=0, emboss=True)
        attr_68124 = '["' + str('Socket_4' + '"]') 
        box_C601D.prop(bpy.context.view_layer.objects.active.modifiers['Lazy_Bones_Center_Snap'], attr_68124, text='Y snap threshold', icon_value=0, emboss=True)
        attr_400D9 = '["' + str('Socket_5' + '"]') 
        box_C601D.prop(bpy.context.view_layer.objects.active.modifiers['Lazy_Bones_Center_Snap'], attr_400D9, text='Z snap threshold', icon_value=0, emboss=True)
        box_BAC79 = layout.box()
        box_BAC79.alert = False
        box_BAC79.enabled = True
        box_BAC79.active = True
        box_BAC79.use_property_split = False
        box_BAC79.use_property_decorate = False
        box_BAC79.alignment = 'Expand'.upper()
        box_BAC79.scale_x = 1.0
        box_BAC79.scale_y = 1.0
        if not True: box_BAC79.operator_context = "EXEC_DEFAULT"
        box_BAC79.label(text='Mirror Axis', icon_value=0)
        row_10D2F = box_BAC79.row(heading='', align=False)
        row_10D2F.alert = False
        row_10D2F.enabled = True
        row_10D2F.active = True
        row_10D2F.use_property_split = False
        row_10D2F.use_property_decorate = False
        row_10D2F.scale_x = 2.0
        row_10D2F.scale_y = 1.0
        row_10D2F.alignment = 'Center'.upper()
        row_10D2F.operator_context = "INVOKE_DEFAULT" if True else "EXEC_DEFAULT"
        row_10D2F.prop(bpy.context.view_layer.objects.active.modifiers['Lazy_Bones_Mirror'], 'use_axis', text='', icon_value=912, emboss=True, index=0)
        row_10D2F.prop(bpy.context.view_layer.objects.active.modifiers['Lazy_Bones_Mirror'], 'use_axis', text='', icon_value=913, emboss=True, index=1)
        row_10D2F.prop(bpy.context.view_layer.objects.active.modifiers['Lazy_Bones_Mirror'], 'use_axis', text='', icon_value=914, emboss=True, index=2)
        box_BAC79.label(text='Mirror Flip', icon_value=0)
        row_3F32F = box_BAC79.row(heading='', align=False)
        row_3F32F.alert = False
        row_3F32F.enabled = True
        row_3F32F.active = True
        row_3F32F.use_property_split = False
        row_3F32F.use_property_decorate = False
        row_3F32F.scale_x = 2.0
        row_3F32F.scale_y = 1.0
        row_3F32F.alignment = 'Center'.upper()
        row_3F32F.operator_context = "INVOKE_DEFAULT" if True else "EXEC_DEFAULT"
        row_3F32F.prop(bpy.context.view_layer.objects.active.modifiers['Lazy_Bones_Mirror'], 'use_bisect_flip_axis', text='', icon_value=912, emboss=True, index=0)
        row_3F32F.prop(bpy.context.view_layer.objects.active.modifiers['Lazy_Bones_Mirror'], 'use_bisect_flip_axis', text='', icon_value=913, emboss=True, index=1)
        row_3F32F.prop(bpy.context.view_layer.objects.active.modifiers['Lazy_Bones_Mirror'], 'use_bisect_flip_axis', text='', icon_value=914, emboss=True, index=2)
        box_42E47 = layout.box()
        box_42E47.alert = False
        box_42E47.enabled = True
        box_42E47.active = True
        box_42E47.use_property_split = False
        box_42E47.use_property_decorate = False
        box_42E47.alignment = 'Expand'.upper()
        box_42E47.scale_x = 1.2999999523162842
        box_42E47.scale_y = 1.2999999523162842
        if not True: box_42E47.operator_context = "EXEC_DEFAULT"
        box_42E47.prop(bpy.context.view_layer.objects.active.modifiers['Lazy_Bones_Remove_Intersections'], 'show_viewport', text='Remove Intersections?', icon_value=596, emboss=True)
        box_CB4CE = layout.box()
        box_CB4CE.alert = False
        box_CB4CE.enabled = True
        box_CB4CE.active = True
        box_CB4CE.use_property_split = False
        box_CB4CE.use_property_decorate = False
        box_CB4CE.alignment = 'Expand'.upper()
        box_CB4CE.scale_x = 1.2999999523162842
        box_CB4CE.scale_y = 1.2999999523162842
        if not True: box_CB4CE.operator_context = "EXEC_DEFAULT"
        attr_FFA73 = '["' + str('Socket_3' + '"]') 
        box_CB4CE.prop(bpy.context.view_layer.objects.active.modifiers['Lazy_Bones_Resample_And_Remove_Small_Edges'], attr_FFA73, text='Resample Edge Count', icon_value=0, emboss=True)
        attr_811C9 = '["' + str('Socket_2' + '"]') 
        box_CB4CE.prop(bpy.context.view_layer.objects.active.modifiers['Lazy_Bones_Resample_And_Remove_Small_Edges'], attr_811C9, text='Edge Deletion Threshold', icon_value=0, emboss=True)
        box_53A0F = layout.box()
        box_53A0F.alert = False
        box_53A0F.enabled = True
        box_53A0F.active = True
        box_53A0F.use_property_split = False
        box_53A0F.use_property_decorate = False
        box_53A0F.alignment = 'Expand'.upper()
        box_53A0F.scale_x = 1.0
        box_53A0F.scale_y = 1.0
        if not True: box_53A0F.operator_context = "EXEC_DEFAULT"
        attr_7A94F = '["' + str('Socket_2' + '"]') 
        box_53A0F.prop(bpy.context.view_layer.objects.active.modifiers['Lazy_Bones_Bone_Preview'], attr_7A94F, text='Point scale', icon_value=0, emboss=True)
        layout.separator(factor=1.0)
        box_3D374 = layout.box()
        box_3D374.alert = True
        box_3D374.enabled = True
        box_3D374.active = True
        box_3D374.use_property_split = False
        box_3D374.use_property_decorate = False
        box_3D374.alignment = 'Expand'.upper()
        box_3D374.scale_x = 1.0
        box_3D374.scale_y = 1.0
        if not True: box_3D374.operator_context = "EXEC_DEFAULT"
        op = box_3D374.operator('sna.lazy_bones_m2e_apply_modifiers_operator_a686a', text='Apply Modifiers', icon_value=0, emboss=True, depress=False)


class SNA_OT_Lazy_Bones_M2E_Apply_Modifiers_Operator_A686A(bpy.types.Operator):
    bl_idname = "sna.lazy_bones_m2e_apply_modifiers_operator_a686a"
    bl_label = "Lazy Bones M2E Apply Modifiers Operator"
    bl_description = ""
    bl_options = {"REGISTER", "UNDO"}

    @classmethod
    def poll(cls, context):
        if bpy.app.version >= (3, 0, 0) and True:
            cls.poll_message_set('')
        return not False

    def execute(self, context):
        bpy.ops.object.convert('INVOKE_DEFAULT', target='MESH')
        return {"FINISHED"}

    def invoke(self, context, event):
        return self.execute(context)


class SNA_AddonPreferences_06E76(bpy.types.AddonPreferences):
    bl_idname = 'lazy_bones'
    sna_lazy_bones_show_tips: bpy.props.BoolProperty(name='Lazy Bones Show Tips', description='', default=True)
    sna_lazy_bones_simulation_cache: bpy.props.BoolProperty(name='Lazy Bones Simulation Cache', description='', default=True)

    def draw(self, context):
        if not (False):
            layout = self.layout 
            box_F8FFF = layout.box()
            box_F8FFF.alert = False
            box_F8FFF.enabled = True
            box_F8FFF.active = True
            box_F8FFF.use_property_split = False
            box_F8FFF.use_property_decorate = False
            box_F8FFF.alignment = 'Expand'.upper()
            box_F8FFF.scale_x = 1.0
            box_F8FFF.scale_y = 1.0
            if not True: box_F8FFF.operator_context = "EXEC_DEFAULT"
            box_F8FFF.prop(bpy.context.preferences.addons['lazy_bones'].preferences, 'sna_lazy_bones_show_tips', text='Enable tips?', icon_value=52, emboss=True)
            box_7C982 = layout.box()
            box_7C982.alert = False
            box_7C982.enabled = True
            box_7C982.active = True
            box_7C982.use_property_split = False
            box_7C982.use_property_decorate = False
            box_7C982.alignment = 'Expand'.upper()
            box_7C982.scale_x = 1.0
            box_7C982.scale_y = 1.0
            if not True: box_7C982.operator_context = "EXEC_DEFAULT"
            box_7C982.prop(bpy.context.preferences.addons['lazy_bones'].preferences, 'sna_lazy_bones_simulation_cache', text='Enable Simulation Cache?', icon_value=722, emboss=True)
            box_7C982.label(text="The cache will allow you to pause and 'replay' the simulation to find a moment ", icon_value=2)
            box_7C982.label(text='you like. However, it can cause glitches. If you begin the simulation and nothing happens', icon_value=0)
            box_7C982.label(text='- disable the cache.', icon_value=0)


def sna_add_to_view3d_mt_object_context_menu_D45AB(self, context):
    if not (False):
        layout = self.layout
        if bpy.context.view_layer.objects.active.type == 'MESH':
            layout.menu('SNA_MT_B094C', text='Lazy Bones', icon_value=200)


class SNA_MT_B094C(bpy.types.Menu):
    bl_idname = "SNA_MT_B094C"
    bl_label = ""

    @classmethod
    def poll(cls, context):
        return not (False)

    def draw(self, context):
        layout = self.layout.column_flow(columns=1)
        layout.operator_context = "INVOKE_DEFAULT"
        op = layout.operator('sna.lazy_bone_mesh_2_edges_operator_65419', text='Mesh-2-Edge Sim', icon_value=89, emboss=True, depress=False)
        layout.separator(factor=1.0)
        if bpy.context.preferences.addons['lazy_bones'].preferences.sna_lazy_bones_show_tips:
            layout.label(text='Simulation is affected by object scale.', icon_value=4)
        if bpy.context.preferences.addons['lazy_bones'].preferences.sna_lazy_bones_show_tips:
            layout.label(text='Scale up for more accurate (but slower) results.', icon_value=4)
        if bpy.context.preferences.addons['lazy_bones'].preferences.sna_lazy_bones_show_tips:
            layout.label(text='Non-manifold objects do not simulate well.', icon_value=4)
        if bpy.context.preferences.addons['lazy_bones'].preferences.sna_lazy_bones_show_tips:
            layout.label(text='Increasing an objects poly-count can improve results.', icon_value=4)


class SNA_OT_Lazy_Bone_Mesh_2_Edges_Operator_65419(bpy.types.Operator):
    bl_idname = "sna.lazy_bone_mesh_2_edges_operator_65419"
    bl_label = "Lazy Bone: Mesh 2 Edges Operator"
    bl_description = "Add the Mesh-2-Edge Simulation node group and begins playback."
    bl_options = {"REGISTER", "UNDO"}

    @classmethod
    def poll(cls, context):
        if bpy.app.version >= (3, 0, 0) and True:
            cls.poll_message_set('')
        return not False

    def execute(self, context):
        if (bpy.context.view_layer.objects.active == None):
            self.report({'ERROR'}, message='No active object.')
        else:
            bpy.context.scene.sna_m2etimelineend = bpy.context.scene.frame_end
            bpy.context.scene.sna_m2etimelinestart = bpy.context.scene.frame_start
            bpy.context.scene.sna_m2ebasemesh = bpy.context.view_layer.objects.active
            bpy.ops.object.duplicate('INVOKE_DEFAULT', )
            bpy.context.view_layer.objects.active.name = 'M2E Sim Mesh'
            bpy.context.scene.sna_m2edupemesh = bpy.context.view_layer.objects.active
            bpy.ops.object.convert('INVOKE_DEFAULT', target='MESH')
            bpy.ops.object.transform_apply('INVOKE_DEFAULT', location=False, rotation=False, scale=True, properties=False, isolate_users=False)
            if (property_exists("bpy.data.node_groups", globals(), locals()) and 'Lazy_Bones_Mesh_2_Edge_Sim' in bpy.data.node_groups):
                pass
            else:
                before_data = list(bpy.data.node_groups)
                bpy.ops.wm.append(directory=os.path.join(os.path.dirname(__file__), 'assets', 'Lazy_Bones_Assets.blend') + r'\NodeTree', filename='Lazy_Bones_Mesh_2_Edge_Sim', link=False)
                new_data = list(filter(lambda d: not d in before_data, list(bpy.data.node_groups)))
                appended_CE56D = None if not new_data else new_data[0]
            sna_lazy_bones_select_and__set_active_dupe_func_4FC0C()
            if (property_exists("bpy.context.scene.sna_m2edupemesh.modifiers", globals(), locals()) and 'Lazy_Bones_Mesh_2_Edge_Sim' in bpy.context.scene.sna_m2edupemesh.modifiers):
                pass
            else:
                bpy.ops.object.modifier_add('INVOKE_DEFAULT', type='NODES')
                bpy.context.active_object.modifiers['GeometryNodes'].node_group = bpy.data.node_groups['Lazy_Bones_Mesh_2_Edge_Sim']
                bpy.context.view_layer.objects.active.modifiers['GeometryNodes'].name = 'Lazy_Bones_Mesh_2_Edge_Sim'
            bpy.context.view_layer.objects.active.use_simulation_cache = bpy.context.preferences.addons['lazy_bones'].preferences.sna_lazy_bones_simulation_cache
            bpy.context.scene.frame_start = 1
            bpy.context.scene.frame_end = 2000
            bpy.context.scene.frame_current = 1
            bpy.context.scene.sna_m2edupemesh.select_set(state=True, view_layer=bpy.context.view_layer, )
            bpy.context.scene.sna_m2ebasemesh.hide_set(state=True, view_layer=bpy.context.view_layer, )
            bpy.ops.screen.animation_play('INVOKE_DEFAULT', )
            bpy.ops.wm.call_panel(name="SNA_PT_MESH2BONE_SIM_4CCC3", keep_open=False)
        return {"FINISHED"}

    def invoke(self, context, event):
        return self.execute(context)


def sna_lazy_bones_select_and__set_active_dupe_func_4FC0C():
    bpy.context.view_layer.objects.active = bpy.context.scene.sna_m2edupemesh
    bpy.context.scene.sna_m2edupemesh.select_set(state=True, view_layer=bpy.context.view_layer, )


class SNA_OT_Symmetrize_World_Space_68Cc8(bpy.types.Operator):
    bl_idname = "sna.symmetrize_world_space_68cc8"
    bl_label = "Symmetrize (World Space)"
    bl_description = ""
    bl_options = {"REGISTER", "UNDO"}

    def sna_symmetry_axis_direction_enum_items(self, context):
        return [("No Items", "No Items", "No generate enum items node found to create items!", "ERROR", 0)]
    sna_symmetry_axis_direction: bpy.props.EnumProperty(name='Symmetry Axis Direction', description='', items=[('-X to +X', '-X to +X', '', 0, 0), ('+X to -X', '+X to -X', '', 0, 1)])
    sna_use_existing_symmetry_suffixes: bpy.props.BoolProperty(name='Use existing symmetry suffixes?', description='', default=False)

    @classmethod
    def poll(cls, context):
        if bpy.app.version >= (3, 0, 0) and True:
            cls.poll_message_set('')
        return not False

    def execute(self, context):
        if (bpy.context.view_layer.objects.active == None):
            self.report({'ERROR'}, message='No active object.')
        else:
            if bpy.context.view_layer.objects.active.type == 'ARMATURE':
                if 'OBJECT'==bpy.context.mode:
                    pass
                else:
                    bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='OBJECT')
                bpy.ops.object.transform_apply('INVOKE_DEFAULT', location=True, rotation=True, scale=True, properties=False, isolate_users=False)
                for i_26609 in range(len(bpy.context.view_layer.objects.selected)):
                    if bpy.context.view_layer.objects.selected[i_26609].type == 'ARMATURE':
                        bpy.context.view_layer.objects.selected[i_26609].data.use_mirror_x = False
                for i_2FEE4 in range(len(bpy.context.view_layer.objects.selected)):
                    if bpy.context.view_layer.objects.selected[i_2FEE4].type == 'ARMATURE':
                        for i_11686 in range(len(bpy.context.view_layer.objects.selected[i_2FEE4].data.bones)):
                            if (self.sna_use_existing_symmetry_suffixes and ('.l' in bpy.context.view_layer.objects.selected[i_2FEE4].data.bones[i_11686].name or '.r' in bpy.context.view_layer.objects.selected[i_2FEE4].data.bones[i_11686].name)):
                                pass
                            else:
                                if ('.l' in bpy.context.view_layer.objects.selected[i_2FEE4].data.bones[i_11686].name or '.r' in bpy.context.view_layer.objects.selected[i_2FEE4].data.bones[i_11686].name):
                                    bpy.context.view_layer.objects.selected[i_2FEE4].data.bones[i_11686].name = bpy.context.view_layer.objects.selected[i_2FEE4].data.bones[i_11686].name.replace('.l', '').replace('.r', '')
                                if (round(tuple(mathutils.Vector(bpy.context.view_layer.objects.selected[i_2FEE4].data.bones[i_11686].head_local) + mathutils.Vector(bpy.context.view_layer.objects.active.location))[0], abs(2)) == 0):
                                    if (round(tuple(mathutils.Vector(bpy.context.view_layer.objects.selected[i_2FEE4].data.bones[i_11686].tail_local) + mathutils.Vector(bpy.context.view_layer.objects.active.location))[0], abs(2)) == 0):
                                        pass
                                    else:
                                        if (round(tuple(mathutils.Vector(bpy.context.view_layer.objects.selected[i_2FEE4].data.bones[i_11686].tail_local) + mathutils.Vector(bpy.context.view_layer.objects.active.location))[0], abs(2)) > 0):
                                            bpy.context.view_layer.objects.selected[i_2FEE4].data.bones[i_11686].name = bpy.context.view_layer.objects.selected[i_2FEE4].data.bones[i_11686].name + '.l'
                                        else:
                                            bpy.context.view_layer.objects.selected[i_2FEE4].data.bones[i_11686].name = bpy.context.view_layer.objects.selected[i_2FEE4].data.bones[i_11686].name + '.r'
                                else:
                                    if (round(tuple(mathutils.Vector(bpy.context.view_layer.objects.selected[i_2FEE4].data.bones[i_11686].head_local) + mathutils.Vector(bpy.context.view_layer.objects.active.location))[0], abs(2)) > 0):
                                        bpy.context.view_layer.objects.selected[i_2FEE4].data.bones[i_11686].name = bpy.context.view_layer.objects.selected[i_2FEE4].data.bones[i_11686].name + '.l'
                                    else:
                                        bpy.context.view_layer.objects.selected[i_2FEE4].data.bones[i_11686].name = bpy.context.view_layer.objects.selected[i_2FEE4].data.bones[i_11686].name + '.r'
                for i_8341C in range(len(bpy.context.view_layer.objects.selected)-1,-1,-1):
                    bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='EDIT')
                    if bpy.context.view_layer.objects.selected[i_8341C].type == 'ARMATURE':
                        bpy.ops.armature.select_all('INVOKE_DEFAULT', action='DESELECT')
                        for i_5673A in range(len(bpy.context.view_layer.objects.selected[i_8341C].data.edit_bones)):
                            if ('.r' if (self.sna_symmetry_axis_direction != '-X to +X') else '.l') in bpy.context.view_layer.objects.selected[i_8341C].data.edit_bones[i_5673A].name:
                                bpy.context.view_layer.objects.selected[i_8341C].data.edit_bones[i_5673A].select = True
                        bpy.ops.armature.delete('INVOKE_DEFAULT', confirm=False)
                bpy.ops.armature.select_all('INVOKE_DEFAULT', action='SELECT')
                bpy.ops.armature.symmetrize('INVOKE_DEFAULT', direction=('NEGATIVE_X' if (self.sna_symmetry_axis_direction == '-X to +X') else 'POSITIVE_X'))
                bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='OBJECT', toggle=False)
                lazy_bone_symmetrize_serpens['sna_lazybones0bonesdeletevar'] = []
                for i_09A62 in range(len(bpy.context.view_layer.objects.selected)):
                    if bpy.context.view_layer.objects.selected[i_09A62].type == 'ARMATURE':
                        if (len(bpy.context.view_layer.objects.selected[i_09A62].data.bones) == 0):
                            lazy_bone_symmetrize_serpens['sna_lazybones0bonesdeletevar'].append(bpy.context.view_layer.objects.selected[i_09A62].name)
                for i_0674D in range(len(lazy_bone_symmetrize_serpens['sna_lazybones0bonesdeletevar'])-1,-1,-1):
                    bpy.ops.object.select_all('INVOKE_DEFAULT', action='DESELECT')
                    bpy.data.objects[lazy_bone_symmetrize_serpens['sna_lazybones0bonesdeletevar'][i_0674D]].select_set(state=True, view_layer=bpy.context.view_layer, )
                    bpy.ops.object.delete('INVOKE_DEFAULT', use_global=False, confirm=False)
                if (bpy.context.view_layer.objects.active == None):
                    pass
                else:
                    bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='OBJECT', toggle=True)
        return {"FINISHED"}

    def draw(self, context):
        layout = self.layout
        if bpy.context.preferences.addons['lazy_bones'].preferences.sna_lazy_bones_show_tips:
            box_0B014 = layout.box()
            box_0B014.alert = False
            box_0B014.enabled = True
            box_0B014.active = True
            box_0B014.use_property_split = False
            box_0B014.use_property_decorate = False
            box_0B014.alignment = 'Expand'.upper()
            box_0B014.scale_x = 1.0
            box_0B014.scale_y = 1.0
            if not True: box_0B014.operator_context = "EXEC_DEFAULT"
            box_0B014.label(text='Bone symmetry relies on the world X axis.', icon_value=4)
            box_0B014.label(text='Your skeleton should be at the world origin and face towards -Y.', icon_value=4)
            box_0B014.label(text='All selected armatures will have their transformations applied.', icon_value=4)
            if self.sna_use_existing_symmetry_suffixes:
                box_0B014.label(text='If bones already have .l/.r suffixes this will be used instead of world position.', icon_value=4)
        layout.separator(factor=1.0)
        box_50A3B = layout.box()
        box_50A3B.alert = False
        box_50A3B.enabled = True
        box_50A3B.active = True
        box_50A3B.use_property_split = False
        box_50A3B.use_property_decorate = False
        box_50A3B.alignment = 'Expand'.upper()
        box_50A3B.scale_x = 1.0
        box_50A3B.scale_y = 1.0
        if not True: box_50A3B.operator_context = "EXEC_DEFAULT"
        box_50A3B.prop(self, 'sna_symmetry_axis_direction', text='X Axis Direction', icon_value=637, emboss=True)
        box_50A3B.separator(factor=1.0)
        box_50A3B.prop(self, 'sna_use_existing_symmetry_suffixes', text='Use existing symmetry suffix?', icon_value=0, emboss=True)
        if bpy.context.preferences.addons['lazy_bones'].preferences.sna_lazy_bones_show_tips:
            if self.sna_use_existing_symmetry_suffixes:
                box_5D43D = box_50A3B.box()
                box_5D43D.alert = True
                box_5D43D.enabled = True
                box_5D43D.active = True
                box_5D43D.use_property_split = False
                box_5D43D.use_property_decorate = False
                box_5D43D.alignment = 'Expand'.upper()
                box_5D43D.scale_x = 1.0
                box_5D43D.scale_y = 1.0
                if not True: box_5D43D.operator_context = "EXEC_DEFAULT"
                box_5D43D.label(text="Bones with ''.l ''or ''.r'' already in their name will not be renamed.", icon_value=2)
                box_5D43D.label(text='They will be symmetrized based on existing names, not world space.', icon_value=0)

    def invoke(self, context, event):
        return context.window_manager.invoke_props_dialog(self, width=500)


class SNA_OT_Delete_Half_World_Space_F888F(bpy.types.Operator):
    bl_idname = "sna.delete_half_world_space_f888f"
    bl_label = "Delete Half (World Space)"
    bl_description = ""
    bl_options = {"REGISTER", "UNDO"}

    def sna_delete_axis_direction_enum_items(self, context):
        return [("No Items", "No Items", "No generate enum items node found to create items!", "ERROR", 0)]
    sna_delete_axis_direction: bpy.props.EnumProperty(name='Delete Axis Direction', description='', items=[('-X (.r)', '-X (.r)', '', 0, 0), ('+X (.l)', '+X (.l)', '', 0, 1)])
    sna_use_existing_symmetry_suffixes: bpy.props.BoolProperty(name='Use existing symmetry suffixes?', description='', default=False)

    @classmethod
    def poll(cls, context):
        if bpy.app.version >= (3, 0, 0) and True:
            cls.poll_message_set('')
        return not False

    def execute(self, context):
        if (bpy.context.view_layer.objects.active == None):
            self.report({'ERROR'}, message='No active object.')
        else:
            if bpy.context.view_layer.objects.active.type == 'ARMATURE':
                if 'OBJECT'==bpy.context.mode:
                    pass
                else:
                    bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='OBJECT', toggle=False)
                bpy.ops.object.transform_apply('INVOKE_DEFAULT', location=True, rotation=True, scale=True, properties=False, isolate_users=False)
                for i_4822B in range(len(bpy.context.view_layer.objects.selected)):
                    if bpy.context.view_layer.objects.selected[i_4822B].type == 'ARMATURE':
                        bpy.context.view_layer.objects.selected[i_4822B].data.use_mirror_x = False
                for i_EAEEF in range(len(bpy.context.view_layer.objects.selected)):
                    if bpy.context.view_layer.objects.selected[i_EAEEF].type == 'ARMATURE':
                        for i_8E112 in range(len(bpy.context.view_layer.objects.selected[i_EAEEF].data.bones)):
                            if (self.sna_use_existing_symmetry_suffixes and ('.l' in bpy.context.view_layer.objects.selected[i_EAEEF].data.bones[i_8E112].name or '.r' in bpy.context.view_layer.objects.selected[i_EAEEF].data.bones[i_8E112].name)):
                                pass
                            else:
                                if ('.l' in bpy.context.view_layer.objects.selected[i_EAEEF].data.bones[i_8E112].name or '.r' in bpy.context.view_layer.objects.selected[i_EAEEF].data.bones[i_8E112].name):
                                    bpy.context.view_layer.objects.selected[i_EAEEF].data.bones[i_8E112].name = bpy.context.view_layer.objects.selected[i_EAEEF].data.bones[i_8E112].name.replace('.l', '').replace('.r', '')
                                if (round(tuple(mathutils.Vector(bpy.context.view_layer.objects.selected[i_EAEEF].data.bones[i_8E112].head_local) + mathutils.Vector(bpy.context.view_layer.objects.active.location))[0], abs(2)) == 0):
                                    if (round(tuple(mathutils.Vector(bpy.context.view_layer.objects.selected[i_EAEEF].data.bones[i_8E112].tail_local) + mathutils.Vector(bpy.context.view_layer.objects.active.location))[0], abs(2)) == 0):
                                        pass
                                    else:
                                        if (round(tuple(mathutils.Vector(bpy.context.view_layer.objects.selected[i_EAEEF].data.bones[i_8E112].tail_local) + mathutils.Vector(bpy.context.view_layer.objects.active.location))[0], abs(2)) > 0):
                                            bpy.context.view_layer.objects.selected[i_EAEEF].data.bones[i_8E112].name = bpy.context.view_layer.objects.selected[i_EAEEF].data.bones[i_8E112].name + '.l'
                                        else:
                                            bpy.context.view_layer.objects.selected[i_EAEEF].data.bones[i_8E112].name = bpy.context.view_layer.objects.selected[i_EAEEF].data.bones[i_8E112].name + '.r'
                                else:
                                    if (round(tuple(mathutils.Vector(bpy.context.view_layer.objects.selected[i_EAEEF].data.bones[i_8E112].head_local) + mathutils.Vector(bpy.context.view_layer.objects.active.location))[0], abs(2)) > 0):
                                        bpy.context.view_layer.objects.selected[i_EAEEF].data.bones[i_8E112].name = bpy.context.view_layer.objects.selected[i_EAEEF].data.bones[i_8E112].name + '.l'
                                    else:
                                        bpy.context.view_layer.objects.selected[i_EAEEF].data.bones[i_8E112].name = bpy.context.view_layer.objects.selected[i_EAEEF].data.bones[i_8E112].name + '.r'
                for i_6C102 in range(len(bpy.context.view_layer.objects.selected)-1,-1,-1):
                    bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='EDIT', toggle=False)
                    if bpy.context.view_layer.objects.selected[i_6C102].type == 'ARMATURE':
                        bpy.ops.armature.select_all('INVOKE_DEFAULT', action='DESELECT')
                        for i_EA8E9 in range(len(bpy.context.view_layer.objects.selected[i_6C102].data.edit_bones)):
                            if ('.r' if (self.sna_delete_axis_direction == '-X (.r)') else '.l') in bpy.context.view_layer.objects.selected[i_6C102].data.edit_bones[i_EA8E9].name:
                                bpy.context.view_layer.objects.selected[i_6C102].data.edit_bones[i_EA8E9].select = True
                        bpy.ops.armature.delete('INVOKE_DEFAULT', confirm=False)
                bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='OBJECT', toggle=False)
                lazy_bone_symmetrize_serpens['sna_lazybones0bonesdeletevar'] = []
                for i_03095 in range(len(bpy.context.view_layer.objects.selected)):
                    if bpy.context.view_layer.objects.selected[i_03095].type == 'ARMATURE':
                        if (len(bpy.context.view_layer.objects.selected[i_03095].data.bones) == 0):
                            lazy_bone_symmetrize_serpens['sna_lazybones0bonesdeletevar'].append(bpy.context.view_layer.objects.selected[i_03095].name)
                for i_63C62 in range(len(lazy_bone_symmetrize_serpens['sna_lazybones0bonesdeletevar'])-1,-1,-1):
                    print(lazy_bone_symmetrize_serpens['sna_lazybones0bonesdeletevar'][i_63C62])
                    bpy.ops.object.select_all('INVOKE_DEFAULT', action='DESELECT')
                    bpy.data.objects[lazy_bone_symmetrize_serpens['sna_lazybones0bonesdeletevar'][i_63C62]].select_set(state=True, view_layer=bpy.context.view_layer, )
                    bpy.ops.object.delete('INVOKE_DEFAULT', use_global=False, confirm=False)
                if (bpy.context.view_layer.objects.active == None):
                    pass
                else:
                    bpy.ops.object.mode_set('INVOKE_DEFAULT', mode='OBJECT', toggle=True)
        return {"FINISHED"}

    def draw(self, context):
        layout = self.layout
        if bpy.context.preferences.addons['lazy_bones'].preferences.sna_lazy_bones_show_tips:
            box_9EE56 = layout.box()
            box_9EE56.alert = False
            box_9EE56.enabled = True
            box_9EE56.active = True
            box_9EE56.use_property_split = False
            box_9EE56.use_property_decorate = False
            box_9EE56.alignment = 'Expand'.upper()
            box_9EE56.scale_x = 1.0
            box_9EE56.scale_y = 1.0
            if not True: box_9EE56.operator_context = "EXEC_DEFAULT"
            box_9EE56.label(text='It is not necessary to delete half to later symmetrize', icon_value=4)
            box_9EE56.label(text='Deleting half is only to visualize will be symmetrized.', icon_value=4)
            box_9EE56.label(text='Bone symmetry relies on the world X axis.', icon_value=4)
            box_9EE56.label(text='Your skeleton should be at the world origin and face towards -Y.', icon_value=4)
            if self.sna_use_existing_symmetry_suffixes:
                box_9EE56.label(text='If bones already have .l/.r suffixes this will be used instead of world position.', icon_value=4)
        layout.prop(self, 'sna_delete_axis_direction', text='Remove from', icon_value=446, emboss=True)
        box_317D0 = layout.box()
        box_317D0.alert = False
        box_317D0.enabled = True
        box_317D0.active = True
        box_317D0.use_property_split = False
        box_317D0.use_property_decorate = False
        box_317D0.alignment = 'Expand'.upper()
        box_317D0.scale_x = 1.0
        box_317D0.scale_y = 1.0
        if not True: box_317D0.operator_context = "EXEC_DEFAULT"
        box_317D0.separator(factor=1.0)
        box_317D0.prop(self, 'sna_use_existing_symmetry_suffixes', text='Use existing symmetry suffix?', icon_value=0, emboss=True)
        if bpy.context.preferences.addons['lazy_bones'].preferences.sna_lazy_bones_show_tips:
            if self.sna_use_existing_symmetry_suffixes:
                box_2EB03 = box_317D0.box()
                box_2EB03.alert = True
                box_2EB03.enabled = True
                box_2EB03.active = True
                box_2EB03.use_property_split = False
                box_2EB03.use_property_decorate = False
                box_2EB03.alignment = 'Expand'.upper()
                box_2EB03.scale_x = 1.0
                box_2EB03.scale_y = 1.0
                if not True: box_2EB03.operator_context = "EXEC_DEFAULT"
                box_2EB03.label(text="Bones with ''.l ''or ''.r'' already in their name will not be renamed.", icon_value=2)
                box_2EB03.label(text='They will be symmetrized based on existing names, not world space.', icon_value=0)

    def invoke(self, context, event):
        return context.window_manager.invoke_props_dialog(self, width=500)


class SNA_GROUP_sna_e2bcomponentsgroup(bpy.types.PropertyGroup):
    e2bcomponents: bpy.props.PointerProperty(name='E2BComponents', description='', type=bpy.types.Object)
    e2bcomponentedgecount: bpy.props.IntProperty(name='E2BComponentEdgeCount', description='', default=0, subtype='NONE')


class SNA_GROUP_sna_e2barmaturesgroup(bpy.types.PropertyGroup):
    e2barmatures: bpy.props.PointerProperty(name='E2BArmatures', description='', type=bpy.types.Object)


def register():
    global _icons
    _icons = bpy.utils.previews.new()
    bpy.utils.register_class(SNA_GROUP_sna_e2bcomponentsgroup)
    bpy.utils.register_class(SNA_GROUP_sna_e2barmaturesgroup)
    bpy.types.Scene.sna_e2bbaseedgecount = bpy.props.IntProperty(name='E2BBaseEdgeCount', description='', default=0, subtype='NONE')
    bpy.types.Scene.sna_e2bcomponentscollection = bpy.props.CollectionProperty(name='E2BComponentsCollection', description='', type=SNA_GROUP_sna_e2bcomponentsgroup)
    bpy.types.Scene.sna_e2barmaturescollection = bpy.props.CollectionProperty(name='E2BArmaturesCollection', description='', type=SNA_GROUP_sna_e2barmaturesgroup)
    bpy.types.Scene.sna_e2bbaseobject = bpy.props.PointerProperty(name='E2BBaseObject', description='', type=bpy.types.Object)
    bpy.types.Scene.sna_m2etimelineend = bpy.props.IntProperty(name='M2ETimelineEnd', description='', default=0, subtype='NONE')
    bpy.types.Scene.sna_m2etimelinestart = bpy.props.IntProperty(name='M2ETimelineStart', description='', default=1, subtype='NONE')
    bpy.types.Scene.sna_m2edupemesh = bpy.props.PointerProperty(name='M2EDupeMesh', description='', type=bpy.types.Object)
    bpy.types.Scene.sna_m2ebasemesh = bpy.props.PointerProperty(name='M2EBaseMesh', description='', type=bpy.types.Object)
    bpy.types.Scene.sna_lb_dissolveby = bpy.props.IntProperty(name='LB_DissolveBy', description='', default=2, subtype='NONE', min=2)
    bpy.utils.register_class(SNA_OT_Lazy_Bones_Select_Visible_Armature_Operator_0Cdc9)
    bpy.utils.register_class(SNA_OT_Lazy_Bones_Dissolve_To_1_0D25F)
    bpy.utils.register_class(SNA_PT_BONE_DOCTOR_7679E)
    bpy.utils.register_class(SNA_OT_Bone_Target_Individual_E2333)
    bpy.utils.register_class(SNA_OT_Lazy_Bones_Hide_Bone_Axis_Operator_02F86)
    bpy.utils.register_class(SNA_OT_Bone_Target_Whole_817Da)
    bpy.utils.register_class(SNA_OT_Lazy_Bones_Bone_Roll_Global_Z_Operator_D986F)
    bpy.utils.register_class(SNA_OT_Lazy_Bones_Bone_Roll_Global_Y_Operator_50848)
    bpy.utils.register_class(SNA_OT_Lazy_Bones_Bone_Roll_Global_Z_Operator_2Ef8D)
    bpy.utils.register_class(SNA_OT_Lazy_Bones_Bone_Roll_Global_X_Operator_Eaa28)
    bpy.utils.register_class(SNA_OT_Lazy_Bones_Bone_Roll_Global_Y_Operator_96065)
    bpy.utils.register_class(SNA_OT_Lazy_Bones_Bone_Roll_Local_Z_Operator_Dc8D7)
    bpy.utils.register_class(SNA_OT_Lazy_Bones_Bone_Roll_Local_X_Operator_10200)
    bpy.utils.register_class(SNA_OT_Lazy_Bones_Bone_Roll_Local_X_Operator_D57Fd)
    bpy.utils.register_class(SNA_OT_Lazy_Bones_Bone_Roll_Global_X_Operator_168Ea)
    bpy.utils.register_class(SNA_OT_Lazy_Bones_Bone_Roll_Local_Z_Operator_C377A)
    bpy.utils.register_class(SNA_OT_Lazy_Bones_Show_Bone_Axis_Operator_026A7)
    bpy.utils.register_class(SNA_OT_Lazy_Bones_Dissolve_By_Fraction_Operator_Df537)
    bpy.utils.register_class(SNA_OT_Lazy_Bones_Switch_Bone_Direction_Operator_10514)
    bpy.utils.register_class(SNA_MT_414F8)
    bpy.utils.register_class(SNA_OT_Edge2Bone_2Cf72)
    bpy.types.VIEW3D_MT_edit_mesh_context_menu.prepend(sna_add_to_view3d_mt_edit_mesh_context_menu_325A3)
    bpy.types.VIEW3D_MT_object_context_menu.prepend(sna_add_to_view3d_mt_object_context_menu_5E1DB)
    bpy.utils.register_class(SNA_MT_ACB99)
    bpy.utils.register_class(SNA_OT_Lattice_2_Edges_Badcf)
    bpy.utils.register_class(SNA_OT_Lazy_Bones_Apply_Sim_Operator_9020B)
    bpy.utils.register_class(SNA_OT_Lazy_Bones_Cancel_Sim_Operator_32Fa6)
    bpy.utils.register_class(SNA_PT_MESH2BONE_SIM_4CCC3)
    bpy.utils.register_class(SNA_PT_SIM_CLEAN_A2455)
    bpy.utils.register_class(SNA_OT_Lazy_Bones_M2E_Apply_Modifiers_Operator_A686A)
    bpy.utils.register_class(SNA_AddonPreferences_06E76)
    bpy.types.VIEW3D_MT_object_context_menu.prepend(sna_add_to_view3d_mt_object_context_menu_D45AB)
    bpy.utils.register_class(SNA_MT_B094C)
    bpy.utils.register_class(SNA_OT_Lazy_Bone_Mesh_2_Edges_Operator_65419)
    bpy.utils.register_class(SNA_OT_Symmetrize_World_Space_68Cc8)
    bpy.utils.register_class(SNA_OT_Delete_Half_World_Space_F888F)


def unregister():
    global _icons
    bpy.utils.previews.remove(_icons)
    wm = bpy.context.window_manager
    kc = wm.keyconfigs.addon
    for km, kmi in addon_keymaps.values():
        km.keymap_items.remove(kmi)
    addon_keymaps.clear()
    del bpy.types.Scene.sna_lb_dissolveby
    del bpy.types.Scene.sna_m2ebasemesh
    del bpy.types.Scene.sna_m2edupemesh
    del bpy.types.Scene.sna_m2etimelinestart
    del bpy.types.Scene.sna_m2etimelineend
    del bpy.types.Scene.sna_e2bbaseobject
    del bpy.types.Scene.sna_e2barmaturescollection
    del bpy.types.Scene.sna_e2bcomponentscollection
    del bpy.types.Scene.sna_e2bbaseedgecount
    bpy.utils.unregister_class(SNA_GROUP_sna_e2barmaturesgroup)
    bpy.utils.unregister_class(SNA_GROUP_sna_e2bcomponentsgroup)
    bpy.utils.unregister_class(SNA_OT_Lazy_Bones_Select_Visible_Armature_Operator_0Cdc9)
    bpy.utils.unregister_class(SNA_OT_Lazy_Bones_Dissolve_To_1_0D25F)
    bpy.utils.unregister_class(SNA_PT_BONE_DOCTOR_7679E)
    bpy.utils.unregister_class(SNA_OT_Bone_Target_Individual_E2333)
    bpy.utils.unregister_class(SNA_OT_Lazy_Bones_Hide_Bone_Axis_Operator_02F86)
    bpy.utils.unregister_class(SNA_OT_Bone_Target_Whole_817Da)
    bpy.utils.unregister_class(SNA_OT_Lazy_Bones_Bone_Roll_Global_Z_Operator_D986F)
    bpy.utils.unregister_class(SNA_OT_Lazy_Bones_Bone_Roll_Global_Y_Operator_50848)
    bpy.utils.unregister_class(SNA_OT_Lazy_Bones_Bone_Roll_Global_Z_Operator_2Ef8D)
    bpy.utils.unregister_class(SNA_OT_Lazy_Bones_Bone_Roll_Global_X_Operator_Eaa28)
    bpy.utils.unregister_class(SNA_OT_Lazy_Bones_Bone_Roll_Global_Y_Operator_96065)
    bpy.utils.unregister_class(SNA_OT_Lazy_Bones_Bone_Roll_Local_Z_Operator_Dc8D7)
    bpy.utils.unregister_class(SNA_OT_Lazy_Bones_Bone_Roll_Local_X_Operator_10200)
    bpy.utils.unregister_class(SNA_OT_Lazy_Bones_Bone_Roll_Local_X_Operator_D57Fd)
    bpy.utils.unregister_class(SNA_OT_Lazy_Bones_Bone_Roll_Global_X_Operator_168Ea)
    bpy.utils.unregister_class(SNA_OT_Lazy_Bones_Bone_Roll_Local_Z_Operator_C377A)
    bpy.utils.unregister_class(SNA_OT_Lazy_Bones_Show_Bone_Axis_Operator_026A7)
    bpy.utils.unregister_class(SNA_OT_Lazy_Bones_Dissolve_By_Fraction_Operator_Df537)
    bpy.utils.unregister_class(SNA_OT_Lazy_Bones_Switch_Bone_Direction_Operator_10514)
    bpy.utils.unregister_class(SNA_MT_414F8)
    bpy.utils.unregister_class(SNA_OT_Edge2Bone_2Cf72)
    bpy.types.VIEW3D_MT_edit_mesh_context_menu.remove(sna_add_to_view3d_mt_edit_mesh_context_menu_325A3)
    bpy.types.VIEW3D_MT_object_context_menu.remove(sna_add_to_view3d_mt_object_context_menu_5E1DB)
    bpy.utils.unregister_class(SNA_MT_ACB99)
    bpy.utils.unregister_class(SNA_OT_Lattice_2_Edges_Badcf)
    bpy.utils.unregister_class(SNA_OT_Lazy_Bones_Apply_Sim_Operator_9020B)
    bpy.utils.unregister_class(SNA_OT_Lazy_Bones_Cancel_Sim_Operator_32Fa6)
    bpy.utils.unregister_class(SNA_PT_MESH2BONE_SIM_4CCC3)
    bpy.utils.unregister_class(SNA_PT_SIM_CLEAN_A2455)
    bpy.utils.unregister_class(SNA_OT_Lazy_Bones_M2E_Apply_Modifiers_Operator_A686A)
    bpy.utils.unregister_class(SNA_AddonPreferences_06E76)
    bpy.types.VIEW3D_MT_object_context_menu.remove(sna_add_to_view3d_mt_object_context_menu_D45AB)
    bpy.utils.unregister_class(SNA_MT_B094C)
    bpy.utils.unregister_class(SNA_OT_Lazy_Bone_Mesh_2_Edges_Operator_65419)
    bpy.utils.unregister_class(SNA_OT_Symmetrize_World_Space_68Cc8)
    bpy.utils.unregister_class(SNA_OT_Delete_Half_World_Space_F888F)
